using System;
using Gtk;
using Foodolini.Activities;
using Foodolini.BusinessLogic;

namespace Foodolini.Activities.Administration
{
	[FoodoliniActivity("User administration", false, "Administration.admin.png")]
	[System.ComponentModel.ToolboxItem(true)]
	public partial class AdministrationActivity : Gtk.Bin, IActivity
	{
		/// <summary>
		/// Handles the UserAdministration: Adding and deleting users from the database.
		/// </summary>
		public AdministrationActivity()
		{
			
			this.Build();
			
			this.UserView.NodeStore = this.userList;
			
			this.UserView.AppendColumn("Name", new CellRendererText(), "text", 0);
			this.UserView.AppendColumn("User name", new CellRendererText(), "text", 1);
			
			
			foreach (var per in Person.ListUsers()){
				this.userList.AddNode(new PersonAdapter(per));	
			}
			
			this.UserView.NodeSelection.Mode = SelectionMode.Single;
			
			this.Shown += HandleShown;
		}
		
		/// <summary>
		/// Creates a new list of users.
		/// </summary>
		private NodeStore userList = new NodeStore(typeof(PersonAdapter));
		
		/// <summary>
		/// Hides the create person table per default.
		/// </summary>
		void HandleShown (object sender, EventArgs e){
		this.CreatePersonTable.Hide();
		this.Login();
		}
		
		/// <summary>
		/// Interface to main window
		/// </summary>
		private IOwner owner;
		
		/// <summary>
		/// Loads the activity Administration.
		/// </summary>
		public void Register(IOwner owner){
			this.owner = owner;
		}
		/// <summary>
		/// Login dialog and password validation called above in HandleShown
		/// </summary>
		private void Login(){
			var login = new LoginDialog(this.owner.Window);
			if(login.Run() != (int)ResponseType.Ok || login.Password != "hello")
				if(!this.owner.PopActivity())
					this.owner.HomeActivity();
			login.Destroy();
		}
		/// <summary>
		/// Unloads the administration activity.
		/// </summary>
		public void Unregister(){
			//this.owner.ActivityLoaded -= this.LoadActivity;
			this.owner = null;
		}
		/// <value>
		/// Returns the widget that represents the Administration Activity.
		/// </value>
		public Widget Widget{
			get{
				return this;
			}
		}
		/// <summary>
		/// Opens the create person table for the registration of person details.
		/// </summary>
		protected virtual void OnCreateNewUserClicked (object sender, System.EventArgs e){
			this.CreatePersonTable.Show();
			this.CreateNewUser.Sensitive = false;
		}
		/// <summary>
		/// Deletes a person from the userList and in the database.
		/// </summary>
		protected virtual void OnDeleteUserButtonClicked (object sender, System.EventArgs e){
			
			var per = (PersonAdapter)this.UserView.NodeSelection.SelectedNode;
			if (per != null){
				per.Person.Delete();
				this.userList.RemoveNode(per);
				this.owner.UpdateUserList();
				OnUserViewCursorChanged(this, new EventArgs());
			}
			
		}
		/// <summary>
		/// CancelsNewUserButton clicked hides CreatePersonTable, removes text from textboxes FullName and UserName.
		/// </summary>
		protected virtual void OnCancelNewUserButtonClicked (object sender, System.EventArgs e){
			this.CreatePersonTable.Hide();
			this.UserNameEntry.Text="";
			this.FullNameEntry.Text="";
			this.CreateNewUser.Sensitive = true;
		}
		/// <summary>
		/// SaveNewUserButton clicked hides CreatePersonTable, removes text from textboxes FullName and UserName.
		/// </summary>
		protected virtual void OnSaveNewUserButtonClicked (object sender, System.EventArgs e){
			
			Person per = new Person(UserNameEntry.Text ,FullNameEntry.Text, BirthDayCalendar.Date, (Gender)GenderCombobox.Active, HeightSpinbutton.Value, WeightSpinbutton.Value, 1.2);  
			per.Save();
			
			this.userList.AddNode(new PersonAdapter(per));
			this.CreatePersonTable.Hide();
			this.UserNameEntry.Text="";
			this.FullNameEntry.Text="";
			this.CreateNewUser.Sensitive = true;
			this.owner.UpdateUserList();
		}
		/// <summary>
		///	DeleteUserButton is not activated before a user is marked in the userList.
		/// </summary>
		protected virtual void OnUserViewCursorChanged (object sender, System.EventArgs e){
			if (this.UserView.NodeSelection.SelectedNode!=null){
				this.DeleteUserButton.Sensitive = true;
			}
			
			else{ 
				this.DeleteUserButton.Sensitive = false;	
			}
			
		}
		/// <summary>
		/// SaveNewUserButton is not activated before text is entered in both text boxes for Fullname and UserName.
		/// </summary>
		protected virtual void ValidateEntry (object sender, System.EventArgs e)
		{
			if (this.FullNameEntry.Text == "" || this.UserNameEntry.Text == ""){
				this.SaveNewUserButton.Sensitive = false;
			}
			else{
				this.SaveNewUserButton.Sensitive = true;
			}
		}
	}
}
