
using System;
using System.Collections.Generic;
using Foodolini.Activities;
using Foodolini.BusinessLogic;
using Gtk;

namespace Foodolini.Activities.Cookbook
{
	
	[FoodoliniActivity("Cookbook", false, "book.png")]
	[System.ComponentModel.ToolboxItem(true)]
	public partial class CookbookActivity : Gtk.Bin, IActivity
	{
		private IOwner owner;
		private NodeStore recipeStorage = new NodeStore(typeof(RecipeAdapter));
		private RecipeViewer recipeViewer = null;
		
		public CookbookActivity()
		{
			this.Build();
			
			//Set up nodeview
			this.nvRecipes.NodeStore = this.recipeStorage;
			this.nvRecipes.AppendColumn("Recipes", new CellRendererText(), delegate(TreeViewColumn tree_column, CellRenderer cell, ITreeNode node){
				RecipeAdapter n = (RecipeAdapter)node;
				CellRendererText c = (CellRendererText)cell;
				c.Text = n.Title;
			});
			this.nvRecipes.AppendColumn("Meal type", new CellRendererText(), delegate(TreeViewColumn tree_column, CellRenderer cell, ITreeNode node){
				RecipeAdapter n = (RecipeAdapter)node;
				CellRendererText c = (CellRendererText)cell;
				c.Text = n.Type;
			});
			this.nvRecipes.AppendColumn("Rating", new CellRendererText(), delegate(TreeViewColumn tree_column, CellRenderer cell, ITreeNode node){
				RecipeAdapter n = (RecipeAdapter)node;
				CellRendererText c = (CellRendererText)cell;
				c.Text = n.Rating.ToString("0.00");
			});
			bxSearchRecipe.ShowAll();
			
			//Set Advanced criteria to hide
			expAdvancedCriteria.Expanded = false;
			scExpirationDate.Sensitive = false;
			scRating.Sensitive = false;
				
		}
	
		/// <summary>
		/// Searches in the database for Recipes that fit the chosen criteria.
		/// </summary>
		protected virtual void OnBtnSearchClicked (object sender, System.EventArgs e)
		{
			SearchRecipes();
		}
		
		/// <summary>
		/// Searches in the database for Recipes that fit the chosen criteria.
		/// </summary>
		protected virtual void OnEntSearchRecipesActivated (object sender, System.EventArgs e)
		{
			SearchRecipes();
		}
		
		/// <summary>
		/// Searches in the database for Recipes that fit the chosen criteria.
		/// </summary>
		private void SearchRecipes()
		{
			this.recipeStorage.Clear();
			Criteria criteria = new Criteria();
			List<string> searchString = new List<string>();
			
			//Get keywords for search
			foreach(string keyword in entSearchRecipes.Text.Split(new char[] {' '}, StringSplitOptions.RemoveEmptyEntries))
				if(keyword.Length > 2)
					searchString.Add(keyword);
			criteria.searchString = searchString;
			List<Recipe> recipes = new List<Recipe>();
			
			//Get meal type	
			criteria.Meal = Meal.None;
			if(cbBreakfast.Active)
				criteria.Meal |= Meal.Breakfast;
			if(cbLunch.Active)
				criteria.Meal |= Meal.Lunch;
			if(cbDinner.Active)
				criteria.Meal |= Meal.Dinner;
			if(cbOther.Active)
				criteria.Meal |= Meal.Other;
			
			//Check if we are searching after ratings
			if(cbRating.Active)
			{
				criteria.rating = true;
				criteria.ratingPriority = (int)scRating.Value;
			}
			
			//Check if we are searching after expiration date
			if(cbExpirateDate.Active)
			{
				criteria.expirationDate = true;
				criteria.expirationDatePriority = (int)scExpirationDate.Value;
			}
			
			recipes = Recipe.ListByCriteria(criteria);
			
			if(recipes.Count != 0) {
				foreach(Recipe recipe in recipes)
					this.recipeStorage.AddNode(new RecipeAdapter(recipe));
				this.nvRecipes.CursorChanged += ViewRecipe;
				this.nvRecipes.RowActivated += EditRecipe;
			}else {
				this.recipeStorage.AddNode(new RecipeAdapter(new Recipe("No recipes found")));
				this.nvRecipes.CursorChanged -= ViewRecipe;
				this.nvRecipes.RowActivated -= EditRecipe;
			}
		}
		
		

		/// <summary>
		/// Sets the sensitivity on th scalebuttons i the advanced search criteria to 
		/// true or false whether Expiration date and/or rating is selected as a search criteria
		/// </summary>
		protected virtual void SearchCriteriaChanged (object sender, System.EventArgs e)
		{
			scExpirationDate.Sensitive = cbExpirateDate.Active;
			scRating.Sensitive = cbRating.Active;
		}
		
		/// <summary>
		/// Searches the cookbook again when a change has been made to a recipe.
		/// </summary>
		void HandleOnRecipeViewed(object sender, RecipeEventArgs e)
		{
			SearchRecipes();//TODO: Maybe stupid to update like this
		}
		
		/// <summary>
		/// Adds a recipe to the search results when created a new one.
		/// </summary>
		void HandleOnRecipeCreated(object sender, RecipeEventArgs e)
		{
			this.recipeStorage.AddNode(new RecipeAdapter(e.Recipe)); //TODO: Maybe stupid to update like this
		}
	
		#region New activities
		/// <summary>
		/// Views the selected Recipe in a RecipeViewer to the right.
		/// If one is already viewn this is destroyed.
		/// </summary>
		protected virtual void ViewRecipe (object sender, System.EventArgs e)
		{	
			if(nvRecipes.NodeSelection.SelectedNode != null){
				if(this.recipeViewer != null)
				{
					this.recipeViewer.OnRecipeViewed -= HandleOnRecipeViewed;
					this.recipeViewer.OnRecipeEdited -= HandleOnRecipeViewed;
					this.recipeViewer.OnRecipeDeleted -= HandleOnRecipeViewed;
					this.recipeViewer.Unregister();
					this.recipeViewer.Destroy();
				}
				
				this.recipeViewer = new RecipeViewer();
				this.recipeViewer.Register(this.owner);
				this.recipeViewer.SetParamenters(((RecipeAdapter)nvRecipes.NodeSelection.SelectedNode).Recipe);
				bxCookbook.Add(this.recipeViewer);
				
				this.recipeViewer.OnRecipeViewed += HandleOnRecipeViewed;
				this.recipeViewer.OnRecipeEdited += HandleOnRecipeViewed;
				this.recipeViewer.OnRecipeDeleted += HandleOnRecipeViewed;
				
				bxCookbook.ShowAll();
			}
		}

		/// <summary>
		/// Opens the "Create recipe" activity where a new recipe can be created.
		/// </summary>
		protected virtual void OnCreateRecipeClick (object sender, System.EventArgs e)
		{
			RecipeCreater rc = this.owner.PushActivity<RecipeCreater>();
			rc.OnRecipeCreated += HandleOnRecipeCreated;
		}
		
		/// <summary>
		/// Opens the "Edit recipe" activity wher an existing recipe can be editted.
		/// </summary>
		protected virtual void EditRecipe (object o, Gtk.RowActivatedArgs args)
		{
			RecipeEditor re = this.owner.PushActivity<RecipeEditor>();
			re.SetParamenters(((RecipeAdapter)this.nvRecipes.NodeSelection.SelectedNode).Recipe);
			re.OnRecipeEdited += HandleOnRecipeViewed;
		}

		#endregion
		
		#region IActivity

		public void Register(IOwner owner){
			this.owner = owner;
		}
		
		public void Unregister(){
			this.owner = null;			
		}


		public Widget Widget{
			get{
				return this;
			}
		}
		#endregion
		
	}
}
