
using System;
using Foodolini.Activities;
using Foodolini.BusinessLogic;
using Gtk;
using System.Collections.Generic;
using System.IO;
using Foodolini.Activities.Ingredients;

namespace Foodolini.Activities.Cookbook
{
	/// <summary>
	/// An activity where an existing recipe can be edited.
	/// </summary>
	[FoodoliniActivity("Edit recipe", true)]
	[System.ComponentModel.ToolboxItem(true)]
	public partial class RecipeEditor : Gtk.Bin, IActivity
	{
		private IOwner owner;
		private Recipe recipe;
		private NodeStore ingredientStorage = new NodeStore(typeof(IngredientAdapter));
		private IList<string> listCategory = new List<string>();
		private List<string> listDirections = new List<string>();
		
		/// <summary>
		/// A constructor that sets up the nodeview.
		/// </summary>
		public RecipeEditor()
		{
			this.Build();	
			
			//Set up nodeview.
			this.nvIngredients.NodeStore = this.ingredientStorage;
			this.nvIngredients.AppendColumn("Added ingredients", new CellRendererText(), (tree_column, cell, node)=>{
				IngredientAdapter n = (IngredientAdapter)node;
				CellRendererText c = (CellRendererText)cell;
				c.Text = n.Name;
			});
		
			CellRendererSpin quantityCell = new CellRendererSpin();
			quantityCell.Editable = true;
			quantityCell.Adjustment = new Adjustment(0,1,100000,1,1,1);
			quantityCell.Digits = 1;
			this.nvIngredients.AppendColumn("Quantity (g)", quantityCell, (column, cell, node) =>
            {
                ((CellRendererSpin)cell).Text = ((IngredientAdapter)node).Quantity.ToString ("0");
            });
			
			quantityCell.Edited += delegate(object o, EditedArgs args) {
				var node = (IngredientAdapter)ingredientStorage.GetNode(new TreePath(args.Path));
				double newValue;
				if(double.TryParse(args.NewText, out newValue)) {
					node.Quantity = newValue;
				}
			};
			
			//HACK: set spinbutton to actual value because of rounding error.
			quantityCell.EditingStarted += (o, args) =>
			{
				var node = (IngredientAdapter)ingredientStorage.GetNode (new TreePath (args.Path));
				((SpinButton)args.Editable).Value = node.Quantity;
			};
			
			bxIngredient.ShowAll();
		}
		
		public event EventHandler<RecipeEventArgs> OnRecipeEdited;

		/// <summary>
		/// Checks if a Title, at least one categori and ingredient has been given.
		/// </summary>
		private void CheckIfFilled()
		{
			if (this.entTitle.Text == "" | !ingredientsAdded | tvDirections.Buffer.Text == "")
				btnSave.Sensitive = false;
			else
				btnSave.Sensitive = true;
		}
		
		private bool ingredientsAdded = false;
		
		/// <summary>
		/// Opens a dialog where a new ingredient and the amount can be selected and added to the Recipe opon saving
		/// </summary>
		protected virtual void OnAddIngredient (object sender, System.EventArgs e)
		{
			Ingredient selectedIngredient;
			IngredientSelector dialog = new IngredientSelector(true);
			if (dialog.Run() == (int)ResponseType.Ok)
			{
				selectedIngredient = dialog.SelectedIngredient;
				this.ingredientStorage.AddNode(new IngredientAdapter(selectedIngredient, dialog.Quantity));
			}
			dialog.Destroy();
			ingredientsAdded = true;
			CheckIfFilled();
		}
		
		protected virtual void OnBtnNewSaveClicked (object sender, System.EventArgs e)
		{
			string title = entTitle.Text;
			if(Recipe.GetByTitle(title) == null & Ingredient.GetByLongDescription(title) == null)
			{
				SaveNewRecipe();
			}
			else
			{
				if(title == recipe.Title)
					SaveNewRecipe();
				else
				{
					ConfirmDialog dialog = new ConfirmDialog("Recipe or ingredient allready exist", "A recipe or ingredient with that name allready exist.\nTry to remane the recipe.");
						if(dialog.Run() == (int)ResponseType.Ok){}
					dialog.Destroy();
				}
			}
		}
		
		/// <summary>
		/// Pops the activity when Cancel is clicked
		/// </summary>
		protected virtual void OnCancelClick (object sender, System.EventArgs e)
		{
			this.owner.PopActivity();
		}
		
		/// <summary>
		/// Removes the selected ingredient form the list. 
		/// If no ingredients are left, the variable ingredientsAdded is set to false. 
		/// CheckIfFilled() is called at the end.
		/// </summary>
		protected virtual void OnDeleteIngredientClick (object sender, System.EventArgs e)
		{
			if(this.nvIngredients.NodeSelection.SelectedNode != null)
			{
				var ingredient = (IngredientAdapter)this.nvIngredients.NodeSelection.SelectedNode;
				this.ingredientStorage.RemoveNode(ingredient);
				if(!this.ingredientStorage.GetEnumerator().MoveNext())
					this.ingredientsAdded = false;
				CheckIfFilled();
			}
		}
		
		/// <summary>
		/// Event called when an entry has been changed. Calls CheckIfFilled()
		/// </summary>
		protected virtual void OnEntryChanged (object sender, System.EventArgs e)
		{
			CheckIfFilled();
		}

		/// <summary>
		/// If no ingredients or recipes has the same name except the current recipe, the recipe is saved in the database.
		/// If there is, a dialog warns the user about this, and askes to change the title.
		/// Button is only active when the title, the categories, the directions and Ingrediens have been set
		/// </summary>
		protected virtual void OnSaveClicked (object sender, System.EventArgs e)
		{
			string title = entTitle.Text;
			if(Recipe.GetByTitle(title) == null & Ingredient.GetByLongDescription(title) == null)
			{
				SaveRecipe();
			}
			else
			{
				if(title == recipe.Title)
					SaveRecipe();
				else
				{
					ConfirmDialog dialog = new ConfirmDialog("Recipe or ingredient allready exist", "A recipe or ingredient with that name allready exist.\nTry to remane the recipe.");
						if(dialog.Run() == (int)ResponseType.Ok){}
					dialog.Destroy();
				}
			}
		}
		
		private void SaveNewRecipe()
		{
			Recipe newRecipe = new Recipe(entTitle.Text);
			
			if(cbxMealtype.Active == 0)
				listCategory.Add("Breakfast");
			else if(cbxMealtype.Active == 1)
				listCategory.Add("Lunch");
			else if (cbxMealtype.Active == 2)					
				listCategory.Add("Dinner");
			else
				listCategory.Add("Other");
			
			newRecipe.Difficulty = (Difficulty)cbxDifficulty.Active;				
				
			newRecipe.PreparationTime = new TimeSpan((int)sbDays.Value, (int)sbHours.Value, (int)sbMinuts.Value, 0);
		
			//Saves the categories
			foreach(string category in this.entCategories.Text.Split(new char[]{','}))
				this.listCategory.Add(category.Trim());
			newRecipe.Categories = listCategory;
						
			//Saves the directions
			foreach(string direction in this.tvDirections.Buffer.Text.Split(new char[]{'\n'}, StringSplitOptions.RemoveEmptyEntries))
				this.listDirections.Add(direction.Trim());
			newRecipe.Directions = listDirections;
			
			//Saves the ingredients
			foreach(IngredientAdapter ingredient in this.ingredientStorage)
				newRecipe.Ingredients.Add(ingredient.Ingredient, ingredient.Quantity);
			
			//Saves the picture is set.
			if (byteArray != null)
				newRecipe.Picture = byteArray;
			
			newRecipe.Save();

			if(OnRecipeEdited != null)
				OnRecipeEdited(this, new RecipeEventArgs(this.recipe));
			this.owner.PopActivity();
		}
		
		
		/// <summary>
		/// Saves the Recipe in the database
		/// Button is only active when the title, the categories, the directions and Ingrediens have been set
		/// </summary>
		private void SaveRecipe()
		{
			this.recipe.Title = entTitle.Text;
			
			//Saves the meal type
			if(cbxMealtype.Active == 0)
				listCategory.Add("Breakfast");
			else if(cbxMealtype.Active == 1)
				listCategory.Add("Lunch");
			else if (cbxMealtype.Active == 2)					
				listCategory.Add("Dinner");
			else
				listCategory.Add("Other");
			
			this.recipe.Difficulty = (Difficulty)cbxDifficulty.Active;				
				
			this.recipe.PreparationTime = new TimeSpan((int)sbDays.Value, (int)sbHours.Value, (int)sbMinuts.Value, 0);
		
			//Saves the categories
			foreach(string category in this.entCategories.Text.Split(new char[]{','}))
				this.listCategory.Add(category.Trim());
			this.recipe.Categories = listCategory;
			
			//Saves the number of servings
			this.recipe.Servings = sbMultiplier.Value;
						
			//Saves the directions
			foreach(string direction in this.tvDirections.Buffer.Text.Split(new char[]{'\n'}, StringSplitOptions.RemoveEmptyEntries))
				this.listDirections.Add(direction.Trim());
			this.recipe.Directions = listDirections;
			
			//Saves the ingredients
			this.recipe.ClearIngredients();
			foreach(IngredientAdapter ingredient in this.ingredientStorage)
				this.recipe.Ingredients.Add(ingredient.Ingredient, ingredient.Quantity);
			
			//Saves the picture if set
			if (byteArray != null)
				this.recipe.Picture = byteArray;
			
			this.recipe.Save();

			if(OnRecipeEdited != null)
				OnRecipeEdited(this, new RecipeEventArgs(this.recipe));
			this.owner.PopActivity();
		}
			
		private FileStream stream;
		private byte[] byteArray;
		
		/// <summary>
		/// Previews the Recipe image when selected in the filechooserwidget
		/// </summary>
		protected virtual void OnSelectPictureClick (object sender, System.EventArgs e)
		{
			stream = File.OpenRead(fcbPicture.Filename);
			byteArray = new byte[stream.Length];
			for(int i = 0; i < stream.Length; i++)
				byteArray[i] = (byte)stream.ReadByte();
			Gdk.Pixbuf px = new Gdk.Pixbuf(byteArray);
			px = px.ScaleSimple(100, 100, Gdk.InterpType.Bilinear);//TODO: Make prettier
			imgRecipe.Pixbuf = px;
		}
		
		/// <summary>
		/// Event called when directions have changed. Calls CheckIfFilled().
		/// </summary>
		protected virtual void OnTvDirectionsKeyReleaseEvent (object o, Gtk.KeyReleaseEventArgs args)
		{
			CheckIfFilled();
		}
		
		/// <summary>
		/// Method called to set the recipe to be editted. Sets the details of the recipe in the entries
		/// </summary>
		/// <param name="recipe">
		/// A <see cref="Recipe"/>
		/// </param>
		public void SetParamenters(Recipe recipe){
			this.recipe = recipe;
			SetRecipeDetails();
		}
		
		/// <summary>
		/// Sets the details of the recipe.
		/// </summary>
		private void SetRecipeDetails(){
			entTitle.Text = this.recipe.Title;
			
			//Adds all categories to the entry, separated by a comma.
			for(int i = 0; i < this.recipe.Categories.Count-1; i++)
			{
				string category = this.recipe.Categories[i];
				if(category == "Breakfast")
					this.cbxMealtype.Active = 0;
				else if (category == "Lunch")
					this.cbxMealtype.Active = 1;
				else if (category == "Dinner")
					this.cbxMealtype.Active = 2;
				else if (category == "Other")
					this.cbxMealtype.Active = 3;
				else
					this.entCategories.Text += category + ", ";
			}
			this.entCategories.Text += this.recipe.Categories[this.recipe.Categories.Count-1];
			
			//Adds dificulty, preparation time and number of servings.
			cbxDifficulty.Active = (int)this.recipe.Difficulty;
			TimeSpan preparationTime = this.recipe.PreparationTime;
			sbDays.Value = preparationTime.Days;
			sbHours.Value = preparationTime.Hours;
			sbMinuts.Value = preparationTime.Minutes;
			sbMultiplier.Value = this.recipe.Servings;
			
			//Adds all steps separated by a new line
			foreach (var direction in this.recipe.Directions)
				this.tvDirections.Buffer.Text += direction + "\n";
			
			//Add all ingredients to the nodeview
			foreach(KeyValuePair<Ingredient, double> ing in this.recipe.Ingredients)
			{
				this.ingredientStorage.AddNode(new IngredientAdapter(ing.Key, ing.Value));
				this.ingredientsAdded = true;
			}
			
			//If the recipe has a picture this is viewed.
			if(this.recipe.Picture != null)
			{
				Gdk.Pixbuf px = new Gdk.Pixbuf(this.recipe.Picture);
				px = px.ScaleSimple(100, 100, Gdk.InterpType.Bilinear);//TODO: Make prettier
				this.imgRecipe.Pixbuf = px;
			}
				
			CheckIfFilled();
		}

		public Recipe Recipe{
			get{return this.recipe;}
		}
		
		#region IActivity
		public void Register(IOwner owner){
			this.owner = owner;
		}
		
		public void Unregister(){
			this.owner = null;
		}


		
		public Widget Widget{
			get{
				return this;
			}
		}
		#endregion	
	}
	

}

