using System;
using System.Collections.Generic;

/// <summary>
/// Classes to query available hardware
/// </summary>
namespace Foodolini.Activities.FoodRegistration.Devices
{
	/// <summary>
	/// Interface for a class that can monitor video capture devices on the system
	/// </summary>
	/// <remarks>When altering this change it to be an abstract class to avoid breaking backwards compatibility in the future.</remarks>
	public interface ICaptureDeviceMonitor : IDisposable
	{
		/// <value>
		/// List potential video sources on the system
		/// </value>
		ICollection<IDeviceInfo> Devices{get;}
		
		/// <summary>
		/// Occurs whenever a device added to the system
		/// </summary>
		event EventHandler<DeviceEventArgs> DeviceAdded;
		
		/// <summary>
		/// Occurs whenever a device removed from the system
		/// </summary>
		event EventHandler<DeviceEventArgs> DeviceRemoved;
		
		/// <value>
		/// Returns true, if DeviceAdded and DeviceRemoved events can be relied upon to provide device status updates
		/// </value>
		/// <remarks>
		/// If this is false, the UI should offer a refresh button that synchronizes the UI to the Devices property.
		/// Note, that this does not imply that DeviceAdded and DeviceRemoved may not be used. However, merly they may
		/// not be relied upon.
		/// </remarks>
		bool SupportsAutoRefresh{get;}
	}
	
	/// <summary>
	/// Information on a video source
	/// </summary>
	public interface IDeviceInfo{ 
		//Here I'm missing multiple inheritance, but this is okay
		// - yes, it could also be solved with composition, but it's not as pretty!
		
		/// <summary>
		/// Human readable name
		/// </summary>
		string Name{get;}
		
		/// <summary>
		/// Device file
		/// </summary>
		string DevicePath{get;}
	}
	
	/// <summary>
	/// Event arguments that represent a video device
	/// </summary>
	public sealed class DeviceEventArgs : EventArgs, IDeviceInfo{
		/// <value>
		/// Human readable name
		/// </value>
		/// <remarks>This may not always be available.</remarks>
		public string Name{get; private set;}
		
		/// <value>
		/// Device file
		/// </value>
		public string DevicePath{get; private set;}
		
		/// <summary>
		/// Create a new instance of DeviceEventArgs
		/// </summary>
		internal DeviceEventArgs(string name, string Device){
			this.Name = name;
			this.DevicePath = Device;
		}
	}
}
