using Foodolini.BusinessLogic;
using System;
using Gtk;
using System.Collections.Generic;
using Foodolini.Activities.FoodRegistration;
using Foodolini.Activities.FoodRegistration.Devices;
using ZBar;
using Foodolini.Activities.Ingredients;
  

namespace Foodolini.Activities.FoodRegistration
{
	/// <summary>
	/// create a new FoodregistrationActivity
	/// open the camera
	/// creates the treeview for scanned fooditems
	/// sets the treeview to single select
	/// </summary>
	[FoodoliniActivity("Food registration", false, "FoodRegistration.cart.png")]
	[System.ComponentModel.ToolboxItem(true)]
	public partial class FoodRegistrationActivity : Gtk.Bin, IActivity
	{
		
		private NodeStore foodList = new NodeStore(typeof(FoodItemAdapter));
		private IOwner owner;
		
		public FoodRegistrationActivity(){
			//Build the designed UI
			this.Build();
			
			//Widget events
			this.Destroyed += HandleDestroyed;
			this.Hidden += HandleHidden; 
			this.Shown += HandleShown;
			
			//Scanner events
			this.scanner.Stopped += HandleStopped;
			this.scanner.Error += HandleError;
			
			//Load the flipped setting
			bool flipped = Foodolini.BusinessLogic.Settings.Instance.GetValue("FlipVideo", "True") == "True";
			this.scanner.Flip = flipped;
			this.FlipButton.Active = flipped;
			
			//Populate the VideoSourceComboBox
			this.PopulateVideoSourceComboBox();
			
			//Load the mute setting
			this.scanner.Mute = Foodolini.BusinessLogic.Settings.Instance.GetValue("MuteScanner", "False") == "True";
			
			//Update the mute button
			this.UpdateMuteButton();
			
			this.FoodView.NodeStore = this.foodList;
			this.FoodView.AppendColumn("Description", new CellRendererText(), "text", 0);
			this.FoodView.AppendColumn("Quantity", new CellRendererText(), delegate(TreeViewColumn tree_column, CellRenderer cell, ITreeNode node){
				FoodItemAdapter n = (FoodItemAdapter)node;
				CellRendererText c = (CellRendererText)cell;
				c.Text = n.Quantity.ToString();
			});
			this.FoodView.AppendColumn("Expiration date", new CellRendererText(), delegate(TreeViewColumn tree_column, CellRenderer cell, ITreeNode node){
				FoodItemAdapter n = (FoodItemAdapter)node;
				CellRendererText c = (CellRendererText)cell;
				c.Text = n.ExpirationDate.ToLongDateString();
			});

			this.FoodView.NodeSelection.Mode = SelectionMode.Single;
			
			this.SelectIngredientButton.Sensitive = false;
			this.QuantitySpinbutton.Sensitive = false;
			this.DuplicateButton.Sensitive = false;
			this.RemoveButton.Sensitive = false;
			this.SaveButton.Sensitive = false;
				
		}
		
		void HandleHidden(object sender, EventArgs e){
			this.scanner.Close();
		}

		void HandleDestroyed(object sender, EventArgs e){
			//Save the flip setting
			Foodolini.BusinessLogic.Settings.Instance["FlipVideo"] = this.scanner.Flip.ToString();
			//Save the mute setting
			Foodolini.BusinessLogic.Settings.Instance["MuteScanner"] = this.scanner.Mute.ToString();
			//Ensure that settings are saved, e.g. commit to database
			Foodolini.BusinessLogic.Settings.Instance.Save();
			//Destroy widgets
			this.scanner.Destroy();
			this.muteImage.Destroy();
			this.unmuteImage.Destroy();
			//Stop listening for new devices
		 	CaptureDeviceMonitor.Instance.Dispose();
		}
		
		void HandleShown (object sender, EventArgs e){
			if(currentDevice != null)
			   this.scanner.Open(currentDevice);
			
			//Check if we should offer the refresh button
			if(CaptureDeviceMonitor.Instance.SupportsAutoRefresh)
				this.RefreshButton.Hide();
		}
		
		#region Scanner control

		private string currentDevice = null;
		
		/// <summary>
		/// Handles Stopped event, if exception occurs in Scanner
		/// </summary>
		void HandleStopped(object sender, EventArgs e){
			//Deselect in VideoSourceComboBox
			this.VideoSourceComboBox.SetActiveIter(this.NoVideoIter);
		}
		
		/// <summary>
		/// Displays hardware related errors that the occurs in the scanner
		/// </summary>
		void HandleError(object sender, ErrorEventArgs e){
			//Display error
			Console.WriteLine("ZBar error i: " + e.Message);
			//Deselect in VideoSourceComboBox
			this.VideoSourceComboBox.SetActiveIter(this.NoVideoIter);
		}
		
		private ListStore VideoSources = new ListStore(typeof(string), typeof(string));
		private TreeIter NoVideoIter;
		private void PopulateVideoSourceComboBox(){
			//Add all entries
			this.VideoSourceComboBox.Model = this.VideoSources;
			this.NoVideoIter = this.VideoSources.AppendValues("No video", null);
			Nullable<TreeIter> source = null;
			foreach(var device in CaptureDeviceMonitor.Instance.Devices){
				var iter = this.VideoSources.AppendValues(device.Name, device.DevicePath);
				if(source == null)
					source = iter;
			}
			//Open the first video entry, don't user OnVideoSourceComboBoxChanged
			if(source != null){
				this.IgnoreVideoSourceComboBoxChanged = true;
				this.VideoSourceComboBox.SetActiveIter(source.Value);
				this.IgnoreVideoSourceComboBoxChanged = false;
				var device = (string)this.VideoSources.GetValue(source.Value, 1);
				this.currentDevice = device;
			}else
				this.VideoSourceComboBox.SetActiveIter(this.NoVideoIter);
			
			//Listen for new devices
			CaptureDeviceMonitor.Instance.DeviceAdded += HandleDeviceAdded;
			CaptureDeviceMonitor.Instance.DeviceRemoved += HandleDeviceRemoved;
		}
		
		protected virtual void OnRefreshButtonClicked(object sender, System.EventArgs e){
			//Devices currently in the combobox
			List<string> listedDevices = new List<string>();
			//Devices currently available on the system
			ICollection<IDeviceInfo> devs = CaptureDeviceMonitor.Instance.Devices;
			//If active device is to be removed, change to NoVideo
			TreeIter iter;
			if(this.VideoSourceComboBox.GetActiveIter(out iter)){
				string name = (string)this.VideoSources.GetValue(iter, 0);
				string devpath = (string)this.VideoSources.GetValue(iter, 1);
				//Check if the device exists
				bool exists = false;
				foreach(var dev in devs){
					if(name != null && name == dev.Name &&
					   devpath != null && devpath == dev.DevicePath)
						exists = true;
				}
				if(!exists)
					this.VideoSourceComboBox.SetActiveIter(this.NoVideoIter);
			}
			//Remove all devices that doesn't exists, and add all that does to listedDevices
			TreeIter i;
			bool hasNext = this.VideoSources.GetIterFirst(out i);
			while(hasNext){
				string name = (string)this.VideoSources.GetValue(i, 0);
				string devpath = (string)this.VideoSources.GetValue(i, 1);
				//Check if the device exists
				bool exists = false;
				foreach(var dev in devs){
					if(name != null && name == dev.Name &&
					   devpath != null && devpath == dev.DevicePath)
						exists = true;
				}
				//Add it to list of stuff in the combobox
				if(devpath != null && !listedDevices.Contains(devpath))
					listedDevices.Add(devpath);
				//Remove it if it doesn't
				if(!exists){
					this.VideoSources.Remove(ref i);
					//Can't go on from i, so just start over... the list can't be long anyway...
					hasNext = this.VideoSources.GetIterFirst(out i);
				}else	//Go to the next iter
					hasNext = this.VideoSources.IterNext(ref i);
			}
			foreach(var dev in devs){
				if(!listedDevices.Contains(dev.DevicePath)){
					//Add new video source
					TreeIter device = this.VideoSources.AppendValues(dev.Name, dev.DevicePath);
					//Select if currrently NoVideoIter
					if(this.VideoSourceComboBox.GetActiveIter(out iter)){
						string devpath = (string)this.VideoSources.GetValue(iter, 1);
						//If null No Video is selected
						if(devpath == null) //Then set the new device active
							this.VideoSourceComboBox.SetActiveIter(device);
					}
				}
			}
		}

		/// <summary>
		/// Handle new device found
		/// </summary>
		void HandleDeviceAdded(object sender, DeviceEventArgs e){
			//Add new video source
			TreeIter device = this.VideoSources.AppendValues(e.Name, e.DevicePath);
			//Select if currrently NoVideoIter
			TreeIter iter;
			if(this.VideoSourceComboBox.GetActiveIter(out iter)){
				string dev = (string)this.VideoSources.GetValue(iter, 1);
				//If null No Video is selected
				if(dev == null) //Then set the new device active
					this.VideoSourceComboBox.SetActiveIter(device);
			}
		}
		
		/// <summary>
		/// Handle device removed
		/// </summary>
		void HandleDeviceRemoved(object sender, DeviceEventArgs e){
			//Change active if the one we shall remove is active
			TreeIter iter;
			if(this.VideoSourceComboBox.GetActiveIter(out iter)){
				string name = (string)this.VideoSources.GetValue(iter, 0);
				string devpath = (string)this.VideoSources.GetValue(iter, 1);
				if(name != null && name == e.Name &&
				   devpath != null && devpath == e.DevicePath)
					this.VideoSourceComboBox.SetActiveIter(this.NoVideoIter);
			}
			//Remove all devices that match
			TreeIter i;
			bool hasNext = this.VideoSources.GetIterFirst(out i);
			while(hasNext){
				string name = (string)this.VideoSources.GetValue(i, 0);
				string devpath = (string)this.VideoSources.GetValue(i, 1);
				
				if(name != null && name == e.Name &&
				   devpath != null && devpath == e.DevicePath){
					this.VideoSources.Remove(ref i);
					//Can't go on from i, so just start over... the list can't be long anyway...
					hasNext = this.VideoSources.GetIterFirst(out i);
				}else	//Go to the next iter
					hasNext = this.VideoSources.IterNext(ref i);
			}
		}
		
		private bool IgnoreVideoSourceComboBoxChanged = false;
		private void OnVideoSourceComboBoxChanged (object sender, System.EventArgs e){
			if(this.IgnoreVideoSourceComboBoxChanged)
				return;
			//Open the selected device
			TreeIter iter;
			if(this.VideoSourceComboBox.GetActiveIter(out iter)){
				string device = (string)this.VideoSources.GetValue(iter, 1);
				if(device != null){
					this.currentDevice = device;
					this.scanner.Open(device);
				}else{
					this.currentDevice = null;
					this.scanner.Close();
				}
			}else{
				this.currentDevice = null;
				this.scanner.Close();
			}
		}
			
		private void OnMuteButtonClicked (object sender, System.EventArgs e){
			this.scanner.Mute = !this.scanner.Mute;
			this.UpdateMuteButton();
		}
		
		private Gtk.Image muteImage = new Gtk.Image(null, "FoodRegistration.muted.png");
		private Gtk.Image unmuteImage = new Gtk.Image(null, "FoodRegistration.audio.png");
		
		/// <summary>
		/// Update the image for the mute button
		/// </summary>
		private void UpdateMuteButton(){
			if(this.scanner.Mute)
				this.MuteButton.Image = this.muteImage;
			else
				this.MuteButton.Image = this.unmuteImage;
		}
		
		/// <summary>
		/// Flip the image on/off
		/// </summary>
		protected virtual void OnFlipButtonToggled (object sender, System.EventArgs e){
			this.scanner.Flip = this.FlipButton.Active;
		}
		
		#endregion
		
		#region IActivity implementation
		
		public Widget Widget{
			get{return this;}
		}
		
		public void Register(IOwner owner){
			this.owner = owner;
			this.owner.ActivityUnloading += OwnerhandleActivityUnloading;
		}
		
		public void Unregister(){
			this.owner.ActivityUnloading -= OwnerhandleActivityUnloading;
			this.owner = null;
		}
		
		#endregion
		
		#region	Save, cancel, unloading
		
		private bool cancelled = true;
		
		/// <summary>
		///  save the fooditem in the treeview when the user clicks save
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		void OnSaveButtonClicked (object sender, System.EventArgs e){
			this.cancelled = true;
			foreach (var node in this.foodList){
				FoodItemAdapter food = (FoodItemAdapter)node;
				food.FoodItem.Save();
			}
			this.foodList.Clear();
			Foodolini.BusinessLogic.Settings.Instance.Save();
			this.owner.PopActivity();
			  
		}
				 
		/// <summary>
		/// returns the user to previousactivity
		/// <remarks 
		/// activate the confirmation dialog in the unloadevent if the user have unsaved items
		/// </remarks>
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		void OnCancelButtonClicked (object sender, System.EventArgs e){
			this.cancelled = !this.foodList.GetEnumerator().MoveNext();
			this.owner.PopActivity();
		}
		
		/// <summary>
		/// if activated by a canclled event promts the user for confirmation before leaveing
		/// and allow the user to call the SaveButton.Click first.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="ActivityUnloadedArgs"/>
		/// </param>
		void OwnerhandleActivityUnloading (object sender, ActivityUnloadedArgs e){
			if(!this.cancelled && e.Activity == this ){
				SaveDialog dlg = new SaveDialog();
				var result = (Gtk.ResponseType)dlg.Run();
				if(result == Gtk.ResponseType.Apply)
					this.SaveButton.Click();
				else if(result == ResponseType.Cancel)
					e.Unload = false;
				//this.foodList.Clear();
				dlg.Hide();
				dlg.Dispose();
			}
		}
		
		#endregion
		
		#region Editor logic
		
		/// <summary>
		/// Currently selected FoodItem, null if none
		/// </summary>
		private FoodItemAdapter current = null;
		
		/// <summary>
		/// remove the selectet fooditem from the treview list
		/// </summary>
		void OnRemoveButtonClicked (object sender, System.EventArgs e){
			if(current != null)
				this.foodList.RemoveNode(current);
			current = null;
			UpdateEditor();
		}
		
		/// <summary>
		/// creates a new fooditem without bar code
		/// </summary>
		void OnAddButtonClicked (object sender, System.EventArgs e){
			//Ask for an ingredient
			IngredientSelector dialog = new IngredientSelector(false);
			if (dialog.Run() == (int)ResponseType.Ok){
				//Clear the barcode
				OnClearBarCodeButtonClicked(this, e);
				                             
				//Create and select a new FoodItem
				var ing = dialog.SelectedIngredient;
				
				current = new FoodItemAdapter(new FoodItem(100, ing, DateTime.Now + ing.ShelfLife, false));
				foodList.AddNode(current);
				this.FoodView.NodeSelection.SelectNode(current);
				this.UpdateEditor();
			}
			dialog.Destroy();
		}

		/// <summary>
		/// opens a new IngredientSelector dialog where the user can select an Ingredient to associate with the barcode
		/// </summary>
		void OnSelectIngredientButtonClicked (object sender, System.EventArgs e){
			if(current != null){
				IngredientSelector dialog = new IngredientSelector(current.FoodItem.Ingredient, false);
				if (dialog.Run() == (int)ResponseType.Ok){
					current.FoodItem.Ingredient = dialog.SelectedIngredient; 
					current.FoodItem.ExpirationDate = DateTime.Now + dialog.SelectedIngredient.ShelfLife;
					this.IngredientLabel.Text = current.FoodItem.Ingredient.ShortDescription;
					this.ExpirationCalendar.Date = current.FoodItem.ExpirationDate;
				}
				dialog.Destroy(); 
			}else
				OnAddButtonClicked(this, e);
		}

		/// <summary>
		/// Current bar code, null if none
		/// </summary>
		private BarCode code = null;
		
		/// <summary>
		/// Create FoodItem for barcode, and set code, show ingredient dialog if barcode is unknown
		/// </summary>
		void OnScannerBarScanned (object sender, BarScannedArgs e){
			//Find the bar code, if it's there
			BarCodeType type = FromZBarType(e.Symbol.Type);
			this.code = BarCode.Find(e.Symbol.Data,type);
			if (this.code != null){ //Has barcode
				current = new FoodItemAdapter(new FoodItem(code.Quantity, code.Ingredient, DateTime.Now + code.Ingredient.ShelfLife, false));
				foodList.AddNode(current);
				this.FoodView.NodeSelection.SelectNode(current);
			}else{ // Don't have bar code
				//Ask for an ingredient
				IngredientSelector dialog = new IngredientSelector(false);
				if (dialog.Run() == (int)ResponseType.Ok){
					var ing = dialog.SelectedIngredient;
					//Create barcode
					code = new BarCode(e.Symbol.Data.ToString(), type);
					code.ProductName = ing.LongDescription;
					code.Quantity = 100;
					code.Ingredient = ing;
					code.Save();
					
					//Create and select a new FoodItem
					current = new FoodItemAdapter(new FoodItem(code.Quantity, code.Ingredient, DateTime.Now + code.Ingredient.ShelfLife, false));
					foodList.AddNode(current);
					this.FoodView.NodeSelection.SelectNode(current);
				}
				dialog.Destroy();
			}
			UpdateEditor();
		}
		
		/// <summary>
		/// clears the barcode label, and set code = null
		/// </summary>
		protected virtual void OnClearBarCodeButtonClicked (object sender, System.EventArgs e){
			this.code = null;
			this.ClearBarCodeButton.Sensitive = false;
			this.BarCodeLabel.Text = "-";
			this.scanner.ResetLastItemScanned();
		}

		/// <summary>
		/// Update the widgets in the editor according to internal state
		/// </summary>
		private void UpdateEditor(){
			//Update sensistivity
			this.SelectIngredientButton.Sensitive = current != null;
			this.ExpirationCalendar.Sensitive = current != null;
			this.RemoveButton.Sensitive = current != null;
			this.DuplicateButton.Sensitive = current != null;
			this.QuantitySpinbutton.Sensitive = current != null;
			this.ClearBarCodeButton.Sensitive = current != null && code != null;
			//Update editor
			if(current != null){
				this.ExpirationCalendar.Date = current.FoodItem.ExpirationDate;
				this.QuantitySpinbutton.Value = current.FoodItem.Quantity;
				this.IngredientLabel.Text = current.FoodItem.Ingredient.LongDescription;
			}else{
				this.ExpirationCalendar.Date = DateTime.Now;
				this.QuantitySpinbutton.Value = 0;
				this.IngredientLabel.Text = "-";
			}
			if(current != null && code != null){
				this.BarCodeLabel.Markup = "<i>" + GLib.Markup.EscapeText(code.ToString()) + "</i>";
			}else
				this.BarCodeLabel.Text = "-";
			
			SaveButton.Sensitive  = false;
			this.cancelled = true;
			if(foodList.GetEnumerator().MoveNext()) {
				this.cancelled = false;
				SaveButton.Sensitive  = true;
			}
		}
		
		/// <summary>
		/// Update current FoodItem and editor, and set barcode = null
		/// </summary>	
		protected virtual void OnFoodViewCursorChanged (object sender, System.EventArgs e){
			this.current = (FoodItemAdapter)this.FoodView.NodeSelection.SelectedNode;
			this.UpdateEditor();
		}

		/// <summary>
		/// Update quantity of current FoodItem and bar code if not null
		/// </summary>
		protected virtual void OnQuantitySpinbuttonValueChanged (object sender, System.EventArgs e){
			if(current != null){
				current.FoodItem.Quantity = QuantitySpinbutton.Value;
				current.ReportChange();
			}
			if(code != null){
				code.Quantity = QuantitySpinbutton.Value;
				code.Save();
			}
		}

		/// <summary>
		/// Duplicate current FoodItem
		/// </summary>
		protected virtual void OnDuplicateButtonClicked (object sender, System.EventArgs e){
			current = new FoodItemAdapter(new FoodItem(current.FoodItem.Quantity, current.FoodItem.Ingredient, current.ExpirationDate, false));
			foodList.AddNode(current);
			this.FoodView.NodeSelection.SelectNode(current);
			this.UpdateEditor();
		}
		
		/// <summary>
		/// Update expiration date of current FoodItem
		/// </summary>
		protected virtual void CalendarChanged (object sender, System.EventArgs e)
		{
			if(current != null){
				current.FoodItem.ExpirationDate = this.ExpirationCalendar.Date;
				current.ReportChange();
			}
		}
		
		#endregion
		
		/// <summary>
		/// Convert ZBar type to Foodolini BarCodeType
		/// </summary>
		private BarCodeType FromZBarType(SymbolType symbol){
			switch(symbol){
			case SymbolType.UPCE:
				return BarCodeType.UPCE;
			case SymbolType.UPCA:
				return BarCodeType.UPCA;
			case SymbolType.CODE128:
				return BarCodeType.CODE128;
			case SymbolType.CODE39:
				return BarCodeType.CODE39;
			case SymbolType.EAN13:
				return BarCodeType.EAN13;
			case SymbolType.EAN8:
				return BarCodeType.EAN8;
			case SymbolType.I25:
				return BarCodeType.I25;
			case SymbolType.ISBN10:
				return BarCodeType.ISBN10;
			case SymbolType.ISBN13:
				return BarCodeType.ISBN13;
			case SymbolType.PDF417:
				return BarCodeType.PDF417;
			}
			throw new Exception("Unknown barcode type...");
		}
	}
}
