using System;
using System.Collections.Generic;
using Gtk;
using Foodolini.BusinessLogic;
using Foodolini.Activities;

namespace Foodolini.Activities.Ingredients
{
    [FoodoliniActivity("Ingredients", false, "Ingredient.png")]
    [System.ComponentModel.ToolboxItem(true)]
    public partial class IngredientsActivity : Gtk.Bin, IActivity
    {
        private NodeStore ingredientStore = new NodeStore(typeof(IngredientAdapter));
        ListStore categoryStore = new ListStore(typeof(string));

        public IngredientsActivity()
        {
            this.Build();
            this.LoadData();
        }

        public IngredientsActivity(Ingredient defaultSelection) : this()
        {
            SelectIngredient(defaultSelection);
        }

		private IngredientAdapter SelectedNode{
			get{
				return this.ingredientsView.NodeSelection.SelectedNode as IngredientAdapter;
			}
		}
		
        internal Ingredient SelectedIngredient
        {
            get
            {
                Ingredient ret = null;
                if (this.SelectedNode != null)
                    ret = SelectedNode.Ingredient;
                return ret;
            }
        }

        public void Register(IOwner owner)
        {
            //Don't need references to owner
        }

        void LoadData()
        {
            // Load categories and select the selectedCategory category
            List<string> categories = new List<string>(Ingredient.Categories);
            categories.Sort();
			categoryStore.AppendValues("Any category (first 500 results)");
            foreach (string category in categories) {
                categoryStore.AppendValues(category);
            }
            this.CategoryCombobox.Model = categoryStore;
            this.CategoryCombobox.QueueResize();
            TreeIter first;
            this.CategoryCombobox.Model.GetIterFirst(out first);
            this.CategoryCombobox.SetActiveIter(first);

            LoadIngredients();

            this.ingredientsView.AppendColumn("Ingredient", new CellRendererText(), "text", 0);
            this.ingredientsView.AppendColumn("Protein (g)", new CellRendererText(), (column, cell, node) =>
            {
                IngredientAdapter n = (IngredientAdapter)node;
                CellRendererText c = (CellRendererText)cell;
                c.Text = n.Protein.ToString("0.00");
            });

            this.ingredientsView.AppendColumn("Carbohydrates (g)", new CellRendererText(), (column, cell, node) =>
            {
                IngredientAdapter n = (IngredientAdapter)node;
                CellRendererText c = (CellRendererText)cell;
                c.Text = n.Carbohydrates.ToString("0.00");
            });

            this.ingredientsView.AppendColumn("Fat (g)", new CellRendererText(), (column, cell, node) =>
            {
                IngredientAdapter n = (IngredientAdapter)node;
                CellRendererText c = (CellRendererText)cell;
                c.Text = n.Fat.ToString("0.00");
            });

            this.ingredientsView.NodeStore = ingredientStore;
        }

        void LoadIngredients()
        {
            if (this.ingredientsView.NodeStore != null)
                this.ingredientsView.NodeStore.Clear();
            // Load all ingredients from the active category
            List<Ingredient> ingredients = new List<Ingredient>(); 
			
			if(this.CategoryCombobox.Active > 0)
				ingredients = new List<Ingredient>(Ingredient.ListByCategory(this.CategoryCombobox.ActiveText, this.SearchEntry.Text));
			else //if(this.SearchEntry.Text.Length >= 3)
				ingredients = new List<Ingredient>(Ingredient.ListByLongDescription(this.SearchEntry.Text));
			
            ingredients.Sort((i1, i2) => i1.LongDescription.CompareTo(i2.LongDescription));
            foreach (var ingredient in ingredients) {
                ingredientStore.AddNode(new IngredientAdapter(ingredient));
            }
        }

        public void Unregister()
        {
            //Haven't got any tried to owner
        }

        protected virtual void OnSearchEntryChanged(object sender, System.EventArgs e)
        {
			
            LoadIngredients();
			// notify about changes in selection
			this.EditIngredientButton.Sensitive = (this.SelectedNode!=null);
			if(OnIngredientSelectionChanged!=null)
				OnIngredientSelectionChanged(this,new EventArgs());
        }

		private void OpenEditDialog(IngredientAdapter selectedNode){
			//TODO: use double-click for selection if in a dialog
            
            IngredientDialog diag = new IngredientDialog(selectedNode.Ingredient);

            ResponseType res = (ResponseType)diag.Run();

            if (res == ResponseType.Ok) {
                diag.SaveIngredient();
                SelectIngredient(diag.Ingredient);
            } else if(res == ResponseType.Reject){
				//Confirmation dialog
				MessageDialog confirm = new MessageDialog(null,
				                                          DialogFlags.DestroyWithParent,
				                                          MessageType.Warning,
				                                          ButtonsType.OkCancel, 
				                                          "Are you sure you want to delete this ingredient?");
				if(confirm.Run()==(int)ResponseType.Ok) {
					Ingredient ingredient = selectedNode.Ingredient;
					ingredientStore.RemoveNode(selectedNode);
                	ingredient.Delete();
				}
				confirm.Destroy();
			}
            diag.Destroy();
		}
		
        protected virtual void OnIngredientsViewRowActivated(object o, Gtk.RowActivatedArgs args)
        {
			var selectedNode = (IngredientAdapter)ingredientStore.GetNode(args.Path);
			OpenEditDialog(selectedNode);
        }

        [GLib.ConnectBeforeAttribute()]
        protected virtual void OnIngredientsViewButtonPressEvent(object o, Gtk.ButtonPressEventArgs args)
        {
            /*if (args.Event.Button == 3) {
                var selectedNode = (IngredientAdapter)ingredientsView.NodeSelection.SelectedNode;
                if (selectedNode != null) {
                    Menu m = new Menu();
                    MenuItem deleteItem = new MenuItem("Delete this item");
                    deleteItem.ButtonPressEvent += (obj, e) =>
                    {
                        Ingredient ingredient = selectedNode.Ingredient;
                        ingredientStore.RemoveNode(selectedNode);
                        ingredient.Delete();
                    };
                    m.Add(deleteItem);
                    m.ShowAll();
                    m.Popup();
                }
            }*/
        }

        internal void SelectIngredient(Ingredient savedIngredient)
        {
            // Set correct category
            int i = 0;
            // iterate over the categories and set the active iter.
            foreach (object[] obj in categoryStore) {
                string cat = obj[0].ToString();
                if (cat == savedIngredient.Category) {
                    TreeIter it;
                    if (CategoryCombobox.Model.GetIterFromString(out it, i.ToString()))
                        CategoryCombobox.SetActiveIter(it);
                    break;
                }
                i++;
            }
            // Reload ingredients
            LoadIngredients();

            // Select the ingredient in nodeview
            foreach (IngredientAdapter node in ingredientsView.NodeStore) {
                if (Ingredient.Compare(node.Ingredient,savedIngredient)) {
                    ingredientsView.NodeSelection.SelectNode(node);
                    break;
                }
            }
        }

        protected virtual void OnCreateIngredientButtonClicked(object sender, System.EventArgs e)
        {
            IngredientDialog dialog = new IngredientDialog(this.CategoryCombobox.ActiveText);
            ResponseType res = (ResponseType)dialog.Run();
            if (res == ResponseType.Ok) {
                dialog.SaveIngredient();
                SelectIngredient(dialog.Ingredient);
            }
            dialog.Destroy();
        }
        
		public event EventHandler OnIngredientSelectionChanged;
        
		/// <summary>
		/// Notify that the selected ingredient has changed. 
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
        protected virtual void OnIngredientsViewCursorChanged (object sender, System.EventArgs e)
        {
			this.EditIngredientButton.Sensitive = (this.SelectedNode!=null);
			
			if(OnIngredientSelectionChanged!=null)
				OnIngredientSelectionChanged(this,new EventArgs());
        }
        
        	
        protected virtual void OnEditIngredientButtonClicked (object sender, System.EventArgs e)
        {
			if(this.SelectedNode != null)
			{
				OpenEditDialog(this.SelectedNode);
			}
        }



        public Widget Widget
        {
            get { return this; }
        }

    }
}
