using System;
using System.Collections;
using System.Collections.Generic;
using Gtk;
using Foodolini.BusinessLogic;
using Foodolini.Activities;
using System.Text;
using Foodolini.Activities.Ingredients;

namespace Foodolini.Activities.ShoppingList
{
    [FoodoliniActivity("Shopping shoppingList", true)]
    [System.ComponentModel.ToolboxItem(true)]
    public partial class ShoppingListActivity : Gtk.Bin, IActivity
    {
        private NodeStore shoppingListStore;
        private Foodolini.BusinessLogic.ShoppingList shoppingList;
        private Person currentUser;

        public ShoppingListActivity()
        {
            this.Build();
        }

        public Widget Widget
        {
            get { return this; }
        }

        private IOwner owner;

        public void Register(IOwner owner)
        {
            this.owner = owner;
            this.owner.CurrentUserChanged += OwnerhandleCurrentUserChanged;

			CellRendererSpin quantityCell = new CellRendererSpin();
			quantityCell.Editable=true;
			quantityCell.Adjustment=new Adjustment(0,0,100000, 50, 50, 50);
			quantityCell.Digits = 0;
            this.shoppingListViewer.AppendColumn("Quantity (g)", 
			quantityCell, (column, cell, node) =>
            {
                ((CellRendererSpin)cell).Text = ((ShoppingListItemAdapter)node).Quantity.ToString ("0");
            });
			
			quantityCell.Edited += delegate(object o, EditedArgs args) {
				var node = (ShoppingListItemAdapter)shoppingListStore.GetNode(new TreePath(args.Path));
				double newValue;
				if(double.TryParse(args.NewText, out newValue)) {
					node.Quantity = newValue;
					// Update shopping list
					this.shoppingList.Save();
				}
			};
			
            this.shoppingListViewer.AppendColumn("Ingredient", new CellRendererText(), "text", 1);
            //this.shoppingListViewer.AppendColumn("Recipe", new CellRendererText(), "text", 0);
        }

        void OwnerhandleCurrentUserChanged(object sender, PersonEventArgs e)
        {
            this.shoppingListViewer.NodeStore = null;
            currentUser = e.Person;
            btnAddIngredient.Sensitive = 
                btnClear.Sensitive = 
                btnPrint.Sensitive = (currentUser != null);

            if (currentUser != null)
                LoadShoppingListData();
        }

        /// <summary>
        /// Loads shopping list data for the current user
        /// and binds the data to the treeview.
        /// </summary>
        private void LoadShoppingListData()
        {
            shoppingListStore = new NodeStore(typeof(ShoppingListItemAdapter));
            shoppingList = new Foodolini.BusinessLogic.ShoppingList(currentUser);

            foreach (var item in shoppingList.GetShoppingListItems())
                this.shoppingListStore.AddNode(new ShoppingListItemAdapter(item));
            
            this.shoppingListViewer.NodeStore = shoppingListStore;
            this.shoppingListViewer.QueueResize();
        }

        public void Unregister()
        {
            this.owner.CurrentUserChanged -= OwnerhandleCurrentUserChanged;
            this.owner = null;
        }

        protected virtual void OnBtnClearActivated(object sender, System.EventArgs e)
        {
            shoppingList.Clear();
            LoadShoppingListData();
        }

        protected virtual void OnBtnAddIngredientClicked(object sender, System.EventArgs e)
        {
            IngredientSelector dialog = new IngredientSelector(true);
            if (dialog.Run() == (int)ResponseType.Ok) {
                double qty = dialog.Quantity;
                Ingredient ingredient = dialog.SelectedIngredient;
                this.shoppingList.AddIngredient(ingredient, qty);
                this.shoppingList.Save();
                LoadShoppingListData();
            }
            dialog.Destroy();
            SetDeleteButtonState();
        }
		
		/// <summary>
		/// Add all ingredients needed for the recipe to the shoppinglist
		/// </summary>
		/// <remarks>
		/// This is all ingredients, and does not depend on what is on storage.
		/// If the nature of the shoppinglist is changed in the future, this may change.
		/// </remarks>
		public void AddRecipe(Recipe recipe, double servings){
			if(this.owner.CurrentUser == null)
				throw new Exception("Cannot add an ingredient to shopping list when no user is selected!");
			this.shoppingList.AddRecipe(recipe, servings);
			this.shoppingList.Save();
			LoadShoppingListData();
		}

        /// <summary>
        /// Prints the shopping shoppingList
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected virtual void OnBtnPrintClicked(object sender, System.EventArgs e)
        {
            /*PrintOperation p = new PrintOperation();

            StringBuilder sb = new StringBuilder();
            sb.Append("<b>Shopping shoppingList</b>\n");
            foreach (ShoppingListItemAdapter item in this.shoppingListStore) {
                sb.Append(string.Format("{0,10} {1}  ", item.Quantity, "g"));
                sb.Append(string.Format("{0,20}", item.Ingredient));
                if (!string.IsNullOrEmpty(item.Recipe)) {
                    sb.Append("\t(" + item.Recipe + ")");
                }
                sb.Append("\n");
            }
			//Compute number of pages needed to print
			p.Paginate += delegate(object o, PaginateArgs args) {
				(o as PrintOperation).NPages = 1;
				args.RetVal = true;
			};
			//Print the stuff
            p.DrawPage += delegate(object o, DrawPageArgs args)
            {
                var layout = args.Context.CreatePangoLayout();
                layout.Wrap = Pango.WrapMode.WordChar;
                layout.SetMarkup(sb.ToString());
                Pango.CairoHelper.ShowLayout(args.Context.CairoContext, layout);
            };
            p.Run(PrintOperationAction.PrintDialog, this.owner.Window);
            p.Dispose();*/

            StringBuilder sb = new StringBuilder();
            //sb.Append("<b>Shopping shoppingList</b>\n");
            foreach (ShoppingListItemAdapter item in this.shoppingListStore) {
                sb.Append(string.Format("{0,10} {1}  ", item.Quantity, "g"));
                sb.Append(string.Format("{0,20}", item.Ingredient));
                if (!string.IsNullOrEmpty(item.Recipe)) {
                    sb.Append("\t(" + item.Recipe + ")");
                }
                sb.Append("\n");
            }
			
			//Widths of the two cols
			double qWidth = 0, dWidth;
			
			//Headline holder
			Pango.Layout headline = null;
			
			//New print job
			PrintOperation p = new PrintOperation();
			//Compute number of pages needed to print
			p.Paginate += delegate(object o, PaginateArgs args) {
				//Setup the layouts and find the desired width for the cols
				foreach(ShoppingListItemAdapter item in this.shoppingListStore){
					var w = item.SetupLayout(args.Context);
					qWidth = Math.Max(qWidth, w);
				}
				
				//Compute col widths
				double docWidth = args.Context.Width;
				qWidth = Math.Min(qWidth, docWidth / 2); //No larger than half the page, probably not needed
				dWidth = docWidth - qWidth - 20 - 20; //20 is the margin
				
				//Setup headline and compute it's height
				headline = args.Context.CreatePangoLayout();
				headline.SetMarkup("<span size='xx-large' face='DejaVu Sans'>Shopping list, " + DateTime.Now.ToString("d") + "</span>");
				//Get height
				int hw, hh;
				headline.GetPixelSize(out hw, out hh);
				double height = hh + 10 + 10 + 10;//10 after first line, 10 before and after last line
				
				//Adjust width and find out many pages are needed
				double pageHeight = args.Context.Height - 20; //Cache the height, -20 for top and bottom margins
				int pageCount = 1;
				foreach(ShoppingListItemAdapter item in this.shoppingListStore){
					var h = item.AdjustWidth(qWidth, dWidth);
					if(h + height > pageHeight){
						pageCount++;
						height = h;
					}else
						height += h;
				}
				
				(o as PrintOperation).NPages = pageCount;
				args.RetVal = true;
			};
			
			IEnumerator rows = null;
			//Print the stuff
            p.DrawPage += delegate(object o, DrawPageArgs args)
            {
				//Get the cairo context for drawing
				Cairo.Context context = args.Context.CairoContext;
				//Use black color
				context.SetSourceRGB(0 ,0 ,0);
				
				//Line style
				context.LineCap = Cairo.LineCap.Round;
				context.LineWidth = 2;
				
				context.MoveTo(10, 10);
				Cairo.PointD pos = context.CurrentPoint;
				
				if(rows == null){
					//Draw line above headline
					context.LineTo(args.Context.Width - 10, 10);
					context.Stroke();
					context.MoveTo(10, 20);
					
					//Get headline height
					int width, height;
					headline.GetPixelSize(out width, out height);
	
					//Draw headline
					Pango.CairoHelper.ShowLayout(context, headline);
					
					//Draw line under headline
					context.RelMoveTo(0, height + 10);
					pos = context.CurrentPoint;
					context.RelLineTo(args.Context.Width - 20, 0);
					context.Stroke();
					context.MoveTo(pos);
					context.RelMoveTo(0, 10);
					pos = context.CurrentPoint;
					
					//Setup rows
					rows = this.shoppingListStore.GetEnumerator();
					if(!rows.MoveNext())
						return; //Nothing more to draw
				}
				
				//Hold where the table starts
				double tableY = pos.Y;
				
				do{
					ShoppingListItemAdapter row = (ShoppingListItemAdapter)rows.Current;
					if(row.NextPage(args.Context.Width - pos.X, args.Context.Height - pos.Y - 10))
						break;
					pos.Y += row.Render(context, pos, qWidth);
				}while(rows.MoveNext());
				
				//Draw line between columns
				context.LineWidth = 1;
				context.SetDash(new double[]{5, 5}, 0);
				context.MoveTo(10 + qWidth + 5, tableY);
				context.LineTo(10 + qWidth + 5, pos.Y + 5);
				context.Stroke();
            };
            p.Run(PrintOperationAction.PrintDialog, this.owner.Window);
            p.Dispose();
        }

        /// <summary>
        /// Goes to the cookbook activity
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected virtual void OnBtnCookbookClicked(object sender, System.EventArgs e)
        {
            this.owner.PushActivity("Cookbook");
        }

        /// <summary>
        /// Deletes an item from the shopping shoppingList.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected virtual void OnBtnDeleteClicked(object sender, System.EventArgs e)
        {
            ShoppingListItemAdapter selectedNode = (ShoppingListItemAdapter)this.shoppingListViewer.NodeSelection.SelectedNode;
            if (selectedNode != null)
                this.shoppingListStore.RemoveNode(selectedNode);
            this.shoppingList.RemoveIngredient(selectedNode.Item.Ingredient);
        }

        private void SetDeleteButtonState()
        {
            if (this.shoppingListViewer.NodeSelection.SelectedNode != null)
                this.btnDelete.Sensitive = true;
            else
                this.btnDelete.Sensitive = false;
        }

        protected virtual void OnShoppingListViewerCursorChanged(object sender, System.EventArgs e)
        {
            SetDeleteButtonState();
        }
        
        /// <summary>
        /// Changes the ingredient in shoppinglist item. 
        /// </summary>
        /// <param name="o">
        /// A <see cref="System.Object"/>
        /// </param>
        /// <param name="args">
        /// A <see cref="Gtk.RowActivatedArgs"/>
        /// </param>
        protected virtual void OnShoppingListViewerRowActivated (object o, Gtk.RowActivatedArgs args)
        {
			var shoppingItem = (ShoppingListItemAdapter)this.shoppingListViewer.NodeSelection.SelectedNode;
			IngredientSelector selector = new IngredientSelector(shoppingItem.Item.Ingredient);
			if(selector.Run() == (int)ResponseType.Ok){
				if(!shoppingList.ContainsIngredient(selector.SelectedIngredient)) {
					shoppingItem.Item.Ingredient = selector.SelectedIngredient;
					shoppingList.Save();
					LoadShoppingListData();
				} else {
					ShowIngredientAlreadyAddedMessage();
				}
			}
			selector.Destroy();
        }

		private void ShowIngredientAlreadyAddedMessage(){
			MessageDialog diag = 
				new MessageDialog(this.owner.Window, 
					              DialogFlags.Modal,
					              MessageType.Info, 
					              ButtonsType.Ok,
					              @"The selected ingredient is already in your shopping list.
									Please update the quantity if you wish to add more.");
			if(diag.Run() == (int)ResponseType.Ok){}
			diag.Destroy();
		}

    }
}
