using System;
using System.Collections.Generic;
using Gtk;
using Foodolini.BusinessLogic;

namespace Foodolini.Activities.ShoppingList
{
	/// <summary>
	/// Adapts a ShoppingListItem to the TreeNode interface, and adds methods for printing
	/// </summary>
	public class ShoppingListItemAdapter : TreeNode
	{

		public new Foodolini.BusinessLogic.ShoppingListItem Item{get;private set;}
		
		public ShoppingListItemAdapter (ShoppingListItem item)
		{
			this.Item=item;
		}
		[Gtk.TreeNodeValue(Column = 0)]
		public string Recipe{
			get{
                if (this.Item.Recipe != null)
                    return this.Item.Recipe.Title;
                else
                    return null;
			}
		}
		
		[Gtk.TreeNodeValue(Column = 1)]
		public string Ingredient{
			get{
				return this.Item.Ingredient.LongDescription;
			}
		}
		
		[Gtk.TreeNodeValue(Column = 2)]
		public double Quantity{
			get{
				return this.Item.Quantity;
			}
			set{
				this.Item.Quantity=value;
			}
		}
		
		#region Printing methods
		
		/// <summary>
		/// Quantity column
		/// </summary>
		private Pango.Layout qCol;
		
		/// <summary>
		/// Description column
		/// </summary>
		private Pango.Layout dCol;
		
		/// <summary>
		/// Setup the layout for printing
		/// </summary>
		/// <returns>
		/// Width of the quantity column
		/// </returns>
		public double SetupLayout(PrintContext context){
			qCol = context.CreatePangoLayout();
			qCol.Alignment = Pango.Alignment.Right;
			qCol.Wrap = Pango.WrapMode.WordChar;
			qCol.SetText(this.Quantity + " g");
			
			int w, h;
			qCol.GetPixelSize(out w, out h);
			
			dCol = context.CreatePangoLayout();
			dCol.Alignment = Pango.Alignment.Left;
			dCol.Wrap = Pango.WrapMode.WordChar;
			dCol.SetText(this.Ingredient + (string.IsNullOrEmpty(this.Recipe) ? "" : " (" + this.Recipe + ")"));
			
			return (double)w;
		}
		
		/// <summary>
		/// Adjust the size of the columns
		/// </summary>
		/// <param name="qColWidth">
		/// Size of the quantity column
		/// </param>
		/// <param name="dColWidth">
		/// Size of the description column
		/// </param>
		/// <returns>
		/// The total height consumed by this row
		/// </returns>
		public double AdjustWidth(double qColWidth, double dColWidth){
			qCol.Width = (int)Math.Ceiling(qColWidth * Pango.Scale.PangoScale);
			dCol.Width = (int)Math.Ceiling(dColWidth * Pango.Scale.PangoScale);
			int qw, qh, dw, dh;
			qCol.GetPixelSize(out qw, out qh);
			dCol.GetPixelSize(out dw, out dh);
			return Math.Max(dh, qh) + 4;
		}
		
		/// <summary>
		/// Check if there's room for this row on the current page
		/// </summary>
		/// <param name="width">
		/// Available width
		/// </param>
		/// <param name="height">
		/// Available height
		/// </param>
		/// <returns>
		/// True, if this row should be moved the next page
		/// </returns>
		public bool NextPage(double width, double height){
			int qw, qh, dw, dh;
			qCol.GetPixelSize(out qw, out qh);
			dCol.GetPixelSize(out dw, out dh);
			return qw + dw + 10 > width || Math.Max(qh, dh) + 6 > height;
		}
		
		/// <summary>
		/// Render this row onto a Cairo.Context
		/// </summary>
		/// <param name="context">
		/// Context this row should be rendered on
		/// </param>
		/// <param name="position">
		/// The position this row should be rendered to
		/// </param>
		/// <param name="qColWidth">
		/// Size of the quantity column
		/// </param>
		/// <returns>
		/// The height of the row
		/// </returns>
		public double Render(Cairo.Context context, Cairo.PointD position, double qColWidth){
			int qw, qh, dw, dh;
			qCol.GetPixelSize(out qw, out qh);
			dCol.GetPixelSize(out dw, out dh);
			position.Y += 3;
			context.MoveTo(position);
			
			Pango.CairoHelper.ShowLayout(context, qCol);
			context.RelMoveTo(qColWidth + 10, 0);
			Pango.CairoHelper.ShowLayout(context, dCol);
			
			return Math.Max(qh, dh) + 3;
		}
		
		#endregion
	}
}
