using System;
using System.Collections.Generic;
using System.Text;
using Foodolini.Database;

using ZBar; 

namespace Foodolini.BusinessLogic
{
	/// <summary>
	/// Barcode class contains product name and quantity about a food item, 
	/// and contains a barcode value and a barcode 
	/// type to associate with an ingredient 
	/// </summary>
	public class BarCode
	{
		private BarCodeRow row;
		private bool modified = true;
		
		private BarCode(BarCodeRow row){
			this.row = row;
			if(this.row.BarCodeRowId != 0)
				this.modified = false;
		}
		
		/// <summary>
		///	Creates a new barcode.
		/// </summary>
		/// <param name="identifier">
		/// A <see cref="System.Int64"/> representing a identifier.
		/// </param>
		/// <param name="productName">
		/// A <see cref="System.String"/>, representing a product name.
		/// </param>
		/// <param name="quantity">
		/// A <see cref="System.Single"/>, representing the amount of the product. 
		/// </param>
		/// <param name="ingredient">
		/// A <see cref="Ingredient"/>, representing the ingredient to wich the barcode is associated with.
		/// </param>
		/// <param name="type">
		/// A <see cref="BarCodeType"/>, representing the type of the barcode.
		/// </param>
		public BarCode(string identifier, string productName, float quantity, Ingredient ingredient, BarCodeType type){
			this.row = new BarCodeRow();
			this.row.Identifier = identifier;
			this.row.FoodDescriptionId = ingredient.Id;
			this.row.ProductName = productName;
			this.row.Quantity = quantity;
			this.ingredient = ingredient;
			this.row.Type = (int)type;
			this.modified = true;
		}

		/// <summary>
		///	Creates a new barcode.
		/// </summary>
		/// <param name="identifier">
		/// A <see cref="System.Int64"/> representing a identifier.
		/// </param>
		/// <param name="type">
		/// A <see cref="BarCodeType"/>, representing the type of the barcode.
		/// </param>
		public BarCode(string identifier, BarCodeType type){
			this.row = new BarCodeRow();
			this.row.Identifier = identifier;
			this.row.Type = (int)type;
			this.modified = true;
		}
		
		/// <summary>
		/// Gets/sets the Identifier as a <see cref="System.String"/>.
		/// </summary>
		public string Identifier{
			get{
				return this.row.Identifier;
			}
			set{
				this.modified = true;
				this.row.Identifier = value;
			}
		}
		
		/// <summary>
		/// Gets/sets the ProductName as a <see cref="System.String"/>.
		/// </summary>
		public string ProductName{
			get{
				return this.row.ProductName;
			}
			set{
				this.modified = true;
				this.row.ProductName = value;
			}
		}
		
		/// <summary>
		/// Gets/sets the Quantity as a <see cref="System.float"/>.
		/// </summary>
        /// <remarks>Default is 1000 grams</remarks>
		public double Quantity{ 
			get{
				return this.row.Quantity.GetValueOrDefault(1000);
			}
			set{
				this.modified = true;
				this.row.Quantity = value;
			}
		}
		
		private Ingredient ingredient = null;
		
		/// <summary>
		/// Gets/sets the Ingredient as a <see cref="Ingredient"/>.
		/// </summary>
		public Ingredient Ingredient{
			get{
				if(this.ingredient == null)
					this.ingredient = new Ingredient(this.row.FoodDescriptionId);
				return this.ingredient;
			}
			set{
				this.modified = true;
				this.ingredient = value;
			}
		}
		
		/// <summary>
		/// Gets/sets the BarCodeType as a <see cref="BarCodeType"/>.
		/// </summary>
		public BarCodeType Type{
			get{
				return (BarCodeType)this.row.Type;
			}
			set{
				this.modified = true;
				this.row.Type = (int)value;
			}
		}
		
        /// <summary>
        /// Searches for a bar code which matches the specified idenifier.
        /// </summary>
        /// <param name="identifier">The bar code to search for</param>
        /// <param name="barCodeType">The type of bar code</param>
        /// <returns>A bar code or null if not found</returns>
        public static BarCode Find(string identifier, BarCodeType barCodeType){
            string condition = "Identifier = @0 AND Type = @1";
			BarCodeRow barcoderow = Settings.Repo.SingleWhere<BarCodeRow>(condition, identifier, (int)barCodeType);
            if (barcoderow != null)
				return new BarCode(barcoderow);
			else
				return null; 
					
		}
		
		public void Save(){
			if(this.ingredient != null){
				this.ingredient.Save();
				if(this.row.FoodDescriptionId != this.ingredient.Id){
					this.row.FoodDescriptionId = this.ingredient.Id;
					Console.WriteLine(this.row.FoodDescriptionId + " in save barcode");
					this.modified = true;
				}
			}
			if(this.row.BarCodeRowId == 0){
				Settings.Repo.Add<BarCodeRow>(this.row);
				Console.WriteLine(this.row.FoodDescriptionId  + "  2in save barcode");
			}
			else{
				if(this.modified)
					Settings.Repo.Update<BarCodeRow>(this.row);
			}
			this.modified = false;
		}
		/// <summary>
		///return the Identifer and type of the barcode as a string
		/// "Value (type)"
		/// </summary>
		/// <returns>
		/// A <see cref="System.String"/>
		/// </returns>
		public override string ToString ()
		{
			Console.WriteLine(this.Type.ToString());
			
			return Identifier + " (" + this.Type.ToString() + ")";
			
	 	}
		

	}
}
