using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using Foodolini.Database;
using System.Diagnostics;

namespace Foodolini.BusinessLogic
{
	public partial class Ingredient
	{
		/// <summary>
		/// A mutable dictionary of Nutrients and amounts as <see cref="System.Double"/>
		/// </summary>
		/// <remarks>
		/// This dictionary is associated with a specific FoodDescription row, it cannot be used to save
		/// the same associations another, in the database existing, FoodDescription row. As it will fail
		/// to update/delete any associates it might have in the database.
		/// However, it can be used to give a not in the database existing FoodDescription row, the same
		/// associations as another FoodDescription row.
		/// 
		/// This dictionary is implemented to minimize then amount of database requests needed to load and
		/// save NutritionalValues, and thus optimize the performance.
		/// </remarks>
		private class NutritionDictionary : IDictionary<Nutrient, double>
		{
			/// <summary>
			/// Cached NutitionalValues
			/// </summary>
			/// <remarks>
			/// A NutritionalValue should only exists in either cachedValues, changedValues or deletedValues.
			/// And should never exist in more than one, when not operating on the object.
			/// (This is an important class invariant)
			/// </remarks>
			private Dictionary<long, NutritionalValue> cachedValues = new Dictionary<long, NutritionalValue>();
			
			/// <summary>
			/// Changed or added NutritionalValues
			/// </summary>
			private Dictionary<long, NutritionalValue> changedValues = new Dictionary<long, NutritionalValue>();
			
			/// <summary>
			/// Deleted NutritionalValues
			/// </summary>
			private Dictionary<long, NutritionalValue> deletedValues = new Dictionary<long, NutritionalValue>();

			/// <summary>
			/// Create an instance of NutritionDictionary for a specific FoodDescription
			/// </summary>
			/// <param name="foodDescriptionId">
			/// Identifier of FoodDescription row to created an instance for, 0 if it's a new row.
			/// </param>
			public NutritionDictionary(long foodDescriptionId){
				if(foodDescriptionId != 0){
					var query = Settings.Repo.Where<NutritionalValue>("FoodDescriptionId = " + foodDescriptionId);
					foreach(NutritionalValue entry in query)
						this.cachedValues.Add(entry.NutritionDefinitionId, entry);
				}
			}

			/// <summary>
			/// Save these associations for a FoodDescription
			/// </summary>
			/// <remarks>
			/// This will save the NutritrionalValues for a specific FoodDescription, note that
			/// existing NutritrionalValues associated with this FoodDescription will only be overwritten,
			/// if this NutritionalDictionary was created specifically for this FoodDescription.
			/// </remarks>
			/// <param name="foodDescriptionId">
			/// FoodDescriptionId to associate these NutritionalValues with.
			/// </param>
			public void Save(long foodDescriptionId){
				//If the Id is not the same as they were loaded for, we must add them
				
				foreach(NutritionalValue val in this.cachedValues.Values){
					if(val.FoodDescriptionId != foodDescriptionId){
						Debug.Fail("This NutritionDictionary was created for another Ingredient!");
						val.FoodDescriptionId = foodDescriptionId;
						Settings.Repo.Add<NutritionalValue>(val);
					}
				}
				
				//Update or save values that have changed
				foreach(NutritionalValue val in this.changedValues.Values){
					//If it was loaded and then edited, and is associated with the foodDescriptionId we're saving for Update it
					if(val.NutritionalValueId != 0 && val.FoodDescriptionId == foodDescriptionId){
						Settings.Repo.Update<NutritionalValue>(val);
					}else{
						Debug.Assert(val.FoodDescriptionId == 0, "This NutritionDictionary was created for another Ingredient!");
						//Either this is a new NutritionalValue or this NutrionalValue is associated with another FoodDescription
						//So we can't use the existing one and need to create a new one.
						val.FoodDescriptionId = foodDescriptionId;
						Settings.Repo.Add<NutritionalValue>(val);
					}
					//Once saved add it to cachedValues
					this.cachedValues.Add(val.NutritionDefinitionId, val);
				}
				//Clear changedValues when they have all been saved and moved to cached
				this.changedValues.Clear();
				//Delete values that have been deleted
				foreach(NutritionalValue val in this.deletedValues.Values){
					//Delete it if it's associated with this FoodDescription and not new
					if(val.NutritionalValueId != 0 && val.FoodDescriptionId == foodDescriptionId)
						Settings.Repo.Delete<NutritionalValue>(val);
					else
						Debug.Assert(val.FoodDescriptionId == foodDescriptionId, "This NutritionDictionary was created for another Ingredient!");
				}
				//Clear deletedValues when we've deleted them
				this.deletedValues.Clear();
			}
			
			/// <summary>
			/// Delete the database representation of this NutritionDictionary
			/// </summary>
			/// <param name="foodDescriptionId">
			/// A <see cref="System.Int64"/>
			/// </param>
			public void Delete(long foodDescriptionId){
				//If the Id is the same as they were loaded for, we must delete it
				foreach(NutritionalValue val in this.cachedValues.Values){
					if(val.FoodDescriptionId == foodDescriptionId){
						Settings.Repo.Delete<NutritionalValue>(val);
						val.FoodDescriptionId = 0;
					}else
						Debug.Assert(val.FoodDescriptionId == 0, "This NutritionDictionary was created for another Ingredient!");
					this.changedValues.Add(val.NutritionDefinitionId, val);
				}
				//Nothing can be cached anymore
				this.cachedValues.Clear();
				//If the Id is the same as they were loaded for, we must delete it
				foreach(NutritionalValue val in this.changedValues.Values){
					if(val.FoodDescriptionId == foodDescriptionId){
						Settings.Repo.Delete<NutritionalValue>(val);
						val.FoodDescriptionId = 0;
					}else
						Debug.Assert(val.FoodDescriptionId == 0, "This NutritionDictionary was created for another Ingredient!");
				}
				//Delete values that have been deleted
				foreach(NutritionalValue val in this.deletedValues.Values){
					if(val.FoodDescriptionId == foodDescriptionId)
						Settings.Repo.Delete<NutritionalValue>(val);
					else
						Debug.Assert(val.FoodDescriptionId == 0, "This NutritionDictionary was created for another Ingredient!");
				}
				//Clear deletedValues as we've deleted them
				this.deletedValues.Clear();
			}

			/// <summary>
			/// Set the amount of a NutrionalDefinition
			/// </summary>
			/// <param name="defId">
			/// Id of the NutrionalDefinition
			/// </param>
			/// <param name="amount">
			/// Amount of the NutrionalDefinition
			/// </param>
			private void SetValue(long defId, double amount){
				NutritionalValue val = null;
				//If in cachedValues change and move it to changedValues
				if(this.cachedValues.TryGetValue(defId, out val)){
					val.Value = amount;
					this.changedValues.Add(defId, val);
					this.cachedValues.Remove(defId);
					//If in changedValues change it...
				} else if(this.changedValues.TryGetValue(defId, out val)){
					val.Value = amount;
					//If in deleteValues change and move it to changedValues
				} else if(this.deletedValues.TryGetValue(defId, out val)){
					val.Value = amount;
					this.changedValues.Add(defId, val);
					this.deletedValues.Remove(defId);
					//If not in any collection, create it and add it to changeValues
				} else{
					val = new NutritionalValue();
					val.FoodDescriptionId = 0;
					val.NutritionDefinitionId = defId;
					val.Value = amount;
					this.changedValues.Add(defId, val);
				}
			}
			
			#region IDictionary<Nutrient, double> implementation
			/// <summary>
			/// Add a NutiontialValue to the dictionary
			/// </summary>
			/// <remarks>This will override any other existing entry for the Nutrient, if such is present.</remarks>
			public void Add(Nutrient key, double value){
				this.SetValue(key.Id, value);
			}

			/// <summary>
			/// Check if the dictionary contains a NutritionalValue for a specific Nutrient
			/// </summary>
			/// <param name="key">
			/// A <see cref="Nutrient"/> to search for.
			/// </param>
			/// <returns>
			/// True, if it contains a NutritionalValue for the Nutrient
			/// </returns>
			public bool ContainsKey(Nutrient key){
				return this.cachedValues.ContainsKey(key.Id) || this.changedValues.ContainsKey(key.Id);
			}

			public bool Remove(Nutrient key){
				NutritionalValue val = null;
				if(this.cachedValues.TryGetValue(key.Id, out val)){
					//Don't care to mark it for deleting if it's not in the database
					if(val.NutritionDefinitionId != 0)
						this.deletedValues.Add(key.Id, val);
					this.cachedValues.Remove(key.Id);
					return true;
				}
				if(this.changedValues.TryGetValue(key.Id, out val)){
					//Don't care to mark it for deleting if it's not in the database
					if(val.NutritionDefinitionId != 0)
						this.deletedValues.Add(key.Id, val);
					this.changedValues.Remove(key.Id);
					return true;
				}
				return false;
			}

			public bool TryGetValue(Nutrient key, out double value){
				return TryGetValue(key.Id, out value);
			}
			
			public bool TryGetValue(long id, out double value){
				NutritionalValue val = null;
				if(this.cachedValues.TryGetValue(id, out val)){
					value = val.Value;
					return true;
				}
				if(this.changedValues.TryGetValue(id, out val)){
					value = val.Value;
					return true;
				}
				value = new double();
				return false;
			}

			public double this[Nutrient key]{
				get{
					double val;
					if(this.TryGetValue(key, out val))
						return val;
					throw new KeyNotFoundException(key.TagName + " was not found in the dictionary.");
				}
				set { this.SetValue(key.Id, value); }
			}

			#region NutritionDefinitionCollection
			private class NutritionDefinitionCollection : ICollection<Nutrient>
			{
				private NutritionDictionary owner;
				public NutritionDefinitionCollection(NutritionDictionary owner){
					this.owner = owner;
				}

				#region IEnumerable<Nutrient> implementation
				public IEnumerator<Nutrient> GetEnumerator(){
					foreach(long id in owner.cachedValues.Keys)
						yield return Nutrient.CreateNutritionDefinition(id);
					foreach(long id in owner.changedValues.Keys)
						yield return Nutrient.CreateNutritionDefinition(id);
				}
				#endregion

                #region IEnumerable implementation
                System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
                {
                    return this.GetEnumerator();
                }
                #endregion

				#region ICollection<Nutrient> implementation
				public void Add(Nutrient item){
					throw new NotSupportedException("This is a read-only collection, modify the NutritionDictionary instead!");
				}

				public void Clear(){
					throw new NotSupportedException("This is a read-only collection, modify the NutritionDictionary instead!");
				}

				public bool Contains(Nutrient item){
					return this.owner.ContainsKey(item);
				}

				public void CopyTo(Nutrient[] array, int arrayIndex){
					if(arrayIndex < 0)
						throw new ArgumentOutOfRangeException("arrayIndex", "arrayIndex cannot be less than 0");
					foreach(Nutrient def in this)
						array[arrayIndex++] = def;
				}

				public bool Remove(Nutrient item){
					throw new NotSupportedException("This is a read-only collection, modify the NutritionDictionary instead!");
				}

				public int Count{
					get { return this.owner.Count; }
				}

				public bool IsReadOnly{
					get { return true; }
				}
				#endregion
			}
			#endregion

			public ICollection<Nutrient> Keys{
				get { return new NutritionDefinitionCollection(this); }
			}

			#region NutritionValueCollection
			private class NutritionValueCollection : ICollection<double>
			{
				private NutritionDictionary owner;
				public NutritionValueCollection(NutritionDictionary owner){
					this.owner = owner;
				}

				#region IEnumerable<double> implementation
				public IEnumerator<double> GetEnumerator(){
					foreach(var val in owner.cachedValues.Values)
						yield return val.Value;
					foreach(var val in owner.changedValues.Values)
						yield return val.Value;
				}
				#endregion

                #region IEnumerable implementation
                System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
                {
                    return this.GetEnumerator();
                }
                #endregion

				#region ICollection<double> implementation
				public void Add(double item){
					throw new NotSupportedException("This is a read-only collection, modify the NutritionDictionary instead!");
				}

				public void Clear(){
					throw new NotSupportedException("This is a read-only collection, modify the NutritionDictionary instead!");
				}

				public bool Contains(double item){
					foreach(double val in this){
						if(val == item)
							return true;
					}
					return false;
				}

				public void CopyTo(double[] array, int arrayIndex){
					if(arrayIndex < 0)
						throw new ArgumentOutOfRangeException("arrayIndex", "arrayIndex cannot be less than 0");
					foreach(double val in this)
						array[arrayIndex++] = val;
				}

				public bool Remove(double item){
					throw new NotSupportedException("This is a read-only collection, modify the NutritionDictionary instead!");
				}

				public int Count{
					get { return this.owner.Count; }
				}

				public bool IsReadOnly{
					get { return true; }
				}
				#endregion
			}
			#endregion

			public ICollection<double> Values{
				get { return new NutritionValueCollection(this); }
			}
			#endregion

			#region IEnumerable<KeyValuePair<Nutrient, double>> implementation
			public IEnumerator<KeyValuePair<Nutrient, double>> GetEnumerator(){
				foreach(KeyValuePair<long, NutritionalValue> pair in this.cachedValues)
					yield return new KeyValuePair<Nutrient, double>(Nutrient.CreateNutritionDefinition(pair.Key), pair.Value.Value);
			}
			#endregion

            #region IEnumerable implementation
            System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
            {
                return this.GetEnumerator();
            }
            #endregion

			#region ICollection<KeyValuePair<Nutrient, double>> implementation
			public void Add(KeyValuePair<Nutrient, double> item){
				this.SetValue(item.Key.Id, item.Value);
			}

			public void Clear(){
				List<long> ids = new List<long>(this.cachedValues.Keys);
				foreach(var id in ids)
					this.Remove(Nutrient.CreateNutritionDefinition(id));
				ids = new List<long>(this.changedValues.Keys);
				foreach(var id in ids)
					this.Remove(Nutrient.CreateNutritionDefinition(id));
			}

			public bool Contains(KeyValuePair<Nutrient, double> item){
				double val;
				return this.TryGetValue(item.Key, out val) && val == item.Value;
			}

			public void CopyTo(KeyValuePair<Nutrient, double>[] array, int arrayIndex){
				if(arrayIndex < 0)
					throw new ArgumentOutOfRangeException("arrayIndex", "arrayIndex cannot be less than 0");
				foreach(KeyValuePair<Nutrient, double> pair in this)
					array[arrayIndex++] = pair;
			}

			public bool Remove(KeyValuePair<Nutrient, double> item){
				NutritionalValue val = null;
				if(this.cachedValues.TryGetValue(item.Key.Id, out val)){
					if(val.Value == item.Value)
						return this.Remove(item.Key);
				}
				if(this.changedValues.TryGetValue(item.Key.Id, out val)){
					if(val.Value == item.Value)
						return this.Remove(item.Key);
				}
				return false;
			}


			public int Count{
				get { return this.cachedValues.Count + this.changedValues.Count; }
			}

			public bool IsReadOnly{
				get { return false; }
			}
			#endregion
		}
	}
}
