using System;
using System.Collections.Generic;
using System.Text;
using Foodolini.Database;

namespace Foodolini.BusinessLogic
{
    public partial class Ingredient
    {
        private FoodDescription row;
        private NutritionDictionary nutrients;
        private bool modified = true;

        internal Ingredient(long foodDescriptionId)
        {
            this.row = Settings.Repo.SingleWhere<FoodDescription>("FoodDescriptionId = " + foodDescriptionId);
            if (this.row == null)
                throw new Exception("Ingredient not found in the database!");
            this.modified = false;
        }

        public Ingredient() { }

        internal Ingredient(FoodDescription row)
        {
            this.row = row;
            this.modified = row.FoodDescriptionId == 0;
        }

        public Ingredient(String name)
        {
            this.row = new FoodDescription();
            this.row.LongDescription = name;
            this.modified = true;
        }

        /// <summary>
        /// Nutridents that this ingredient contains
        /// </summary>
        public IDictionary<Nutrient, double> Nutrients
        {
            get
            {
                if (this.nutrients == null)
                    this.nutrients = new NutritionDictionary(this.Id);
                return this.nutrients;
            }
        }
        /* Don't make a setter here, the dictionary is mutable, and when saved it will not Update/Delete any 
             * other NutritionalValue associations than the ones that were in the database at the time it was created
             * and associated with the FoodDescription row it was created for.
             * Thus:
             * Ingredient A, B;
             * ...
             * A.Save();
             * B.Save();
             * B.Nutrients = A.Nutrients;
             * B.Nutrients.Remove(...);
             * B.Save(); //Won't remove the association in database
             * 
             * This B.Nutrients = A.Nutrients should be impossible!
             */

        /// <summary>
        /// If of the FoodDescription row
        /// </summary>
        internal long Id
        {
            get { return this.row.FoodDescriptionId; }
        }
        /* Setting to an, in database existing, FoodDescription will likely cause bugs.
             * See NutritionDictionary or Nutrients
             * */

        /// <summary>
        /// Commercial name.
        /// </summary>
        public string CommercialName
        {
            get { return this.row.CommercialName; }
            set
            {
                this.modified = true;
                this.row.CommercialName = value;
            }
        }

        /// <summary>
        /// Get/set short description.
        /// </summary>
        public string ShortDescription
        {
            get { return this.row.ShortDescription; }
            set
            {
                this.modified = true;
                this.row.ShortDescription = value;
            }
        }

        /// <summary>
        /// get/set long description.
        /// </summary>
        public string LongDescription
        {
            get { return this.row.LongDescription; }
            set
            {
                this.modified = true;
                this.row.LongDescription = value;
            }
        }

        /// <summary>
        /// Gets/sets the product manufacturer.
        /// </summary>
        public string Manufacturer
        {
            get { return this.row.Manufacturer; }
            set
            {
                this.modified = true;
                this.row.Manufacturer = value;
            }
        }

        /// <summary>
        /// Expected shelf life, TimeSpan.Zero if unknown.
        /// </summary>
        public TimeSpan ShelfLife
        {
            get { return new TimeSpan(this.row.ShelfLife.GetValueOrDefault(TimeSpan.Zero.Ticks)); }
            set
            {
                this.modified = true;
                this.row.ShelfLife = value.Ticks;
            }
        }

        /// <summary>
        /// Expected expiration after opening, TimeSpan.MaxValue if none is available.
        /// </summary>
        public TimeSpan ExpirationAfterOpening
        {
            get { return new TimeSpan(this.row.ExpirationAfterOpening.GetValueOrDefault(new TimeSpan(1,0,0,0).Ticks)); }
            set
            {
                this.modified = true;
                if (value == TimeSpan.MaxValue)
                    this.row.ExpirationAfterOpening = null;
                else
                    this.row.ExpirationAfterOpening = value.Ticks;
            }
        }

        /// <summary>
        /// Gets/sets the Category, string.Empty if none.
        /// </summary>
        /// <remarks></remarks>
        public string Category
        {
            get
            {
                LoadFoodGroups();
                string val;
                if (this.row.FoodGroupId.HasValue && foodGroups.TryGetValue(this.row.FoodGroupId.Value, out val))
                    return val;
                return string.Empty;
            }
            set
            {
                this.modified = true;
                this.row.FoodGroupId = GetFoodGroupId(value);
            }
        }

        /// <summary>
        /// The amount of protein in the ingredient.
        /// </summary>
        public double Protein
        {
            get { return this.row.Protein.GetValueOrDefault(); }
        }

        /// <summary>
        /// The amount of carbohydrates in the ingredient.
        /// </summary>
        public double Carbohydrates
        {
            get { return this.row.Carbohydrates.GetValueOrDefault(); }
        }

        /// <summary>
        /// The amount of fat in the ingredient.
        /// </summary>
        public double Fat
        {
            get { return this.row.Fat.GetValueOrDefault(); }
        }

        /// <summary>
        /// Get a FoodGroup id for a category
        /// </summary>
        /// <remarks>Throw an exception if the category doesn't exist.</remarks>
        private static long? GetFoodGroupId(string category)
        {
            long id = 0;
            LoadFoodGroups();
            foreach (var pair in foodGroups) {
                if (pair.Value == category)
                    id = pair.Key;
            }
            if (id == 0 && category == string.Empty)
                return null;
            else if (id == 0)
                throw new Exception("The FoodGroup '" + category + "' doesn't exist!");
            else
                return id;
        }

        /// <summary>
        /// List Ingredients by category
        /// </summary>
        /// <param name="category">
        /// A <see cref="System.String"/> category to list
        /// </param>
        public static IEnumerable<Ingredient> ListByCategory(string category)
        {
            foreach (var fd in Settings.Repo.Where<FoodDescription>("FoodGroupId = @0", GetFoodGroupId(category)))
                yield return new Ingredient(fd);
        }

        /// <summary>
        /// List Ingredients by category and search string
        /// </summary>
        /// <param name="category">
        /// A <see cref="System.String"/> category to list
        /// </param>
        /// <param name="searchString">
        /// A <see cref="System.String"/> to search for in LongDescription
        /// </param>
        /// <returns>
        /// A <see cref="IEnumerable<Ingredient>"/>
        /// </returns>
        public static IEnumerable<Ingredient> ListByCategory(string category, string searchString)
        {
            foreach (var fd in Settings.Repo.Where<FoodDescription>("FoodGroupId = @0 AND LongDescription LIKE @1", GetFoodGroupId(category), "%" + searchString + "%"))
                yield return new Ingredient(fd);
        }
		
		/// <summary>
        /// List Ingredients by search string
        /// </summary>
        /// <param name="searchString">
        /// A <see cref="System.String"/> to search for in LongDescription
        /// </param>
        /// <returns>
        /// A <see cref="IEnumerable<Ingredient>"/>
        /// </returns>
        public static IEnumerable<Ingredient> ListByLongDescription(string searchString)
        {
            foreach (var fd in Settings.Repo.Where<FoodDescription>("LongDescription LIKE @0 limit 500", "%" + searchString + "%"))
                yield return new Ingredient(fd);
        }

        /// <summary>
        /// Loads a single Ingredient by LongDescription.
        /// </summary>
        /// <param name="description"></param>
        /// <returns></returns>
        public static Ingredient GetByLongDescription(string description)
        {
            FoodDescription row = Settings.Repo.SingleWhere<FoodDescription>("longdescription = @0", description);
			if (row != null)
				return new Ingredient(row);
			return null;
        }

        /// <summary>
        /// Loads a single Ingredient by id.
        /// </summary>
        /// <param name="ingredientId"></param>
        /// <returns></returns>
        internal static Ingredient GetById(long ingredientId)
        {
            return new Ingredient(Settings.Repo.SingleWhere<FoodDescription>(" FoodDescriptionId = @0 ", ingredientId));
        }

        /// <summary>
        /// Save this to the database
        /// </summary>
        public void Save()
        {
            //Save or update the FoodDescription row

            if (this.nutrients != null) {
                // Ensure columns on ingredient match the nutrients
                List<Nutrient> definitions = new List<Nutrient>(Nutrient.ListNutritionDefinitions());

                if (this.nutrients.ContainsKey(Nutrient.Protein))
                    this.row.Protein = this.nutrients[Nutrient.Protein];

                if (this.nutrients.ContainsKey(Nutrient.Carbohydrates))
                    this.row.Carbohydrates = this.nutrients[Nutrient.Carbohydrates];

                if (this.nutrients.ContainsKey(Nutrient.Fat))
                    this.row.Fat = this.nutrients[Nutrient.Fat];
            }

            if (this.row.FoodDescriptionId == 0)
                Settings.Repo.Add<FoodDescription>(this.row);
            else {
                if (this.modified)
                    Settings.Repo.Update<FoodDescription>(this.row);
            }

            //TODO: Save the associations
            if (this.nutrients != null)
                this.nutrients.Save(this.row.FoodDescriptionId);

            this.modified = false;
        }

        /// <summary>
        /// Delete this from database
        /// </summary>
        public void Delete()
        {
            long rowid = this.row.FoodDescriptionId;
            if (this.row.FoodDescriptionId != 0) {
                if (this.nutrients != null)
                    this.nutrients.Delete(rowid);
                Settings.Repo.Delete<FoodDescription>(this.row);
            }
            this.row.FoodDescriptionId = 0;
        }

        /// <summary>
        /// Lazily loaded list of FoodGroups
        /// </summary>
        private static Dictionary<long, string> foodGroups = null;

        /// <summary>
        /// Get a list of all valid FoodGroups
        /// </summary>
        public static ICollection<string> Categories
        {
            get
            {
                LoadFoodGroups();
                return foodGroups.Values;
            }
        }
		
		/// <summary>
		/// Returns the fooditem if ingredient is in storage, else returns null
		/// </summary>
		public FoodItem InStorage()
		{
			FoodItem foodItem = null;
			foreach(FoodItem item in FoodItem.ListFoodItems())
			{
				if(Ingredient.Compare(this, item.Ingredient))
				{
					if(foodItem == null)
						foodItem = item;
					else if(DateTime.Compare(foodItem.ExpirationDate, item.ExpirationDate) > 0)
						foodItem = item;
				}
			}
			
			return foodItem;
		}

        /// <summary>
        /// Checks if two ingredients have the same ID.
        /// </summary>
        /// <param name="ingredient1"></param>
        /// <param name="ingredient2"></param>
        /// <returns></returns>
        public static bool Compare(Ingredient ingredient1, Ingredient ingredient2)
        {
            return ingredient1.Id.Equals(ingredient2.Id);
        }

        public override string ToString()
        {
            return this.Id.ToString();
        }

        /// <summary>
        /// Loads foodGroups from database if not loaded.
        /// </summary>
        private static void LoadFoodGroups()
        {
            if (foodGroups == null) {
                foodGroups = new Dictionary<long, string>();
                foreach (var fg in Settings.Repo.All<FoodGroup>())
                    foodGroups.Add(fg.FoodGroupId, fg.Description);
            }
        }
    }
}
