using System;
using System.Collections.Generic;
using System.Text;
using Foodolini.Database;

namespace Foodolini.BusinessLogic
{
	public class Person
	{
		
		/// <summary>
		/// The underlying row 
		/// </summary>
		private User row;
		
		/// <summary>
		/// True, if this object have been modified with respect to the database
		/// </summary>
		private bool modified = true;

		/// <summary>
		/// Create Person from underlying row 
		/// </summary>
		/// <param name="user">
		/// Underlying row
		/// </param>
		private Person(User user){
			this.row = user;
			this.modified = user.UserId == 0;
		}
		
		/// <summary>
		/// Create a new person
		/// </summary>
		public Person(string userName, 
		              string fullName, 
		              DateTime birthDay, 
		              Gender gender, 
		              double height, 
		              double weight, 
		              double activityFactor){
			this.row = new User();
			this.row.UserName = userName;
			this.row.FullName = fullName;
			this.row.BirthDate = birthDay;
			this.row.DietId = null;
			this.row.Gender = (int)gender;
			this.row.Height = height;
			this.row.Weight = weight;
			this.row.ActivityFactor = activityFactor;
			this.modified = true;
			
		}
		
		/// <summary>
		/// Gets the primary key of the underlying row, 0 if not saved 
		/// </summary>
		internal long Id{
			get{
				return this.row.UserId;
			}
		}

		/// <summary>
		/// Gets the username of this Person 
		/// </summary>
		public string UserName {
			get { return this.row.UserName; }
			set { 
				this.modified = true;
				this.row.UserName = value; 
			}
		}

		/// <summary>
		/// Gets/sets the fullname of this person 
		/// </summary>
		public string FullName {
			get { return this.row.FullName; }
			set { 
				this.modified = true;
				this.row.FullName = value; 
			}
		}

		/// <summary>
		/// Gets/sets the birthdate of this person 
		/// </summary>
		public DateTime BirthDate {
			get {
				return this.row.BirthDate;
			}
			set {
				this.modified = true;
				this.row.BirthDate = value;
			}
		}

		/// <summary>
		/// Gets/sets the gender of this person 
		/// </summary>
		public Gender Gender {
			get {
				return (Gender)this.row.Gender;
			}
			set {
				this.modified = true;
				this.row.Gender = (int)value;
			}
		}

		/// <summary>
		/// Gets/sets the weight of this person 
		/// </summary>
		public double Weight {
			get {
				return this.row.Weight;
			}
			set {
				this.modified = true;
				this.row.Weight = value;
			}
		}

		/// <summary>
		/// Gets/sets the height of this person 
		/// </summary>
		public double Height {
			get {
				return this.row.Height;
			}
			set {
				this.modified = true;
				this.row.Height = value;
			}
		}

		/// <summary>
		/// Lazily loaded diet cache 
		/// </summary>
		private Diet diet = null;
		
		/// <summary>
		/// Gets/sets the diet of this person 
		/// </summary>
		public Diet Diet {
			get {
				if(this.diet == null){
					this.diet = this.row.DietId != null ? new Diet(this.row.DietId.Value) : Diet.DefaultDiet();
				}
				return this.diet;
			}
			set {
				this.diet = value;
			}
		}

        /// <summary>
        /// Activity factor for a person's BMR.
        /// </summary>
        /// <remarks>
        /// <list type="table">
        /// <listheader>
        ///     <term>Activity factor</term>
        ///     <description>Activity level</description>
        /// </listheader>
        /// <item><term>1.2</term><description>Little or no exercise. Sedentary</description></item>
        /// <item><term>1.375</term><description>Light exercise/sports 1-3 days/week. Light</description></item>
        /// <item><term>1.55</term><description>Moderate exercise/sport 3-5 days/week. Moderate</description></item>
        /// <item><term>1.725</term><description>Hard exercise/sport 6-7 days/week. Hard</description></item>
        /// <item><term>1.9</term><description>Very hard exercise/sport & physical job or 2x training) Extra</description></item>
        /// </list>
        /// </remarks>
        public double ActivityFactor
        {
            get
            {
                return this.row.ActivityFactor;
            }
            set
            {
                this.row.ActivityFactor = value;
                modified = true;
            }
        }

		/// <summary>
		/// Gets the age of the person in years.
		/// </summary>
		public int Age {
			get { return (DateTime.Now - BirthDate).Days / 365; }
		}

		/// <summary>
		/// Gets the Basal Metabolic Rate of the Person using the Mifflin St. Jeor BMR formula.
		/// </summary>
		public int BMR {
			get {
				// Mifflin St. Jeor.
				if (Gender == Gender.Male)
					return (int)(10 * Weight + 6.25 * Height - 5 * Age + 5);
				else
					return (int)(10 * Weight + 6.25 * Height - 5 * Age - 161);
			}
		}

		/// <summary>
        /// The energy resting metabolic rate per hour.
		/// </summary>
		public int METS {
			get { return (int)((this.BMR * 1.2) / 24); }
		}

		/// <summary>
		/// Get all FoodItems consumed by this person
		/// </summary>
		/// <returns>
		/// A <see cref="IEnumerable"/> that iterates over all FoodItems consumed by this person
		/// </returns>
		public IEnumerable<FoodItem> GetConsumedFoodItems(){
			var query = Settings.Repo.Where<FoodItemRow>("ConsumedBy = @0", this.Id);
			foreach(var item in query)
				yield return new FoodItem(item);
		}
		
		/// <summary>
		/// Get all FoodItems consumed by this person between start and end 
		/// </summary>
		/// <param name="start">
		/// Look for FoodItems consumed after this <see cref="DateTime"/>
		/// </param>
		/// <param name="end">
		/// Look for FoodItems consumed before this <see cref="DateTime"/>
		/// </param>
		/// <returns>
		/// An <see cref="IEnumerable<FoodItem>"/> that iterates over all FoodItems consumed by this person between start and end.
		/// </returns>
		public IEnumerable<FoodItem> GetConsumedFoodItems(DateTime start, DateTime end){
			var query = Settings.Repo.Where<FoodItemRow>("ConsumedBy = @0 AND ConsumedDate > @1 AND ConsumedDate < @2", this.Id, start, end);
			foreach(var row in query)
				yield return new FoodItem(row);
		}

		public IEnumerable<Exercise> GetPerformedExercises (){
			var query = Settings.Repo.Where<ExerciseRow>("userid = @0", this.Id);
			foreach(var row in query)
				yield return new Exercise(row);
		
		}

		/// <summary>
		/// Save this object to database 
		/// </summary>
		public void Save(){
			//If the diet have been loaded, save it
			if(this.diet != null){
				this.diet.Save();
				//If the diet have changed, mark this as modified
				if(this.row.DietId != this.diet.DietId){
					this.row.DietId = this.diet.DietId;
					this.modified = true;
				}
			}
			if(this.row.UserId == 0){
				LoadCache();
				Settings.Repo.Add<User>(this.row);
				cachedPersons.Add(this.row.UserId, this);
			}else if(this.modified)
				Settings.Repo.Update<User>(this.row);
			this.modified = false;
		}
		
		/// <summary>
		/// Delete this person from the database 
		/// </summary>
		public void Delete(){
			if(this.row.UserId != 0){
				LoadCache();
				cachedPersons.Remove(this.row.UserId);
				Settings.Repo.Delete<User>(this.row);
			}
			this.modified = true;
		}
		
		/// <summary>
		/// Cached persons 
		/// </summary>
		private static Dictionary<long, Person> cachedPersons = null;
		
		/// <summary>
		/// Cache all persons 
		/// </summary>
		private static void LoadCache(){
			if(cachedPersons == null){
				cachedPersons = new Dictionary<long, Person>();
				foreach(var user in Settings.Repo.All<User>())
					cachedPersons.Add(user.UserId, new Person(user));
			}
		}
		
		/// <summary>
		/// Get a person by primary key of the underlying row 
		/// </summary>
		internal static Person GetById(long userId){
			LoadCache();
			
			if(!cachedPersons.ContainsKey(userId))
				throw new Exception("Person does not exists!!");
			return cachedPersons[userId];
		}

		/// <summary>
		/// List all persons 
		/// </summary>
		public static IEnumerable<Person> ListUsers (){
			//Note: we're using IEnumerable in order to make it possible to maintain API compability
			//		Even though we might remove the user cache...
			LoadCache();
			foreach(var p in cachedPersons.Values)
				yield return p;
		}
	}
}
