using System;
using System.Collections.Generic;
using System.Text;
using Foodolini.Database;
using System.Diagnostics;

namespace Foodolini.BusinessLogic
{
	public partial class Recipe
	{
		private class RatingDictionary : IDictionary<Person, double>{

			private Dictionary<long, Rating> cachedRatings = new Dictionary<long, Rating>();
			private Dictionary<long, Rating> changedRatings = new Dictionary<long, Rating>();
			private Dictionary<long, Rating> deletedRatings = new Dictionary<long, Rating>();
			
			/// <summary>
			/// Create a RatingDictionary for a specific recipe
			/// </summary>
			/// <param name="recipeId">
			/// RecipeRowId to create for, 0 for an unsaved recipe
			/// </param>
			public RatingDictionary(long recipeId){
				if(recipeId != 0){
					var query = Settings.Repo.Where<Rating>("RecipeId = @0", recipeId);
					foreach(var rating in query)
						this.cachedRatings.Add(rating.UserId, rating);
				}
			}
			
			/// <summary>
			/// Set a rating from a person
			/// </summary>
			/// <remarks>This will overwrite existing ratings if any, or create a new one.</remarks>
			/// <param name="person">
			/// Person Id who assigned this rating
			/// </param>
			/// <param name="rating">
			/// Rating to set
			/// </param>
			private void setValue(long person, double rating){
				Rating val = null;
				//If this a change, do it and mark it changed
				if(this.cachedRatings.TryGetValue(person, out val)){
					val.Score = rating;
					this.cachedRatings.Remove(person);
					this.changedRatings.Add(person, val);
				//If this is a change on an already changed
				}else if(this.changedRatings.TryGetValue(person, out val)){
					val.Score = rating;
				//If this i a change on something that was deleted
				}else if(this.deletedRatings.TryGetValue(person, out val)){
					val.Score = rating;
					this.deletedRatings.Remove(person);
					this.changedRatings.Add(person, val);
				//If this is a new rating
				}else{
					val = new Rating();
					val.Score = rating;
					val.UserId = person;
					val.RecipeId = 0;
					this.changedRatings.Add(person, val);
				}
			}
			
			/// <summary>
			/// Save these relations to the database
			/// </summary>
			/// <param name="recipeId">
			/// Must be the same as the one it was created for, unless it was create with 0
			/// </param>
			public void Save(long recipeId){
				//Check if any of these are not saved
				foreach(Rating rating in this.cachedRatings.Values){
					if(rating.RecipeId != recipeId){
						Debug.Fail("This RatingDictionary was created for another recipe!");	
						rating.RecipeId = recipeId;
						Settings.Repo.Add<Rating>(rating);
					}
				}
				//Add/Update changed ratings
				foreach(Rating rating in this.changedRatings.Values){
					//If it was loaded and then edited, and is associated with the recipeId we're saving for, Update it
					if(rating.RatingId != 0 && rating.RecipeId == recipeId){
						Settings.Repo.Update<Rating>(rating);
					}else{
						Debug.Assert(rating.RatingId == 0, "This RatingDictionary was created for another recipe!");
						//Either this is a new Rating or this Rating is associated with another Recipe
						//So we can't use the existing one and need to create a new one.
						rating.RecipeId = recipeId;
						Settings.Repo.Add<Rating>(rating);
					}
					//Once saved add it to cachedRatings
					this.cachedRatings.Add(rating.UserId, rating);
				}
				this.changedRatings.Clear();
				//Delete ratings for deletion
				foreach(Rating rating in this.deletedRatings.Values){
					//Delete it if it's associated with this recipeId and not new
					if(rating.RatingId != 0 && rating.RecipeId == recipeId)
						Settings.Repo.Delete<Rating>(rating);
					else
						Debug.Assert(rating.RecipeId == recipeId, "This RatingDictionary was created for another recipe!");
				}
				//Clear deletedRatings when we've deleted them
				this.deletedRatings.Clear();
			}
			
			/// <summary>
			/// Delete the assocations this dictionary represent from the database
			/// </summary>
			public void Delete(long recipeId){
				//If the Id is the same as they were loaded for, we must delete it
				foreach(Rating rating in this.cachedRatings.Values){
					if(rating.RecipeId == recipeId){
						Settings.Repo.Delete<Rating>(rating);
						rating.RatingId = 0;
					}else
						Debug.Assert(rating.RecipeId == 0, "This RatingDictionary was created for another recipe!");
					this.changedRatings.Add(rating.UserId, rating);
				}
				//Nothing can be cached anymore
				this.cachedRatings.Clear();
				//If the Id is the same as they were loaded for, we must delete it
				foreach(Rating rating in this.changedRatings.Values){
					if(rating.RatingId != 0 && rating.RecipeId == recipeId){
						Settings.Repo.Delete<Rating>(rating);
						rating.RatingId = 0;
					}else
						Debug.Assert(rating.RecipeId == 0, "This RatingDictionary was created for another recipe!");
				}
				//Delete ratings that have been deleted
				foreach(Rating rating in this.deletedRatings.Values){
					if(rating.RatingId != 0 && rating.RecipeId == recipeId)
						Settings.Repo.Delete<Rating>(rating);
					else
						Debug.Assert(rating.RecipeId == 0, "This RatingDictionary was created for another recipe!");
				}
				//Clear deletedRatings as we've deleted them
				this.deletedRatings.Clear();
			}
			
			#region IDictionary<Person, double> implementation
			public void Add (Person key, double value){
				this.setValue(key.Id, value);
			}
			
			public bool ContainsKey (Person key){
				return this.cachedRatings.ContainsKey(key.Id) || this.changedRatings.ContainsKey(key.Id);
			}
			
			public bool Remove (Person key){
				Rating val = null;
				if(this.cachedRatings.TryGetValue(key.Id, out val)){
					//Don't mark for deletion if not in db
					if(val.RatingId != 0)
						this.deletedRatings.Add(key.Id, val);
					this.cachedRatings.Remove(key.Id);
					return true;
				}
				if(this.changedRatings.TryGetValue(key.Id, out val)){
					//Don't mark for deletion if not in db
					if(val.RatingId != 0)
						this.deletedRatings.Add(key.Id, val);
					this.changedRatings.Remove(key.Id);
					return true;
				}
				return false;
			}
			
			public bool TryGetValue (Person key, out double value){
				Rating val = null;
				if(this.cachedRatings.TryGetValue(key.Id, out val)){
					value = val.Score;
					return true;
				}
				if(this.changedRatings.TryGetValue(key.Id, out val)){
					value = val.Score;
					return true;
				}
				value = new double();
				return false;
			}

			
			public double this[Person key] {
				get {
					double val;
					if(this.TryGetValue(key, out val))
						return val;
					throw new KeyNotFoundException(key.FullName + " was not found in the dictionary.");
				}
				set {
					this.setValue(key.Id, value);
				}
			}
			
			#region PersonCollection
			
			private class PersonCollection : ICollection<Person>{
				private RatingDictionary owner;
				
				public PersonCollection(RatingDictionary owner){
					this.owner = owner;
				}
				
				#region IEnumerable implementation
				System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator (){
					return this.GetEnumerator();
				}
				#endregion

				#region IEnumerable<Person> implementation
				public IEnumerator<Person> GetEnumerator (){
					foreach(var p in this.owner.cachedRatings.Keys)
						yield return Person.GetById(p);
					foreach(var p in this.owner.changedRatings.Keys)
						yield return Person.GetById(p);
				}
				#endregion

				#region ICollection<Person> implementation
				public void Add (Person item){
					throw new NotSupportedException("This is a read-only collection, modify the RatingDictionary instead!");
				}
				
				public void Clear (){
					throw new NotSupportedException("This is a read-only collection, modify the RatingDictionary instead!");
				}
				
				public bool Contains (Person item){
					return this.owner.ContainsKey(item);
				}
				
				public void CopyTo (Person[] array, int arrayIndex){
					if(arrayIndex < 0)
						throw new ArgumentOutOfRangeException("arrayIndex", "arrayIndex cannot be less than 0");
					foreach(Person val in this)
						array[arrayIndex++] = val;
				}
				
				public bool Remove (Person item){
					throw new NotSupportedException("This is a read-only collection, modify the RatingDictionary instead!");
				}
				
				public int Count {
					get {
						return this.owner.Count;
					}
				}
				
				public bool IsReadOnly {
					get {
						return true;
					}
				}
				#endregion
			}	
			#endregion
				
			public ICollection<Person> Keys {
				get {
					return new PersonCollection(this);
				}
			}
			
			#region RatingCollection
			
			private class RatingCollection : ICollection<double>{
				private RatingDictionary owner;
				
				public RatingCollection(RatingDictionary owner){
					this.owner = owner;
				}
				
				#region IEnumerable implementation
				System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator (){
					return this.GetEnumerator();
				}
				#endregion

				#region IEnumerable<double> implementation
				public IEnumerator<double> GetEnumerator (){
					foreach(var v in this.owner.cachedRatings.Values)
						yield return v.Score;
					foreach(var v in this.owner.changedRatings.Values)
						yield return v.Score;
				}
				#endregion

				#region ICollection<double> implementation
				public void Add (double item){
					throw new NotSupportedException("This is a read-only collection, modify the RatingDictionary instead!");
				}
				
				public void Clear (){
					throw new NotSupportedException("This is a read-only collection, modify the RatingDictionary instead!");
				}
				
				public bool Contains (double item){
					foreach(double d in this){
						if(d == item)
							return true;
					}
					return false;
				}
				
				public void CopyTo (double[] array, int arrayIndex){
					if(arrayIndex < 0)
						throw new ArgumentOutOfRangeException("arrayIndex", "arrayIndex cannot be less than 0");
					foreach(double val in this)
						array[arrayIndex++] = val;
				}
				
				public bool Remove (double item){
					throw new NotSupportedException("This is a read-only collection, modify the RatingDictionary instead!");
				}
				
				public int Count {
					get {
						return this.owner.Count;
					}
				}
				
				public bool IsReadOnly {
					get {
						return true;
					}
				}
				#endregion
			}	
			#endregion
			
			public ICollection<double> Values {
				get {
					return new RatingCollection(this);
				}
			}
			#endregion

			#region IEnumerable<KeyValuePair<Person, double>> implementation
			public IEnumerator<KeyValuePair<Person, double>> GetEnumerator (){
				foreach(var entry in this.cachedRatings)
					yield return new KeyValuePair<Person,double>(Person.GetById(entry.Key), entry.Value.Score);
				foreach(var entry in this.changedRatings)
					yield return new KeyValuePair<Person,double>(Person.GetById(entry.Key), entry.Value.Score);
			}

			#endregion

			#region IEnumerable implementation
			System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator (){
				return this.GetEnumerator();
			}

			#endregion

			#region ICollection<KeyValuePair<Person, double>> implementation
			public void Add (KeyValuePair<Person, double> item){
				this.setValue(item.Key.Id, item.Value);
			}
			
			public void Clear (){
				List<long> ids = new List<long>(this.cachedRatings.Keys);
				foreach(var id in ids)
					this.Remove(Person.GetById(id));
				ids = new List<long>(this.changedRatings.Keys);
				foreach(var id in ids)
					this.Remove(Person.GetById(id));
			}
			
			public bool Contains (KeyValuePair<Person, double> item){
				double val;
				return this.TryGetValue(item.Key, out val) && val == item.Value;
			}
			
			public void CopyTo (KeyValuePair<Person, double>[] array, int arrayIndex){
				if(arrayIndex < 0)
					throw new ArgumentOutOfRangeException("arrayIndex", "arrayIndex cannot be less than 0");
				foreach(KeyValuePair<Person, double> pair in this)
					array[arrayIndex++] = pair;
			}
			
			public bool Remove (KeyValuePair<Person, double> item){
				Rating val = null;
				if(this.cachedRatings.TryGetValue(item.Key.Id, out val)){
					if(val.Score == item.Value)
						return this.Remove(item.Key);
				}
				if(this.changedRatings.TryGetValue(item.Key.Id, out val)){
					if(val.Score == item.Value)
						return this.Remove(item.Key);
				}
				return false;
			}
			
			public int Count {
				get {
					return this.cachedRatings.Count + this.changedRatings.Count;
				}
			}
			
			public bool IsReadOnly {
				get {
					return false;
				}
			}
			#endregion
			
		}
	}
}
