using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Foodolini.Database;

namespace Foodolini.BusinessLogic
{
    /// <summary>
    /// Represents a shopping list.
    /// </summary>
    public class ShoppingList
    {
        /// <summary>
        /// Owner of the shopping list.
        /// </summary>
        private Person owner;

        /// <summary>
        /// Instantiates a shopping list with the provided owner.
        /// </summary>
        /// <param name="person"></param>
        public ShoppingList(Person person)
        {
            this.owner = person;
            // Instantiate the internal lists
            GetShoppingListItems();
        }

        /// <summary>
        /// The items in the shopping list.
        /// </summary>
        private List<ShoppingListItem> shoppingList;

		/// <summary>
		/// Loads and returns the shopping list for the current user. 
		/// </summary>
		/// <returns>
		/// A <see cref="IEnumerable<ShoppingListItem>"/>
		/// </returns>
        public IEnumerable<ShoppingListItem> GetShoppingListItems()
        {
            // if it's empty we load it from the database.
            if (shoppingList == null) {
                shoppingList = new List<ShoppingListItem>(ShoppingListItem.GetShoppingListItems(this.owner));
            }
            return shoppingList;
        }

        /// <summary>
        /// Adds the given quantity of an ingredient to the shopping list.
        /// If the ingredient is already in the list, the quantity will be added.
        /// </summary>
        /// <param name="ingredient"></param>
        /// <param name="quantity"></param>
        public bool AddIngredient(Ingredient ingredient, double quantity)
        {
            bool added = false;
			var shoppingItem = this.shoppingList.Find(item => Ingredient.Compare(item.Ingredient,ingredient));
            if (shoppingItem == null) {
                added = true;
                this.shoppingList.Add(new ShoppingListItem(ingredient, quantity, this.owner));
            } else{
				shoppingItem.Quantity += quantity;
			}
            return added;
        }

        /// <summary>
        /// Adds a recipe's ingredient to the shopping list.
        /// </summary>
        /// <param name="recipe"></param>
        /// <param name="servings"></param>
        public void AddRecipe(Recipe recipe, double servings)
        {
			foreach (KeyValuePair<Ingredient, double> item in recipe.Ingredients)
				this.AddIngredient(item.Key, item.Value*servings);
        }

		/// <summary>
		/// Determines if the specified ingredient is already in the shopping list. 
		/// </summary>
		/// <param name="ingredient">
		/// A <see cref="Ingredient"/>
		/// </param>
		/// <returns>
		/// A <see cref="System.Boolean"/>
		/// </returns>
		public bool ContainsIngredient(Ingredient ingredient){
			return this.shoppingList.Find(item => Ingredient.Compare(item.Ingredient,ingredient)) != null;
		}
		
        /// <summary>
        /// Removes the ingredient from the shopping list.
        /// </summary>
        /// <param name="ingredient"></param>
        public void RemoveIngredient(Ingredient ingredient)
        {
            ShoppingListItem toRemove = this.shoppingList.Find(item => item.Ingredient == ingredient);
            RemoveItem(toRemove);
        }

        /// <summary>
        /// Removes a given item from the shopping list
        /// </summary>
        /// <param name="item"></param>
        private void RemoveItem(ShoppingListItem item)
        {
            if (item != null) {
                // Delete from database
                item.Delete();
                // Then remove from the list
                this.shoppingList.Remove(item);
            }
        }

        /// <summary>
        /// Removes the recipe from the shopping list.
        /// </summary>
        /// <param name="recipe"></param>
        public void RemoveRecipe(Recipe recipe)
        {
            // The shopping list may only contain one instance of each recipe.
            // Therefore, we can use the id to remove the recipe.
            ShoppingListItem toRemove = this.shoppingList.Find(item => item.Recipe.Id == recipe.Id);
            RemoveItem(toRemove);
        }

        /// <summary>
        /// Saves the shopping list to the database.
        /// </summary>
        public void Save()
        {
            // Save all items
            foreach (ShoppingListItem item in this.shoppingList)
                item.Save();
        }

        /// <summary>
        /// Deletes all entries in the shopping list from the database.
        /// </summary>
        public void Clear()
        {
            foreach (ShoppingListItem item in this.shoppingList)
                item.Delete();
            // Reset the private list.
            this.shoppingList = new List<ShoppingListItem>();
        }
    }
}
