using System;
using System.Collections.Generic;
using System.Text;
using Foodolini.Database;

namespace Foodolini.BusinessLogic
{
    /// <summary>
    /// A sports activity.
    /// </summary>
	public class SportsActivity
	{
		private bool modified = true;
		private SportsActivityRow row;
		
		/// <summary>
		/// Create a new unsaved SportActivity
		/// </summary>
		/// <param name="name">
		/// Name of the activity, human readable
		/// </param>
		/// <param name="METS">
		/// Energy expenditure of the sports activity.
		/// </param>
		public SportsActivity(string name, double METS){
			this.row = new SportsActivityRow();
			this.row.Name = name;
			this.row.METS = METS;
			this.modified = true;
		}
		
		/// <summary>
		/// Create a sports activity from it's underlying row id
		/// </summary>
		/// <param name="activityId">
		/// Row id
		/// </param>
		internal SportsActivity(long activityId){
            this.row = Settings.Repo.SingleWhere<SportsActivityRow>("SportsActivityRowId = @0", activityId);
			if(row == null)
				throw new Exception("The SportsActivity does not exist.");
			this.modified = false;
        }
		
		/// <summary>
		/// Create a sports activity from it's underlying row
		/// </summary>
		/// <param name="row">
		/// The underlying row
		/// </param>
		internal SportsActivity(SportsActivityRow row){
            this.row = row;
			this.modified = row.SportsActivityRowId != 0;
        }
		
		
		/// <summary>
		/// Primary key for the underlying row 
		/// </summary>
        internal long Id { 
			get{
				return this.row.SportsActivityRowId;
			}
		}

        /// <summary>
        /// The name of the sports activity.
        /// </summary>
		public string Name
		{
			get{
				return this.row.Name;
			}
			set{
				this.modified = true;
				this.row.Name = value;
			}
		}

        /// <summary>
        /// Energy expenditure of the sports activity.
        /// </summary>
		public double METS
		{
			get{
				return this.row.METS;
			}
			set{
				this.modified = true;
				this.row.METS = value;
			}
		}

        /// <summary>
        /// Saves changes to the database.
        /// </summary>
		public void Save(){
			if(this.row.SportsActivityRowId == 0)
				Settings.Repo.Add<SportsActivityRow>(this.row);
			else if(this.modified)
				Settings.Repo.Update<SportsActivityRow>(this.row);
			this.modified = false;
		}
		
		/// <summary>
        /// Permanently deletes the sports activity.
        /// </summary>
		public void Delete(){
			if(this.row.SportsActivityRowId != 0)
				Settings.Repo.Delete<SportsActivityRow>(this.row);
			this.modified = true;
		}

        /// <summary>
        /// Gets a list of all available sports activities.
        /// </summary>
        /// <returns></returns>
		public static IEnumerable<SportsActivity> ListActivities(){
			foreach(var act in Settings.Repo.All<SportsActivityRow>())
				yield return new SportsActivity(act);
		}
	}
}
