using System;
using System.Collections.Generic;
using Foodolini.Database;

/// <summary>
/// Logic layer, business logic component
/// </summary>
namespace Foodolini.BusinessLogic
{
	/// <summary>
	/// Singleton object that holds all settings
	/// </summary>
	public class Settings : IDisposable
	{
		private Repository repository = null;
		
		private Settings(){
		}
		
		/// <summary>
		/// Get the repository
		/// </summary>
		internal Repository Repository{
			get{
				if(this.repository == null)
					throw new ObjectDisposedException("Object has been disposed or you are not connected to a database.");
				return this.repository;
			}
		}
		
		/// <value>
		/// Get/set settings
		/// </value>
		public string this[string key]{
			get{
				var setting = this.repository.SingleWhere<Setting>("Key = @0", key);
				if(setting == null)
					return null;
				return setting.Value;
			}
			set{
				var setting = this.repository.SingleWhere<Setting>("Key = @0", key);
				if(setting == null){
					setting = new Setting();
					setting.Key = key;
					setting.Value = value;
					this.repository.Add(setting);
				}else{
					setting.Value = value;
					this.repository.Update(setting);
				}
			}
		}
		
		/// <summary>
		/// Get a setting, null if not set
		/// </summary>
		public string GetValue(string key){
			return this[key];
		}
		
		/// <summary>
		/// Get a setting, default if not set
		/// </summary>
		public string GetValue(string key, string @default){
			string retval = this[key];
			if(retval == null)
				return @default;
			return retval;
		}
		
		/// <summary>
		/// Save settings
		/// </summary>
		/// <remarks>This commits any changes to the database.</remarks>
		public void Save(){
			this.repository.Commit();
		}
		
		/// <summary>
		/// Open an Sqlite Database
		/// </summary>
		/// <param name="database">
		/// Path to Sqlite database
		/// </param>
		public void OpenSqliteDatabase(string database){
			if(this.repository != null)
				this.repository.Dispose();
			this.repository = Repository.ConnectSqlite(database);
		}
		
		/// <summary>
		/// Open an Sqlite Database
		/// </summary>
		/// <param name="database">
		/// Path to Sqlite database
		/// </param>
		/// <param name="disableCommit">
		/// If true transactions will never be commited
		/// </param>
		public void OpenSqliteDatabase(string database, bool disableCommit){
			if(this.repository != null)
				this.repository.Dispose();
			this.repository = Repository.ConnectSqlite(database, disableCommit);
		}
		
		private static Settings instance = null;
		public static Settings Instance{
			get{
				if(instance == null || instance.repository == null)
					instance = new Settings();
				return instance;
			}
		}
		
		/// <summary>
		/// A shortcut to Settings.Instance.Repository
		/// </summary>
		internal static Repository Repo{
			get{
				return Settings.Instance.Repository;
			}
		}
		
		#region IDisposable Implementation
		//This pattern for implementing IDisposable is recommended by:
		//Framework Design Guidelines, 2. Edition, Section 9.4
		
		/// <summary>
		/// Dispose this object
		/// </summary>
		/// <remarks>
		/// This boolean disposing parameter here ensures that objects with a finalizer is not disposed,
		/// this is method is invoked from the finalizer. Do overwrite, and call, this method in base 
		/// classes if you use any unmanaged resources.
		/// </remarks>
		/// <param name="disposing">
		/// A <see cref="System.Boolean"/> False if called from the finalizer, True if called from Dispose.
		/// </param>
		protected virtual void Dispose(bool disposing){
			if(disposing){
				//Release finalizable resources, at the moment none.
				if(this.repository != null)
					this.repository.Dispose();
				this.repository = null;
			}
		}
		
		/// <summary>
		/// Release resources held by this object
		/// </summary>
		public void Dispose(){
			//We're disposing this object and can release objects that are finalizable
			this.Dispose(true);
			GC.SuppressFinalize(this);
		}
		
		/// <summary>
		/// Finalize this object
		/// </summary>
		~Settings(){
			//Dispose this object, but do NOT release finalizable objects, we don't know in which order
			//these are release and they may already be finalized.
			this.Dispose(false);
		}
		#endregion
		
		/// <summary>
		/// Close the database connection
		/// </summary>
		/// <remarks>
		/// The same as dispose
		/// </remarks>
		public void Close(){
			this.Dispose();
		}
	}
}
