
using System;
using System.Collections.Generic;
using System.Reflection;
using System.Text;
using System.Data.Common;
using Foodolini.Database.Sql;

/// <summary>
/// Database layer, model and repository component
/// </summary>
namespace Foodolini.Database
{
	
	/// <summary>
	/// A repository can store simple classes
	/// </summary>
	/// <remarks>
	/// Types used with this model may not contain properties that are not both get and set.
	/// The properties must not be assumed case sensitive.
	/// </remarks>
	public class Repository : IDisposable
	{
		/// <summary>
		/// Database connection, this is remain open untill disposed/closed
		/// </summary>
		private DbConnection connection = null;
		
		/// <summary>
		/// SqlFactory for building sql statments
		/// </summary>
		private SqlStrategy builder;
		
		/// <summary>
		/// List of tables, so that we need not check before every operation
		/// </summary>
		private List<string> tables = new List<string>();
		
		/// <summary>
		/// Current transaction
		/// </summary>
		/// <remarks>
		/// Transactions is a requirement for Sqlite to offer proper performance.
		/// </remarks>
		private DbTransaction transaction = null;

		/// <summary>
		/// Creates an instance of Repository
		/// </summary>
		/// <param name="connection">
		/// A <see cref="DbConnection"/> to the database, the constructor will Open this connection, just initialize it.
		/// </param>
		/// <param name="builder">
		/// A <see cref="SqlFactory"/> for building sql statements for the database engine.
		/// </param>
		public Repository(DbConnection connection, SqlStrategy builder){
			this.connection = connection;
			this.builder = builder;
			this.connection.Open();
			this.transaction = this.connection.BeginTransaction();
			using(DbCommand cmd = this.connection.CreateCommand()){
				cmd.CommandText = this.builder.TableQuery();
				using(DbDataReader rdr = cmd.ExecuteReader()){
					if(rdr.HasRows){
						while(rdr.Read())
							this.tables.Add(rdr.GetString(0));
					}
				}
			}
		}
		
		/// <summary>
		/// Commit changes to the database
		/// </summary>
		/// <remarks>
		/// This method commits current transaction and starts a new transaction.
		/// Do not call this method too often, as it may cause performance issues.
		/// Invoke this method whenever the database variant is upheld.
		/// </remarks>
		public void Commit(){
			if(!this.disableCommit){
				this.transaction.Commit();
				this.transaction = this.connection.BeginTransaction();
			}
		}
		
		/// <summary>
		/// Rollback the database to last commit
		/// </summary>
		public void Rollback(){
			this.transaction.Rollback();
			this.transaction = this.connection.BeginTransaction();
		}
		
		/// <summary>
		/// Create a Repository connected to a database
		/// </summary>
		/// <param name="database">
		/// Path to the database, or :memory: for in-memory database
		/// </param>
		/// <returns>
		/// Repository for the database
		/// </returns>
		public static Repository ConnectSqlite(string database){
			#if MONO
			var conn = new Mono.Data.Sqlite.SqliteConnection("data source=" + database + ";Pooling=true;FailIfMissing=false");
			#else
			var conn = new System.Data.SQLite.SQLiteConnection("data source=" + database + ";Pooling=true;FailIfMissing=false");
			#endif
			return new Repository(conn, new Sqlite());
		}
		
		/// <summary>
		/// True, if commit should be disabled, used for testing
		/// </summary>
		private bool disableCommit = false;
		
		/// <summary>
		/// Create a Repository connected to a database
		/// </summary>
		/// <param name="database">
		/// Path to the database, or :memory: for in-memory database
		/// </param>
		/// <param name="disableCommit">
		/// Disable commit, used for testing
		/// </param>
		/// <returns>
		/// Repository for the database
		/// </returns>
		public static Repository ConnectSqlite(string database, bool disableCommit){
			Repository repo = ConnectSqlite(database);
			repo.disableCommit = disableCommit;
			return repo;
		}
		
		
		/// <summary>
		/// Ensures the existance of a table
		/// </summary>
		/// <remarks>
		/// This method does not check that the columns of the table exists and are correct.
		/// Just that a table with the name of the type + "s" exists.
		/// This method uses a catched list of tables, so it is not expensive.
		/// </remarks>
		private void EnsureTable<T>() where T : class, new(){
			//Check if connection have been disposed/closed
			if(this.connection == null)
				throw new ObjectDisposedException("connection", "The database connection have been closed/disposed.");
			if(!this.tables.Contains(this.TableName<T>()))
				this.CreateTable<T>();
		}
		
		/// <summary>
		/// Get table name of a type
		/// </summary>
		/// <returns>
		/// A <see cref="System.String"/> table name
		/// </returns>
		private string TableName<T>() where T : class, new(){
			return typeof(T).Name + "s";
		}
		
		/// <summary>
		/// Creates a table for type T
		/// </summary>
		/// <remarks>
		/// A property named classnamed + 'Id' will be used as primary key, should be a long.
		/// </remarks>
		public void CreateTable<T>() where T : class, new(){
			//Check if connection have been disposed/closed
			if(this.connection == null)
				throw new ObjectDisposedException("connection", "The database connection have been closed/disposed.");
			//Create a command
			using(DbCommand cmd = this.connection.CreateCommand()){
				//Find columns and their type
				Dictionary<string,Type> cols = new Dictionary<string, Type>();
				foreach(PropertyInfo property  in typeof(T).GetProperties()){
					if(string.Compare(property.Name, typeof(T).Name + "Id", true) == 0)
						continue;
					cols.Add(property.Name, property.PropertyType);
				}
				//Build sql statement
				cmd.CommandText = this.builder.CreateTableQuery(this.TableName<T>(), cols, typeof(T).Name + "Id");
				
				//Execute the sql statement
				cmd.ExecuteNonQuery();
				
				//Add to cached list of tables in the database
				this.tables.Add(this.TableName<T>());
			}
		}
		
		/// <summary>
		/// Add an instance of T to the database
		/// </summary>
		/// <remarks>
		/// Note that the items primary key will be updated to match that of the newly inserted row.
		/// </remarks>
		/// <param name="item">
		/// The item to add to the database
		/// </param>
		public void Add<T>(T item) where T : class, new(){
			//Ensure the table exists, so that we can operate on it, this also complains if connection is disposed
			this.EnsureTable<T>();
			//Create a command for the database
			using(DbCommand cmd = this.connection.CreateCommand()){
				//Make list of parameters and add parameters with the correct keywords to the DbCommand
				List<string> parameters = new List<string>();
				foreach(PropertyInfo property  in typeof(T).GetProperties()){
					if(string.Compare(property.Name, typeof(T).Name + "Id") != 0){
						parameters.Add(property.Name);
						var par = cmd.CreateParameter();
						par.ParameterName = "@" + property.Name;
						par.Value = property.GetValue(item, null);
						cmd.Parameters.Add(par);
					}
				}
				//Generate sql statement
				cmd.CommandText = this.builder.InsertQuery(this.TableName<T>(), parameters);
				
				//Execute the sql statement and get the primary key of the inserted row
				long id = (long)cmd.ExecuteScalar();
				
				//The the primary key on item
				foreach(PropertyInfo property  in typeof(T).GetProperties()){
					if(string.Compare(property.Name, typeof(T).Name + "Id") == 0)
						property.SetValue(item, id, null);
				}
			}
		}
		
		/// <summary>
		/// Update a row in the database
		/// </summary>
		/// <remarks>
		/// The row is found using the primary key, if the type has no primary key, this method will throw and exception.
		/// </remarks>
		/// <param name="item">
		/// Instance of T to update in the database
		/// </param>
		/// <returns>
		/// True, if one row was affected.
		/// </returns>
		public bool Update<T>(T item) where T : class, new(){
			//Ensure the table exists, so that we can operate on it, this also complains if connection is disposed
			this.EnsureTable<T>();
			//Create a command for the database
			using(DbCommand cmd = this.connection.CreateCommand()){
				//Make a list of parameters and get the primary key of item
				List<string> parameters = new List<string>();
				long id = 0;
				foreach(PropertyInfo property  in typeof(T).GetProperties()){
					if(string.Compare(property.Name, typeof(T).Name + "Id") == 0 && property.PropertyType == typeof(long)){
						id = (long)property.GetValue(item, null);
					}else{
						parameters.Add(property.Name);
						var par = cmd.CreateParameter();
						par.ParameterName = "@" + property.Name;
						par.Value = property.GetValue(item, null);
						cmd.Parameters.Add(par);
					}
				}
				//If primary id doesn't exists, throw an exception.
				if(id == 0)
					throw new Exception("Type must have an identifier");
				//Add primary key as parameter to the DbCommand
				var para = cmd.CreateParameter();
				para.ParameterName = "@" + typeof(T).Name + "Id";
				para.Value = id;
				cmd.Parameters.Add(para);
				
				//Generate the sql statement
				cmd.CommandText = this.builder.UpdateQuery(this.TableName<T>(), parameters, typeof(T).Name + "Id");
				
				//Execute the sql statement
				int retval = cmd.ExecuteNonQuery();
				
				//If more that one row was update alert the user
				if(retval > 1)
					throw new Exception("More than one row was updated!");
				
				//Return true if exactly one row was updated
				return retval == 1; //TODO: Figure out if this is good: If updating a row to hold the same data as it already has doesn't affect it may be this should be discarded
			}
		}
		
		/// <summary>
		/// Delete a row from the database
		/// </summary>
		/// <remarks>
		/// This method uses primary key to identify which row to delete. The primary key on item is set to 0, once deleted.
		/// </remarks>
		/// <param name="item">
		/// Item representing the row to be deleted
		/// </param>
		/// <returns>
		/// True, if exactly one row was deleted.
		/// </returns>
		public bool Delete<T>(T item) where T : class, new(){
			//Ensure the table exists, so that we can operate on it, this also complains if connection is disposed
			this.EnsureTable<T>();
			//Create a database command
			using(DbCommand cmd = this.connection.CreateCommand()){
				//Find the id
				long id = 0;
				foreach(PropertyInfo property in typeof(T).GetProperties()){
					if(string.Compare(property.Name, typeof(T).Name + "Id") == 0 && property.PropertyType == typeof(long)){
						id = (long)property.GetValue(item, null);
						//If read as 0, return false
						if(id == 0)
							return false;
						//Set it to 0
						property.SetValue(item, 0, null);
					}
				}
				//If primary id doesn't exists, throw an exception.
				if(id == 0)
					throw new Exception("Type must have an identifier");
				//Build sql statement
				cmd.CommandText = this.builder.DeleteQuery(this.TableName<T>(), typeof(T).Name + "Id");
				//Add primary key as parameter to the DbCommand
				var para = cmd.CreateParameter();
				para.ParameterName = "@" + typeof(T).Name + "Id";
				para.Value = id;
				cmd.Parameters.Add(para);
				
				//Execute sql statement
				int retval = cmd.ExecuteNonQuery();
				
				//Alert that we've deleted more than one row
				if(retval > 1)
					throw new Exception("More than one row was deleted in the database");
				
				//Return true if one row was affected
				return retval == 1;
			}
		}
		
		/// <summary>
		/// Deletes all rows where the condition is satisfied
		/// </summary>
		/// <param name="condition">
		/// Condition, @0 will be replaced with the first parameter
		/// </param>
		/// <param name="parameters">
		/// Parameters to the condition, inserted into the condition with @0, @1 and so on.
		/// </param>
		/// <returns>
		/// Number of rows deleted
		/// </returns>
		public int DeleteWhere<T>(string condition, params object[] parameters) where T : class, new(){
			//Ensure the table exists, so that we can operate on it, this also complains if connection is disposed
			this.EnsureTable<T>();
			//Create a database command
			using(DbCommand cmd = this.connection.CreateCommand()){
				cmd.CommandText = this.builder.DeleteWhereQuery(this.TableName<T>(), condition);
				int pCount = 0;
				foreach(object param in parameters){
					var p = cmd.CreateParameter();
					p.ParameterName = "@" + pCount++;
					p.Value = param;
					cmd.Parameters.Add(p);
				}
				return cmd.ExecuteNonQuery();
			}
		}
		
		/// <summary>
		/// Executes a where Sql statement with a custom condition
		/// </summary>
		/// <param name="condition">
		/// Condition to execute
		/// </param>
		/// <param name="parameters">
		/// Parameters to the Sql condition, {0} is substituted for the first parameter, and {1} for next and so on.
		/// </param>
		/// <returns>
		/// The result of the query as instances of T
		/// </returns>
		public IEnumerable<T> Where<T>(string condition, params object[] parameters) where T : class, new(){
			return this.Query<T>(this.builder.WhereQuery(this.TableName<T>(), condition), parameters);
		}
		
		/// <summary>
		/// Executes a where Sql statement with a custom condition, returns only one row or null
		/// </summary>
		/// <remarks>
		/// This is the same as <see cref="Repository.Where"/> except it only returns one row
		/// </remarks>
		/// <param name="condition">
		/// Condition to execute
		/// </param>
		/// <param name="parameters">
		/// Parameters to the Sql condition, {0} is substituted for the first parameter, and {1} for next and so on.
		/// </param>
		/// <returns>
		/// The result of the query as instance of T or null
		/// </returns>
		public T SingleWhere<T>(string condition, params object[] parameters) where T : class, new(){
			return this.SingleQuery<T>(this.builder.PagedWhereQuery(this.TableName<T>(), condition, 0, 1), parameters);
		}
		
		/// <summary>
		/// List all rows in the table for T
		/// </summary>
		/// <returns>
		/// The result of the query as instances of T
		/// </returns>
		public IEnumerable<T> All<T>() where T : class, new(){
			return this.Query<T>(this.builder.AllQuery(this.TableName<T>()));
		}
		
		/// <summary>
		/// List all rows in the table for T, paged
		/// </summary>
		/// <param name="page">
		/// Page number, starting from 0
		/// </param>
		/// <param name="pageSize">
		/// Page size, should be >= 1
		/// </param>
		/// <returns>
		/// The result of the query as instances of T
		/// </returns>
		public IEnumerable<T> Paged<T>(int page, int pageSize) where T : class, new(){
			return this.Query<T>(this.builder.PagedQuery(this.TableName<T>(), page, pageSize));
		}
		
		/// <summary>
		/// 
		/// </summary>
		/// <param name="sql">
		/// A <see cref="System.String"/>
		/// </param>

		/// <summary>
		/// Execute a custom sql query
		/// </summary>
		/// <param name="sql">
		/// Sql query to execute
		/// </param>
		/// <param name="parameters">
		/// Parameters to the Sql query, {0} is substituted for the first parameter, and {1} for next and so on.
		/// </param>
		/// <returns>
		/// The result of the query as instances of T
		/// </returns>
		public IEnumerable<T> Query<T>(string sql, params object[] parameters) where T : class, new(){
			//Ensure the table exists, so that we can operate on it, this also complains if connection is disposed
			this.EnsureTable<T>();
			
			using(DbCommand cmd = this.connection.CreateCommand()){
				cmd.CommandText = sql;
				int pCount = 0;
				foreach(object param in parameters){
					var p = cmd.CreateParameter();
					p.ParameterName = "@" + pCount++;
					p.Value = param;
					cmd.Parameters.Add(p);
				}
				using(DbDataReader rdr = cmd.ExecuteReader()){ 
					//If there's no result break the generator
					if(!rdr.HasRows)
						yield break;
					//While we have result produce rows and yield them
					while(rdr.Read()){
						//Create new instance of target type
						T row = new T();
						//Foreach field on the row
						for(int i = 0; i < rdr.FieldCount; i++){
							//Foreach property on target type
							PropertyInfo[] properties = typeof(T).GetProperties();
							foreach(PropertyInfo property  in properties){
								//If the property name and row name match
													
								if(string.Compare(property.Name, rdr.GetName(i), true) == 0){
									//Get the value if not null
									object val = null;
									if(!rdr.IsDBNull(i)){
										//If not a null value, get the value
										val = rdr.GetValue(i);
										//Get the target property type
										Type targetType = property.PropertyType;
										//If target type and column type doesn't match we need to convert them
										if(targetType != rdr.GetFieldType(i)){
											//If target type is a nullable
											if(targetType.IsGenericType && // typeof(Nullable<>) doesn't work, probably a Mono specific bug...
											   targetType.GetGenericTypeDefinition() == typeof(Nullable<int>).GetGenericTypeDefinition()){
												//If nullable<Something> target type should be Something, e.g.
												// find the underlying type and convert to that
												targetType = Nullable.GetUnderlyingType(targetType);
											}
											//Convert column data to target type
											val = Convert.ChangeType(val, targetType);
										}
									}
									//Set the value on the target type
									property.SetValue(row, val, null);
								}
							}
						}
						//Yield the target type
						
						yield return row;
					}
				}
			}
			
			
		}
		
		/// <summary>
		/// Execute a custom sql query, and return only the first row or null
		/// </summary>
		/// <param name="sql">
		/// Sql query to execute
		/// </param>
		/// <param name="parameters">
		/// Parameters to the Sql query, {0} is substituted for the first parameter, and {1} for next and so on.
		/// </param>
		/// <returns>
		/// The result of the query as instance of T or null
		/// </returns>
		public T SingleQuery<T>(string sql, params object[] parameters) where T : class, new(){
			var query = this.Query<T>(sql, parameters); 
			foreach(T t in query)
				return t;
			return null;
		}
		
		#region IDisposable Implementation
		//This pattern for implementing IDisposable is recommended by:
		//Framework Design Guidelines, 2. Edition, Section 9.4
		
		/// <summary>
		/// Dispose this object
		/// </summary>
		/// <remarks>
		/// This boolean disposing parameter here ensures that objects with a finalizer is not disposed,
		/// this is method is invoked from the finalizer. Do overwrite, and call, this method in base 
		/// classes if you use any unmanaged resources.
		/// </remarks>
		/// <param name="disposing">
		/// A <see cref="System.Boolean"/> False if called from the finalizer, True if called from Dispose.
		/// </param>
		protected virtual void Dispose(bool disposing){
			if(disposing){
				//Release finalizable resources, at the moment none.
				if(this.transaction != null){
					if(!this.disableCommit)
						this.transaction.Commit();
					this.transaction.Dispose();
					this.transaction = null;
				}
				if(this.connection != null)
					this.connection.Dispose();
				this.connection = null;
			}
		}
		
		/// <summary>
		/// Release resources held by this object
		/// </summary>
		public void Dispose(){
			//We're disposing this object and can release objects that are finalizable
			this.Dispose(true);
			GC.SuppressFinalize(this);
		}
		
		/// <summary>
		/// Finalize this object
		/// </summary>
		~Repository(){
			//Dispose this object, but do NOT release finalizable objects, we don't know in which order
			//these are release and they may already be finalized.
			this.Dispose(false);
		}
		#endregion
		
		/// <summary>
		/// Close the database connection
		/// </summary>
		/// <remarks>
		/// The same as dispose
		/// </remarks>
		public void Close(){
			this.Dispose();
		}
	}
}
