
using System;
using System.Text;
using System.Data.Common;
using System.Collections.Generic;
using System.Data.SqlTypes;
using Foodolini.Database.Sql;

namespace Foodolini.Database.Sql
{
	/// <summary>
	/// An SqlBuilder for building sql statements for Sqlite
	/// </summary>
	public class Sqlite : SqlStrategy
	{
		
		/// <summary>
		/// Builds a query that lists all tables
		/// </summary>
		/// <returns>
		/// An sql statement for quering all tables
		/// </returns>
		public override string TableQuery(){
			return "SELECT name FROM sqlite_master WHERE type = 'table'";
		}
		
		/// <summary>
		/// Sqlite specific create table statement
		/// </summary>
		public override string CreateTableQuery(string table, IDictionary<string, Type> columns, string primaryKey){
			StringBuilder sql = new StringBuilder();
			sql.AppendFormat("CREATE TABLE {0} ({1} INTEGER PRIMARY KEY, ", table, primaryKey);
			foreach(KeyValuePair<string, Type> col in columns){
				sql.AppendFormat("{0} {1}, ", col.Key, this.GetTypeAffinity(col.Value));
			}
			sql.Remove(sql.Length - 2, 2);
			sql.Append(")");
			return sql.ToString();
		}
		
		/// <summary>
		/// Gets the affinity of a column using it's .Net type
		/// </summary>
		/// <remarks>
		/// For more information on column affinity, see http://www.sqlite.org/datatype3.html
		/// </remarks>
		/// <param name="type">
		/// Type to find affinity for
		/// </param>
		/// <returns>
		/// Sqlite storage class
		/// </returns>
		private string GetTypeAffinity(Type type){
			if(type == typeof(string))
				return "TEXT";
			if(type == typeof(long) ||
			   type == typeof(int) ||
			   type == typeof(Enum))
				return "INTEGER NOT NULL";
			if(type == typeof(long?) ||
			   type == typeof(int?))
				return "INTEGER";
			//TODO: Optimize type selection for int
			if(type == typeof(byte[]))
				return "BLOB";
			if(type == typeof(double))
				return "FLOAT NOT NULL";
			if(type == typeof(double?))
				return "FLOAT";
			if(type == typeof(float))
				return "REAL NOT NULL";
			if(type == typeof(float?))
				return "REAL";
			if(type == typeof(DateTime))
				return "DATETIME NOT NULL";
			if(type == typeof(DateTime?))
				return "DATETIME";
			if(type == typeof(TimeSpan))
				return "INTEGER NOT NULL";
			if(type == typeof(TimeSpan?))
				return "INTEGER";
			if(type == typeof(bool))
				return "BIT NOT NULL";
			if(type == typeof(bool?))
				return "BIT";	
			throw new SqlTypeException("Type of " + type.ToString() + " cannot be saved to an Sqlite database");
		}
		
		/// <returns>
		/// An Sqlite specific insert statement
		/// </returns>
		public override string InsertQuery(string table, ICollection<string> parameters){
			StringBuilder sql = new StringBuilder();
			sql.AppendFormat("INSERT INTO {0} (", table);
			StringBuilder param = new StringBuilder();
			foreach(string p in parameters){
				sql.AppendFormat("{0}, ", p);
				param.AppendFormat("@{0}, ", p);
			}
			sql.Remove(sql.Length - 2, 2);
			param.Remove(param.Length - 2, 2);
			sql.AppendFormat(") VALUES ({0}); SELECT last_insert_rowid()", param);
			return sql.ToString();
		}
		
		/// <returns>
		/// An Sqlite specific update statement
		/// </returns>
		public override string UpdateQuery(string table, ICollection<string> parameters, string identifier){
			StringBuilder sql = new StringBuilder();
			sql.AppendFormat("UPDATE {0} SET ", table);
			foreach(string p in parameters){
				sql.AppendFormat("{0}=@{0}, ", p);
			}
			sql.Remove(sql.Length - 2, 2);
			sql.AppendFormat(" WHERE {0}=@{0}", identifier);
			return sql.ToString();
		}
		
		/// <returns>
		/// An Sqlite specific where query
		/// </returns>
		public override string WhereQuery(string table, string condition){
			StringBuilder sql = new StringBuilder();
			sql.AppendFormat("SELECT * FROM {0} WHERE {1}", table, condition);
			return sql.ToString();
		}
		
		/// <returns>
		/// An Sqlite specific where paged query
		/// </returns>
		public override string PagedWhereQuery(string table, string condition, int page, int pageSize){
			StringBuilder sql = new StringBuilder();
			sql.AppendFormat("SELECT * FROM {0} WHERE {1} LIMIT {2} OFFSET {3}", table, condition, pageSize, page * pageSize);
			return sql.ToString();
		}
		
		/// <returns>
		/// An Sqlite specific query
		/// </returns>
		public override string AllQuery(string table){
			StringBuilder sql = new StringBuilder();
			sql.AppendFormat("SELECT * FROM {0}", table);
			return sql.ToString();
		}
		
		/// <returns>
		/// An Sqlite specific paged query statement
		/// </returns>
		public override string PagedQuery(string table, int page, int pageSize){
			StringBuilder sql = new StringBuilder();
			sql.AppendFormat("SELECT * FROM {0} LIMIT {1} OFFSET {2}", table, pageSize, page * pageSize);
			return sql.ToString();
		}
		
		/// <returns>
		/// An Sqlite specific delete query
		/// </returns>
		public override string DeleteQuery(string table, string identifier){
			StringBuilder sql = new StringBuilder();
			sql.AppendFormat("DELETE FROM {0} WHERE {1}=@{1}", table, identifier);
			return sql.ToString();
		}
		
		/// <summary>
		/// 
		/// </summary>
		/// <param name="table">
		/// A <see cref="System.String"/>
		/// </param>
		/// <param name="condition">
		/// A <see cref="System.String"/>
		/// </param>
		/// <returns>
		/// A <see cref="System.String"/>
		/// </returns>
		public override string DeleteWhereQuery(string table, string condition){
			return string.Format("DELETE FROM {0} WHERE {1}", table, condition);
		}
	}
}
