
using System;
using NUnit.Framework;

namespace Foodolini.Database.Test
{
	/// <summary>
	/// Unit test to the Database layer of Picture
	/// </summary>
	[TestFixture]
	public class PictureTest : DatabaseTest
	{
		/// <summary>
		/// Create 5 MiB pseudo random data for stress testing :)
		/// </summary>
		/// <remarks>
		/// The c
		/// </remarks>
		[TestFixtureSetUp]
		public void LoadStressTestVector(){
			Random prgn = new Random((int)(DateTime.Now.Ticks % int.MaxValue));
			prgn.NextBytes(stressTestVector);
		}
		
		/// <summary>
		/// Small test vector
		/// </summary>
		private static readonly byte[] smallTestVector = {45, 78, 92, 0, 1,2, 34};
		
		/// <summary>
		/// Large test vector for stress testing (5 MiB)
		/// </summary>
		private static readonly byte[] stressTestVector = new byte[5 * 1024 * 1024];
		
		/// <summary>
		/// Test that creates a table
		/// </summary>
		[Test]
		public void CreateTable(){
			this.repository.CreateTable<Picture>();
		}
		
		/// <summary>
		/// Test that tries to save a simple picture
		/// </summary>
		/// <remarks>
		/// Please note that this test will also automatically create a Pictures table on-the-fly, so if
		/// the CreateTable test failed, this test is likely to fail too. However, this test does *not* depend
		/// on the CreateTable test.
		/// </remarks>
		[Test]
		public void Save(){
			//Create a new instance of Picture
			Picture p = new Picture();
			p.Image = smallTestVector;
			//Add it to the repository
			this.repository.Add<Picture>(p);
		}
		
		/// <summary>
		/// Test that saves and load a picture
		/// </summary>
		[Test]
		public void SaveLoad(){
			//Create a new instance of Picture
			Picture picture = new Picture();
			picture.Image = smallTestVector;
			
			//Add it to the repository
			this.repository.Add<Picture>(picture);
			
			//Loop over all the picture in the database, at this time only one
			Picture pic = null;
			foreach(var p in this.repository.All<Picture>()){
				if(pic != null)
					Assert.Fail("Database contained more that one item, after running Save()");
				pic = p;
			}
			
			//Ensure that the length matches
			Assert.AreEqual(smallTestVector.Length, pic.Image.Length, "Byte array lenght mismatch");
			
			//Check that the content matches
			for(int i = 0; i < smallTestVector.Length; i++)
				Assert.AreEqual(smallTestVector[i], pic.Image[i], "Byte array mismatch");
		}
		
		/// <summary>
		/// Stress test the database by loading and saving a large picture
		/// </summary>
		/// <remarks>
		/// This test uses 5 MiB pseudo random test vector generated in <see cref="PictureTest.LoadStressTestVector()"/>
		/// </remarks>
		[Test]
		public void StressTest(){
			//Create a new picture
			Picture picture = new Picture();
			//Give a huge set of random data :)
			picture.Image = stressTestVector;
			
			//Save the image
			this.repository.Add<Picture>(picture);
			
			//Iterate over all pictures in the database
			Picture pic = null;
			foreach(var p in this.repository.All<Picture>()){
				//If pic is not null there's more that one picture in the database, that's not possible.
				if(pic != null)
					Assert.Fail("Database contained more that one item, after running Save()");
				pic = p;
			}
			
			//Check that the length matches
			Assert.AreEqual(stressTestVector.Length, pic.Image.Length, "Byte array lenght mismatch");
			
			//Check that the bytes match
			for(int i = 0; i < stressTestVector.Length; i++)
				Assert.AreEqual(stressTestVector[i], pic.Image[i], "Byte array mismatch");
		}
		
		//TODO: Test Update, Delete, Where with multiple items in the database
	}
}
