
using System;
using Foodolini.Activities;
using Foodolini.BusinessLogic;
using Gtk;
using System.Collections.Generic;
using System.IO;
using Foodolini.Activities.Ingredients;

namespace Foodolini.Activities.Cookbook
{
	
	/// <summary>
	/// An activity where a new recipe can be created
	/// </summary>
	[FoodoliniActivity("Create recipe", false, "recipe.png")]
	[System.ComponentModel.ToolboxItem(true)]
	public partial class RecipeCreater : Gtk.Bin, IActivity
	{
		private IOwner owner;
		private Recipe recipe;
		private NodeStore ingredientStorage = new NodeStore(typeof(IngredientAdapter));
		private IList<string> listCategory = new List<string>();
		private List<string> listDirections = new List<string>();
		bool ingredientsAdded = false;
		
		/// <summary>
		/// Constructor that sets up the nodeview and sets the sensitivity on the save button to false
		/// </summary>
		public RecipeCreater()
		{
			this.Build();
			
			//Sets up the nodeview.
			this.nvIngredients.NodeStore = this.ingredientStorage;
			this.nvIngredients.AppendColumn("Added ingredients", new CellRendererText(), (tree_column, cell, node)=>{
				IngredientAdapter n = (IngredientAdapter)node;
				CellRendererText c = (CellRendererText)cell;
				c.Text = n.Name;
			});
			
			CellRendererSpin quantityCell = new CellRendererSpin();
			quantityCell.Editable = true;
			quantityCell.Adjustment = new Adjustment(0,1,100000,1,1,1);
			quantityCell.Digits = 1;
			this.nvIngredients.AppendColumn("Quantity (g)", quantityCell, (column, cell, node) =>
            {
                ((CellRendererSpin)cell).Text = ((IngredientAdapter)node).Quantity.ToString ("0");
            });
			
			quantityCell.Edited += delegate(object o, EditedArgs args) {
				var node = (IngredientAdapter)ingredientStorage.GetNode(new TreePath(args.Path));
				double newValue;
				if(double.TryParse(args.NewText, out newValue)) {
					node.Quantity = newValue;
				}
			};
			
			//HACK: set spinbutton to actual value because of rounding error.
			quantityCell.EditingStarted += (o, args) =>
			{
				var node = (IngredientAdapter)ingredientStorage.GetNode (new TreePath (args.Path));
				((SpinButton)args.Editable).Value = node.Quantity;
			};
			
			bxIngredient.ShowAll();
			
			//sets the sensitivity to false as no entries are filled at start.
			this.btnSave.Sensitive = false;
		}
		
		public event EventHandler<RecipeEventArgs> OnRecipeCreated;
		
		/// <summary>
		/// Checks if a Title, at least one categori and ingredient has been given.
		/// </summary>
		private void CheckIfFilled()
		{
			if (this.entTitle.Text == "" | !ingredientsAdded | tvDirections.Buffer.Text == "")
			{
				btnSave.Sensitive = false;
			}
			else
			{
				btnSave.Sensitive = true;
			}
			
		}
		
		/// <summary>
		/// Opens a dialog where a new ingredient and the amount can be selected and added to the Recipe opon saving
		/// </summary>
		protected virtual void OnAddIngredient (object sender, System.EventArgs e)
		{
			Ingredient selectedIngredient;
			IngredientSelector dialog = new IngredientSelector(true);
			if (dialog.Run() == (int)ResponseType.Ok)
			{
				selectedIngredient = dialog.SelectedIngredient;
				this.ingredientStorage.AddNode(new IngredientAdapter(selectedIngredient, dialog.Quantity));
			}
			dialog.Destroy();
			ingredientsAdded = true;
			CheckIfFilled();
		}
		
		/// <summary>
		/// Removes the selected ingredient form the list. 
		/// If no ingredients are left, the variable ingredientsAdded is set to false. 
		/// CheckIfFilled() is called at the end.
		/// </summary>
		protected virtual void OnBtnRemoveIngredientClicked (object sender, System.EventArgs e)
		{
			if(this.nvIngredients.NodeSelection.SelectedNode != null)
			{
				var ingredient = (IngredientAdapter)this.nvIngredients.NodeSelection.SelectedNode;
				this.ingredientStorage.RemoveNode(ingredient);
				this.ingredientsAdded = false;
				if(this.ingredientStorage.GetEnumerator().MoveNext())
					this.ingredientsAdded = true;
				CheckIfFilled();
			}
		}
		
		/// <summary>
		/// Pops the activity when Cancel is clicked.
		/// </summary>
		protected virtual void OnCancelClick (object sender, System.EventArgs e)
		{
			this.owner.PopActivity();
		}
		
		/// <summary>
		/// Event called when an entry has been changed. Calls CheckIfFilled().
		/// </summary>
		protected virtual void OnEntryChanged (object sender, System.EventArgs e)
		{
			CheckIfFilled();
		}
		
		/// <summary>
		/// Event called when directions have changed. Calls CheckIfFilled().
		/// </summary>
		protected virtual void OnTvDirectionsKeyReleaseEvent (object o, Gtk.KeyReleaseEventArgs args)
		{
			CheckIfFilled();
		}
		
		/// <summary>
		/// Saves the Recipe in the database
		/// Button is only active when the title, the categories, the directions and Ingrediens have been set
		/// </summary>
		protected virtual void OnSaveClicked (object sender, System.EventArgs e)
		{
			string title = entTitle.Text;
			if(Recipe.GetByTitle(title) == null & Ingredient.GetByLongDescription(title) == null)
			{
				this.recipe = new Recipe(title);
				
				//Saves the meal type
				if(cbxMealtype.Active == 0)
					listCategory.Add("Breakfast");
				else if(cbxMealtype.Active == 1)
					listCategory.Add("Lunch");
				else if (cbxMealtype.Active == 2)
					listCategory.Add("Dinner");
				else
					listCategory.Add("Other");
				
				this.recipe.Difficulty = (Difficulty)cbxDifficulty.Active;
				
				this.recipe.PreparationTime = new TimeSpan((int)sbDays.Value, (int)sbHours.Value, (int)sbMinuts.Value, 0);
				
				//Saves the directions
				foreach(string category in this.entCategories.Text.Split(new char[]{','}))
					listCategory.Add(category.Trim());
				this.recipe.Categories = listCategory;
				
				//Saves the number of servings
				this.recipe.Servings = sbMultiplier.Value;
	
				//Saves the directions
				foreach(string direction in this.tvDirections.Buffer.Text.Split(new char[]{'\n'}, StringSplitOptions.RemoveEmptyEntries))
					listDirections.Add(direction.Trim());
				this.recipe.Directions = listDirections;
			
				//Saves the ingredients
				foreach(IngredientAdapter ingredient in this.ingredientStorage)
					this.recipe.Ingredients.Add(ingredient.Ingredient, ingredient.Quantity);
				
				//Saves the picture if set
				if (byteArray != null)
					recipe.Picture = byteArray;
				
				if(OnRecipeCreated != null)
					OnRecipeCreated(this, new RecipeEventArgs(this.recipe));
				
				this.recipe.Save();
				this.owner.PopActivity();
			}
			else
			{
				ConfirmDialog dialog = new ConfirmDialog("Recipe or ingredient already exist", "A recipe or ingredient with that name already exists.\nTry to remane the recipe.");
					if(dialog.Run() == (int)ResponseType.Ok){}
				dialog.Destroy();
			}
		}
		
		private byte[] byteArray;
		/// <summary>
		/// Previews the Recipe image when selected in the filechooserwidget
		/// </summary>
		protected virtual void OnSelectPictureClick (object sender, System.EventArgs e)
		{
			using(FileStream stream = File.OpenRead(fcbPicture.Filename)) {
				byteArray = new byte[stream.Length];
				for(int i = 0; i < stream.Length; i++)
					byteArray[i] = (byte)stream.ReadByte();
				Gdk.Pixbuf px = new Gdk.Pixbuf(byteArray);
				px = px.ScaleSimple(100, 100, Gdk.InterpType.Bilinear);//TODO: Make prettier
				imgRecipe.Pixbuf = px;
			}
		}
		
		/// <value>
		/// Getter for the current recipe.
		/// </value>
		public Recipe Recipe{
			get {return this.recipe;}
		}

		#region IActivity
		public void Register(IOwner owner){
			this.owner = owner;
		}
		
		public void Unregister(){
			this.owner = null;
		}
		
		public Widget Widget{
			get{
				return this;
			}
		}
		#endregion
	}
}
