//Compile this file for Linux support
using System;
using System.IO;
using System.Threading;
using System.Diagnostics;
using System.Collections.Generic;
using NDesk.DBus;
using org.freedesktop.Hal;
using org.freedesktop.DBus;

namespace Foodolini.Activities.FoodRegistration.Devices{
	/// <summary>
	/// Video capture device monitor for Linux
	/// </summary>
	/// <remarks>
	/// If compiled with USE_DBUS, this class will be able to provide correct events
	/// and it will also run a NDesk.DBus.Bus on a worker thread.
	/// Note: This is a Singleton class, and it should be disposed of after use.
	/// It's not strictly necessary to dispose this object, however, it have been made
	/// possible.
	/// </remarks>
	public class LinuxCaptureDeviceMonitor : ICaptureDeviceMonitor
	{	
		private Bus dbus;
		private Thread worker;
		private HalManager hal;
		private bool disposed = false;
		
		private LinuxCaptureDeviceMonitor(){
			this.dbus = Bus.System;
			this.worker = new Thread(this.iterate);
			this.worker.Start();
			//Check if HAL is running
			if(this.dbus.NameHasOwner("org.freedesktop.Hal")){
				//Create hal object
				this.hal = this.dbus.GetObject<HalManager>("org.freedesktop.Hal", new ObjectPath("/org/freedesktop/Hal/Manager"));
				//List devices
				foreach(ObjectPath path in this.hal.FindDeviceByCapability("video4linux.video_capture")){
					var device = this.DeviceInfo(path);
					if(device != null)
						this.devices.Add(path.ToString(), device);
				}
				//Subscribe to proper events
				this.hal.DeviceAdded += HandleDeviceAdded;
				this.hal.DeviceRemoved += HandleDeviceRemoved;
			}else{
				this.hal = null;
				this.worker.Abort();
				this.worker = null;
			}
		}

		#region HAL Signal handling
		
		private DeviceEventArgs DeviceInfo(ObjectPath path){
			Device device = this.dbus.GetObject<Device>("org.freedesktop.Hal", path);
			//info.capabilities is not a mandatory property
			if(device.PropertyExists("info.capabilities")){
				//Check that is has capture capability
				foreach(var cap in device.GetPropertyStringList("info.capabilities")){
					if(cap == "video4linux.video_capture"){
						string name = null;
						string devpath = device.GetPropertyString("video4linux.device");
						//info.product is not a mandatory property
						if(device.PropertyExists("info.product"))
							name = device.GetPropertyString("info.product");
						else
							name = Path.GetFileName(devpath);
						return new DeviceEventArgs(name, devpath);
					}
				}
			}
			return null;
		}
		
		/// <summary>
		/// Handles HAL dbus signal for device added
		/// </summary>
		void HandleDeviceAdded(ObjectPath path){
			var device = this.DeviceInfo(path);
			if(device != null){
				GLib.IdleHandler idle = delegate(){
					this.devices.Add(path.ToString(), device);
					if(this.DeviceAdded != null)
						this.DeviceAdded(this, device);
					return false;
				};
				GLib.Idle.Add(idle);
			}
		}
		
		/// <summary>
		/// Handles HAL dbus signal for device removed
		/// </summary>
		void HandleDeviceRemoved(ObjectPath path){
			GLib.IdleHandler idle = delegate(){
				IDeviceInfo device;
				if(this.devices.TryGetValue(path.ToString(), out device)){
					this.devices.Remove(path.ToString());
					if(this.DeviceRemoved != null)
						this.DeviceRemoved(this, (DeviceEventArgs)device);
				}
				return false;
			};
			GLib.Idle.Add(idle);
		}
		
		#endregion
		
		/// <summary>
		/// Runs the dbus event loop
		/// </summary>
		private void iterate(){
			while(true){
				//All sorts of bad stuff may happen on this bus when we open and close it too much...
				int fails = 0;
				try{
					this.dbus.Iterate();
				}
				catch(ThreadAbortException e){
					break;
				}
				catch(Exception e){
				 	fails++;
					Console.WriteLine("dbus misbehaved: " + e.Message);
					if(fails > 5)
						break;
				}
			}
		}
		
		#region Singleton implementation
		
		/// <summary>
		/// Singleton instance of VideoSourceManager
		/// </summary>
		private static LinuxCaptureDeviceMonitor instance = null;
		
		/// <summary>
		/// Gets and instance of VideoSourceManager
		/// </summary>
		public static LinuxCaptureDeviceMonitor Instance{
			get{
				if(instance == null || instance.disposed){
					if(instance != null){
						instance.DeviceAdded = null;
						instance.DeviceRemoved = null;
					}
					instance = new LinuxCaptureDeviceMonitor();
				}
				return instance;
			}
		}
		
		#endregion
		
		/// <summary>
		/// Devices on the system and their path in hal/dbus
		/// </summary>
		/// <remarks>May only be accessed from the UI-thread.</remarks>
		private Dictionary<string, IDeviceInfo> devices = new Dictionary<string, IDeviceInfo>();
		
		#region ICaptureDeviceMonitor implementation
		
		/// <summary>
		/// Occurs whenever a device is added to the system
		/// </summary>
		/// <remarks>This event is only usable if HAL is available.</remarks>
		public event EventHandler<DeviceEventArgs> DeviceAdded;
		
		/// <summary>
		/// Occurs whenever a device removed from the system
		/// </summary>
		/// <remarks>This event is only usable if HAL is available.</remarks>
		public event EventHandler<DeviceEventArgs> DeviceRemoved;
		
		/// <summary>
		/// List potential video sources on the system
		/// </summary>
		/// <remarks>
		/// If HAL (Hardware Abstraction Layer) is running this list the devices provided by
		/// HAL which offer video capture capability.
		/// 
		/// If HAL is not running it list all /dev/video* devices, and attempts to get their name
		/// using /lib/udev/v4l_id, see the udev source (link available in source comments) for
		/// more information on this utility.
		/// If finding a name for a device fails using /lib/udev/v4l_id, it will provide "Video *"
		/// as name for /dev/video*.
		/// 
		/// Alternately, define UDEV_RULES and the "/dev/v4l/by-id/" symlinks will be used
		/// to find the names. Note this likely depends on Ubuntu 9.10 udev rules.
		/// Thus the other approach is preffered, if HAL is not running.
		/// </remarks>
		/// <value>
		/// A <see cref="IDictionary"/> of Name and device
		/// </value>
		public ICollection<IDeviceInfo> Devices{
			get{
				if(this.hal != null)
					return this.devices.Values;
				else{
					var retval = new List<IDeviceInfo>();
#if UDEV_RULES
					try{
						//A small hack that depends on Ubuntu's udev rules, to find a name
						//See /lib/udev/rules.d/60-persistent-v4l.rules
						foreach(var device in Directory.GetFiles("/dev/v4l/by-id/")){
							var name = Path.GetFileName(device);
							name = name.Remove(0, name.IndexOf("_") + 1);
							name = name.Remove(name.IndexOf("-video-index"));
							if(name == string.Empty)
								throw new Exception("Hack that jumps aways from here :) ");
							retval.Add(new DeviceEventArgs(name, device));
						}
					}
					catch{
						//rescue if the hack fails... :)
						foreach(var device in Directory.GetFiles("/dev/", "video*")){
							//I can at least add a space to make it look good :)
							var name = "Video " + Path.GetFileName(device).Remove(0, 5);
							retval.Add(new DeviceEventArgs(name, device));
						}
					}
#else
					foreach(var device in Directory.GetFiles("/dev/", "video*")){
						string name = null;
						//Assume it supports capture
						bool supports_capture = true;
						
						//Attempt to use /lib/udev/v4l_id a small program used in udev rules, but seems to be available to userspace
						//v4l_id is afaik not documented, but it's short and the source can be found in extras/v4l_id/v4l_id.c
						//of the udev development tree: linux/hotplug/udev.git
						//See: http://git.kernel.org/?p=linux/hotplug/udev.git;a=blob;f=extras/v4l_id/v4l_id.c;hb=HEAD
						try{
							using(Process v4l_id = new Process()){
								v4l_id.StartInfo.UseShellExecute = false;
								v4l_id.StartInfo.FileName = "/lib/udev/v4l_id";
								v4l_id.StartInfo.CreateNoWindow = true;
								v4l_id.StartInfo.Arguments = device;
								v4l_id.StartInfo.RedirectStandardOutput = true;
								v4l_id.Start();
								string input;
								while((input = v4l_id.StandardOutput.ReadLine()) != null){
									if(input.StartsWith("ID_V4L_PRODUCT="))
										name = input.Remove(0, 15);
									if(input.StartsWith("ID_V4L_CAPABILITIES="))
										supports_capture = input.Contains(":capture:");
								}
								v4l_id.WaitForExit();
								if(v4l_id.ExitCode != 0)
									throw new Exception("Don't use anything we've got here...");
							}
						}
						catch(Exception e){
							//I can at least add a space to make it look good :)
							name = "Video " + System.IO.Path.GetFileName(device).Remove(0, 5);
						}
						
						if(supports_capture)
							retval.Add(new DeviceEventArgs(name, device));
					}
#endif
					return retval;
				}
			}
		}
		
		/// <value>
		/// True, if HAL is running, then auto refresh is supported
		/// </value>
		public bool SupportsAutoRefresh{
			get{
				return true;
			}
		}

		#endregion

		#region IDisposable implementation
		
		/// <summary>
		/// Stop listing for devices
		/// </summary>
		public void Dispose(){
			if(this.worker != null)
				this.worker.Abort();
			this.worker = null;
			this.DeviceAdded = null;			
			this.DeviceRemoved = null;
			this.disposed = true;
		}
		
		#endregion
	}
	
	/// <summary>
	/// Static class for getting an instance of ICaptureDeviceMonitor for the current platform
	/// </summary>
	public static class CaptureDeviceMonitor{
		public static ICaptureDeviceMonitor Instance{
			get{
				return LinuxCaptureDeviceMonitor.Instance;
			}
		}
	}
}

/// <summary>
/// Interfaces for HAL over dbus
/// </summary>
/// <remarks>
/// This namespace only provides a subset of the HAL dbus interfaces and these interfaces are not complete
/// </remarks>
namespace org.freedesktop.Hal{
	/// <summary>
	/// An interface to dbus objects that implements the org.freedesktop.Hal.Manager interface
	/// </summary>
	/// <remarks>
	/// According to HAL dbus specification only /org/freedesktop/Hal/Manager does this.
	/// This interface only specifies a subset of the methods and signals offered by the
	/// org.freedesktop.Hal.Manager interface. For information on additional methods and 
	/// signal, see HAL dbus interface specification:
	/// /usr/share/doc/hal-doc/spec/hal-spec.html
	/// </remarks>
	[Interface("org.freedesktop.Hal.Manager")]
	interface HalManager : Introspectable{
		
		ObjectPath[] FindDeviceByCapability(string capability);
		event DeviceHandler DeviceAdded;
		event DeviceHandler DeviceRemoved;
	}
	
	/// <summary>
	/// Delegate for the DeviceAdded and DeviceRemoved events on HalManager
	/// </summary>
	public delegate void DeviceHandler(ObjectPath path);
	
	/// <summary>
	/// An interface to dbus objects that implement the org.freedesktop.Hal.Device interface
	/// </summary>
	/// <remarks>
	/// This interface is implemented by all objects prefixed with /org/freedesktop/Hal/devices/
	/// E.g. all devices HAL knows about.
	/// This interface only specifies a subset of the methods and signals offered by the
	/// org.freedesktop.Hal.Device interface. For information on additional methods and signal,
	/// see HAL dbus interface specification: /usr/share/doc/hal-doc/spec/hal-spec.html
	/// </remarks>
	[Interface("org.freedesktop.Hal.Device")]
	interface Device : Introspectable{
		string GetPropertyString(string name);
		string[] GetPropertyStringList(string name);
		bool PropertyExists(string name);
	}
}
