
using System;
using Foodolini.BusinessLogic;
using Gtk;

namespace Foodolini.Activities
{
	/// <summary>
	/// The interface an activity use to interface the main window
	/// </summary>
	public interface IOwner
	{
		/// <summary>
		/// Occurs when the current user is changed
		/// </summary>
		event EventHandler<PersonEventArgs> CurrentUserChanged;
		
		/// <summary>
		/// Gets the current user
		/// </summary>
		Person CurrentUser{ get; }
		
		/// <summary>
		/// Push an activity onto the activity stack, and display it
		/// </summary>
		/// <param name="activity">
		/// Activity to stack
		/// </param>
		/// <returns>
		/// The activity that was created or null of not found
		/// </returns>
		IActivity PushActivity(string activity);
		
		/// <summary>
		/// Push an activity of type T onto the activity stack, and display it
		/// </summary>
		/// <returns>
		/// The activity that was created or null of not found
		/// </returns>
		T PushActivity<T>() where T : class, new();
		
		/// <summary>
		/// Pop the top-most activity, and make the next activity on the stack visible, go-to home if no activity is next
		/// </summary>
		/// <remarks>
		/// If calling from the top-most activity and this method returns true, beaware that the Widget of your activity have been destoryed.
		/// </remarks>
		/// <returns>
		/// True, if activity could be unloaded
		/// </returns>
		bool PopActivity();
		
		/// <summary>
		/// Load an activity, unloads the entire activity stack
		/// </summary>
		/// <remarks>
		/// If calling from the top-most activity and this method returns true, beaware that the Widget of your activity have been destoryed.
		/// Also note that this method unload the activity stack from the top of the stack, if an activity along the way chooses not to unload
		/// this activity will be top-most and shown.
		/// </remarks>
		/// <param name="activity">
		/// The Activity to load
		/// </param>
		/// <returns>
		/// True, if activity exists and activity stack could be unloaded
		/// </returns>
		bool LoadActivity(string activity);
		
		/// <summary>
		/// Toggles the ShoppingList. If expanded, it will be collapsed, and if collapsed it will be expanded.
		/// </summary>
		void ToggleShoppingList(bool state);
		
		/// <summary>
		/// Goto the home activity
		/// </summary>
		bool HomeActivity();
		
		Window Window{get;}
		
		void UpdateUserList();		
		
		/// <summary>
		/// Occurs when an activity is unloading
		/// </summary>
		/// <remarks>
		/// Do only handle this is the Activity referenced in the arguments is the activity you are in.
		/// Be careful about changing the Unload property of the event arguments as this will cause the unload to
		/// fail.
		/// </remarks>
		event EventHandler<ActivityUnloadedArgs> ActivityUnloading;
		
		/// <summary>
		/// Add all ingredients needed for the recipe to the shoppinglist
		/// </summary>
		/// <remarks>
		/// This is all ingredients, and does not depend on what is on storage.
		/// If the nature of the shoppinglist is changed in the future, this may change.
		/// </remarks>
		void AddRecipe(Recipe recipe, double servings);
	}


	/// <summary>
	/// Custom event arguments for when a different user has been selected.
	/// </summary>
	public class PersonEventArgs : EventArgs
	{
	    public Person Person { get; private set; }
	    public PersonEventArgs(Person person)
	    {
	        this.Person = person;
	    }
	}
	
	/// <summary>
	/// Event arguments for IActivty.ActivityUnloading
	/// </summary>
	public class ActivityUnloadedArgs : EventArgs{
		/// <summary>
		/// Set this to false to avoid being unloaded
		/// </summary>
		public bool Unload{get; set;}
			
		/// <value>
		/// Activity that is to be unloaded
		/// </value>
		public IActivity Activity{get; private set;}
		
		/// <summary>
		/// Create an instance of ActivityUnloadedArgs
		/// </summary>
		/// <param name="activity">
		/// Activity to unload
		/// </param>
		public ActivityUnloadedArgs(IActivity activity){
			this.Unload = true;
			this.Activity = activity;
		}
	}
	
}