using System;
using Foodolini.BusinessLogic;
using Gtk;
using System.Collections.Generic;
using Foodolini.Activities;

namespace Foodolini.Activities.Ingredients
{
	/// <summary>
	/// A dialog window which allows the user to either view/edit the details
	/// of a specific <see cref="Ingredient"/>, delete or create a new one.  
	/// </summary>
	public partial class IngredientDialog : Gtk.Dialog
	{
		/// <summary>
		/// The <see cref="Ingredient"/> instance.  
		/// </summary>
		public Ingredient Ingredient { get; private set; }
		
		/// <summary>
		/// The nodestore containing nutrients.
		/// </summary>
		NodeStore nutrientsStore = new NodeStore (typeof(NutrientAdapter));
		
		/// <summary>
		/// The liststore containing food categories.
		/// </summary>
		ListStore categoryStore = new ListStore (typeof(string));

		/// <summary>
		/// The initially selected category. 
		/// </summary>
		private string selectedCategory = string.Empty;
		/// <summary>
		/// The default name of the new Ingredient. 
		/// </summary>
		private static string defaultName = "New ingredient";

		/// <summary>
		/// Instantiates a new ingredient dialog.
		/// </summary>
		/// <param name="ingredient">
		/// A <see cref="Ingredient"/> to bind to the dialog.
		/// </param>
		/// <param name="isNew">
		/// A <see cref="System.Boolean"/> whether the Ingredient provided is
		/// new.
		/// </param>
		/// <param name="selectedCategory">
		/// A <see cref="System.String"/> containing the category to select
		/// in the category combobox.
		/// </param>
		public IngredientDialog (Ingredient ingredient, bool isNew, string selectedCategory)
		{
			this.Build ();
			this.Ingredient = ingredient;
			this.selectedCategory = selectedCategory;
			SetIngredient (isNew);
		}

		/// <summary>
		/// Constructor for creating a new Ingredient object with a selected category. 
		/// </summary>
		/// <param name="selectedCategory">
		/// A <see cref="System.String"/> containing the category to select
		/// in the category combobox.
		/// </param>
		public IngredientDialog (string selectedCategory) : this(new Ingredient (defaultName), true, selectedCategory)
		{
		}

		/// <summary>
		/// Instantiates an ingredient dialog with the provided ingredient. 
		/// </summary>
		/// <param name="ingredient">
		/// A <see cref="Ingredient"/>
		/// </param>
		public IngredientDialog (Ingredient ingredient) : this(ingredient, false, string.Empty)
		{
		}

		public IngredientDialog () : this(new Ingredient (defaultName), true, string.Empty)
		{
		}

		/// <summary>
		/// Initializes all widgets
		/// </summary>
		/// <param name="isNew">
		/// A <see cref="System.Boolean"/>
		/// </param>
		private void SetIngredient (bool isNew)
		{

			// Set text boxes
			this.Title = this.Ingredient.LongDescription;

			if (isNew)
				btnDelete.Sensitive = false;

			if (!isNew)
				txtLongDescription.Text = this.Ingredient.LongDescription ?? string.Empty;
			txtShortDescription.Text = this.Ingredient.ShortDescription ?? string.Empty;
			txtCommercialName.Text = this.Ingredient.CommercialName ?? string.Empty;
			txtManufacturer.Text = this.Ingredient.Manufacturer ?? string.Empty;

			// shelflife
			if (this.Ingredient.ShelfLife != TimeSpan.Zero) {
				this.sbShelfDays.Value = this.Ingredient.ShelfLife.Days;
			}

			// expiration 
			if (this.Ingredient.ExpirationAfterOpening != TimeSpan.MaxValue) {
				this.sbOpenDays.Value = this.Ingredient.ExpirationAfterOpening.Days;
			}

			// Load categories
			List<string> categories = new List<string> (Ingredient.Categories);
			categories.Sort ();

			foreach (string category in categories)
				categoryStore.AppendValues (category);

			cbFoodGroup.Model = categoryStore;
			this.cbFoodGroup.QueueResize ();

			if (!isNew)
				SelectActiveCategory (this.Ingredient.Category);
			else
				SelectActiveCategory (selectedCategory);

			// Get nutrients and sort by sort order
			var nutDef = new List<Nutrient> (Nutrient.ListNutritionDefinitions ());
			nutDef.Sort ((n1, n2) => (n1.SortOrder.CompareTo (n2.SortOrder)));

			// Assign values to nutrients connected to the ingredient
			foreach (var n in nutDef) {
				double amount = 0;
				if (this.Ingredient.Nutrients.ContainsKey (n))
					amount = this.Ingredient.Nutrients[n];
				NutrientAdapter node = new NutrientAdapter (n, amount);
				this.nutrientsStore.AddNode (node);
			}
			this.nutrientsView.NodeStore = nutrientsStore;

			// Add the columns to the nodeview
			this.nutrientsView.AppendColumn ("Definition", new CellRendererText (), "text", 0);

			// Set ud the amount column
			TreeViewColumn valueColumn = new TreeViewColumn ();
			valueColumn.Title = "Value     ";
			CellRendererSpin valueRenderer = new CellRendererSpin ();
			valueRenderer.Editable = true;
			valueRenderer.Adjustment = new Adjustment (0.0, 0, double.MaxValue, 0.01, 0.01, 1);
			valueRenderer.Digits = 2;
			valueColumn.PackStart (valueRenderer, true);
			valueColumn.SetCellDataFunc (valueRenderer, (column, cell, node) => { ((CellRendererSpin)cell).Text = ((NutrientAdapter)node).Value.ToString ("0.00"); });

			//HACK: set spinbutton to actual value because of rounding error
			valueRenderer.EditingStarted += (o, args) =>
			{
				var node = (NutrientAdapter)nutrientsStore.GetNode (new TreePath (args.Path));
				((SpinButton)args.Editable).Value = node.Value;
			};

			valueRenderer.Edited += (o, args) =>
			{
				var node = (NutrientAdapter)nutrientsStore.GetNode (new TreePath (args.Path));
				double newValue;
				if (double.TryParse (args.NewText, out newValue))
					node.Value = newValue;
			};

			this.nutrientsView.AppendColumn (valueColumn);
			this.nutrientsView.AppendColumn ("Unit", new CellRendererText (), "text", 2);
		}

		/// <summary>
		/// Searches for the category in the combox and sets the
		/// corresponding iter. 
		/// </summary>
		/// <param name="category">
		/// A <see cref="System.String"/>
		/// </param>
		private void SelectActiveCategory (string category)
		{
			int i = 0;
			// iterate over the categories and set the active iter.
			foreach (object[] o in categoryStore) {
				string cat = o[0].ToString ();
				if (cat == category) {
					TreeIter it;
					if (categoryStore.GetIterFromString (out it, i + ""))
						cbFoodGroup.SetActiveIter (it);
					break;
				}
				i++;
			}
		}

		/// <summary>
		/// Saves changes to the ingredient 
		/// </summary>
		public void SaveIngredient ()
		{
			// Set text properties
			this.Ingredient.ShortDescription = txtShortDescription.Text;
			this.Ingredient.LongDescription = txtLongDescription.Text;
			this.Ingredient.Manufacturer = txtManufacturer.Text;
			this.Ingredient.CommercialName = txtCommercialName.Text;

			// set shelf life
			TimeSpan shelfLife = new TimeSpan (DateTime.MinValue.AddDays (sbShelfDays.ValueAsInt).Ticks);
			this.Ingredient.ShelfLife = shelfLife;

			// set expiration after opening
			TimeSpan expirationAfterOpening = new TimeSpan (DateTime.MinValue.AddDays (sbOpenDays.ValueAsInt).Ticks);
			if (expirationAfterOpening.Ticks > 0)
				this.Ingredient.ExpirationAfterOpening = expirationAfterOpening;
			else
				this.Ingredient.ExpirationAfterOpening = TimeSpan.MaxValue;

			// Set the category
			TreeIter it;
			if (this.cbFoodGroup.GetActiveIter (out it))
				this.Ingredient.Category = this.categoryStore.GetValue (it, 0).ToString ();

			foreach (NutrientAdapter node in nutrientsStore) {
				if (this.Ingredient.Nutrients.ContainsKey (node.Nutrient)) {
					this.Ingredient.Nutrients[node.Nutrient] = node.Value;
					// we only need to save the nutrient if is greater than zero
				} else if (node.Value > 0) {
					this.Ingredient.Nutrients.Add (node.Nutrient, node.Value);
				}
			}
			this.Ingredient.Save ();
		}

		protected virtual void OnNutrientsViewRowActivated (object o, Gtk.RowActivatedArgs args)
		{
			TreeIter it;
			nutrientsView.Selection.GetSelected (out it);
		}
	}
}
