using System;
using Gtk;
using Foodolini.BusinessLogic;
using Foodolini.Activities;
using Foodolini.Activities.FoodRegistration;
using System.Collections.Generic;

namespace Foodolini.Activities.Inventory
{
	/// <summary>
	/// The Main Inventoy Activity display the FoodItem in the storage to the user 
	/// and allow the user to consume, edit or delete a selected FoodItem. 
	/// </summary>	
	[FoodoliniActivity("Inventory",false,"Frigde.png")]
	[System.ComponentModel.ToolboxItem(true)]
	public partial class InventoryActivity : Gtk.Bin, IActivity
	{
		private NodeStore storage = new NodeStore (typeof(FoodItemAdapter));
		
		/// <summary>
		/// creates a new Inventory Activity 
		/// </summary>
		public InventoryActivity ()
		{
			this.Build ();
			this.Shown += HandleShown;
			this.Editbnt.Sensitive = false;	
			this.Deletebnt .Sensitive = false;
			this.ConsumeBnt.Sensitive = false;
			
			this.foodItemView.AppendColumn ("Food", new CellRendererText (), "text", 0);
			this.foodItemView.AppendColumn ("Quantity (g)", new CellRendererText (), "text", 1);
			this.foodItemView.AppendColumn ("Expiration date", new CellRendererText (), (column, cell, node) =>
			{
				FoodItemAdapter n = (FoodItemAdapter)node;
				CellRendererText c = (CellRendererText)cell;
				c.Text = n.ExpirationDate.ToShortDateString();
			});

			this.foodItemView.AppendColumn ("Open/Closed", new CellRendererText(), "text",  3);
			this.foodItemView.NodeStore = storage;
		}

		private IOwner owner;
		public void Register (IOwner owner){
			this.owner = owner;
		}
		
		/// <summary>
		/// creates the treeview and updates the displayed data. 
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/>
		/// </param>
		void HandleShown(object sender, EventArgs e)
		{
			storage.Clear();  
			var foodItems = new List<FoodItem>(FoodItem.ListFoodItems ());
			foodItems.Sort((f1, f2) => f1.ExpirationDate.CompareTo(f2.ExpirationDate));
			foreach (var item in foodItems)
				storage.AddNode (new FoodItemAdapter (item));
		}


		public void Unregister (){
			this.owner = null;
		}
		
		/// <summary>
		/// push a FoodRegistartion activity. Were the user can registere new fooditem and then return to the inventory. 
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnRegisterBtnClicked (object sender, System.EventArgs e)
		{	
			this.owner.PushActivity<FoodRegistrationActivity>();  	
		}
		
		/// <summary>
		/// opens delete dialog.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnDeletebntClicked (object sender, System.EventArgs e)
		{
			OpenDeleteDialog(); 
		}
		
		/// <summary>
		/// creates a EditDialog for the selected FoodItem were the user can edit the FoodItem. 
		/// </summary>
		private void OpenEditDialog(){
			if(this.foodItemView.NodeSelection.SelectedNode != null){
				var node = (FoodItemAdapter)this.foodItemView.NodeSelection.SelectedNode;
					if (node.FoodItem != null){
						EditFoodItemDialog dia = new EditFoodItemDialog(node.FoodItem); 
						ResponseType res = (ResponseType)dia.Run();
		
		            	if (res == ResponseType.Ok)
							dia.Food.Save(); 
						else
							dia.DontSave();  
						
						dia.Destroy();
					}
			}
			UpdateFoodItem();
		}

		/// <summary>
		/// open edit dialog 
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnEditbntClicked (object sender, System.EventArgs e)
		{
			OpenEditDialog();
		}
		
		/// <summary>
		/// creates a delete FoodItem dialog for the selected FoodItem were the user can delete the FoodItem. 
		/// </summary>	
		private void OpenDeleteDialog(){
			var food = (FoodItemAdapter)this.foodItemView.NodeSelection.SelectedNode;
			if (food != null){ 
				DeleteDialog dia = new DeleteDialog();
				
					ResponseType res = (ResponseType)dia.Run();
	
	            	if (res == ResponseType.Ok){
						storage.RemoveNode(food);
						food.FoodItem.Delete();
					}
				dia.Destroy();  
			}
			UpdateFoodItem();
		}
		/// <summary>
		/// updates the sensitivity of the buttons on the screen.
		/// so that the user cant use them if the system can not perform the action at the given time. 
		/// </summary>
		private void UpdateGrey()
		{
			var food = (FoodItemAdapter)this.foodItemView.NodeSelection.SelectedNode;
			if (food != null){ 
				this.Editbnt.Sensitive = true;	
				this.Deletebnt.Sensitive = true;
				this.ConsumeBnt.Sensitive = true;
			}else{
				this.Editbnt.Sensitive = false;	
				this.Deletebnt.Sensitive = false;
				this.ConsumeBnt.Sensitive = false;
			}
		}
		
		/// <summary>
		/// creates a consume fooditem dialog were the user can select an amount to consume. 
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnConsumeBntClicked (object sender, System.EventArgs e)
		{
			var food = (FoodItemAdapter)this.foodItemView.NodeSelection.SelectedNode;
			if (food != null){ 
				ConsumeFoodItem dia = new ConsumeFoodItem(food.FoodItem,owner.CurrentUser);
				
					ResponseType res = (ResponseType)dia.Run();
	
	            	if (res == ResponseType.Ok){
						if(dia.Quantity == food.FoodItem.Quantity){
							food.FoodItem.Consume(owner.CurrentUser); 
						
							food.FoodItem.Save(); 
							//Datetime is a value type (e.g. struct) it's always NOT NULL!!!
								storage.Clear();  
								foreach (var item in FoodItem.ListFoodItems ())
								storage.AddNode (new FoodItemAdapter (item));
								this.foodItemView.NodeStore = storage;
						}else{
							food.FoodItem.Consume(owner.CurrentUser,dia.Quantity); 
							food.FoodItem.Save(); 
						}
					}
				dia.Destroy();  
				UpdateFoodItem();
				
			}
		}
		
		protected virtual void OnFoodItemViewCursorChanged (object sender, System.EventArgs e)
		{
			UpdateFoodItem();
		}
		
		private void UpdateFoodItem(){
			if (foodItemView.NodeSelection.SelectedNode != null ){
				FoodItemAdapter item = (FoodItemAdapter)foodItemView.NodeSelection.SelectedNode;
				this.lbAmount.Text  = string.Format("Quantity: {0} g",item.FoodItem.Quantity);
				TimeSpan timeRemaining = item.FoodItem.ExpirationDate - System.DateTime.Now;   
				if(timeRemaining.Days < 0) {
					this.lbExpire.Markup  = "<i>Expired</i>";
				} else
					this.lbExpire.Text  = string.Format("Expires in {0} days",Math.Ceiling(timeRemaining.TotalDays));
				this.lbIngre.Markup  = string.Format("<b>{0}</b>",item.FoodItem.Ingredient.LongDescription);
				this.lbshelf.Text  = string.Format("Shelf life: {0} days", item.FoodItem.Ingredient.ShelfLife.Days);   
			}
			else{
				this.lbAmount.Text = "";
				this.lbExpire.Text = "";
				this.lbIngre.Markup = "";
				this.lbshelf.Text = "";
			}
			UpdateGrey();
		}
			
		protected virtual void OnFoodItemViewRowActivated (object o, Gtk.RowActivatedArgs args)
		{
			OpenEditDialog();
		}
	


		public Widget Widget {
			get { return this; }
		}
	}
}
