using System;
using System.Collections.Generic;
using System.Text;
using Foodolini.Database; 

namespace Foodolini.BusinessLogic
{
    /// <summary>
    /// A FoodItem, representing a physical instance of milk.
    /// </summary>
    /// <remarks>
    /// FoodItem contains: 
    ///     an ID number used as a unique identification number when searching the database,
    ///     a quantity defining the amount of the fooditem,
    ///     an Ingredient, an ExpirationDate, an IsOpen for when it was opened, a ConsumedDate, 
    ///     and a Person who has comsumed the foodItem.
    /// </remarks>
    public class FoodItem
    {
		/// <summary>
		/// The row that this FoodItem represents 
		/// </summary>
		private FoodItemRow row;
		
		/// <summary>
		/// True, if this object is modified with respect to the database 
		/// </summary>
		private bool modified = true;
		
        /// <summary>
        /// Creates a new foodItem.
        /// </summary>
        public FoodItem(double quantity, Ingredient ingredient, DateTime expirationDate, bool isOpen){
			this.row = new FoodItemRow();
            this.row.Quantity = quantity;
            this.ingredient = ingredient;
            this.row.ExpirationDate = expirationDate;
            this.row.IsOpen = isOpen;
            this.row.ConsumedDate = null;
            this.row.ConsumedBy = null;
            this.row.RegisteredDate = DateTime.Now;
			this.modified = true;
        }

        public FoodItem(double quantity, Ingredient ingredient, DateTime expirationDate, bool isOpen, DateTime consumedDate, Person consumedBy){
			this.row = new FoodItemRow();
            this.row.Quantity = quantity;
            this.ingredient = ingredient;
            this.row.ExpirationDate = expirationDate;
            this.row.IsOpen = isOpen;
            this.row.ConsumedDate = consumedDate;
            this.consumedBy = consumedBy;
            this.row.RegisteredDate = DateTime.Now;
			this.modified = true;
        }
		
		/// <summary>
		/// Create a new instance of FoodItem from underlying FoodItemRow 
		/// </summary>
		internal FoodItem(FoodItemRow row){
			this.row = row;
			this.modified = false;
		}
		
        /// <summary>
        /// Gets/sets the Identifier as a <see cref="System.Int64"/>.
        /// </summary>
        public long Id
        {
            get{
				return this.row.FoodItemRowId;
			}
        }
		
		/// <summary>
		/// Amount if content in the FoodItem in Gram 
		/// </summary>
        public double Quantity
        {
            get{
				return this.row.Quantity;
			}
            set{
				this.modified = true;
				this.row.Quantity = value;
			}

        }
		
		/// <summary>
		/// Lazily loaded ingredient 
		/// </summary>
		private Ingredient ingredient = null;
		
		/// <summary>
		/// Get the ingredient this FoodItem consists of
		/// </summary>
		/// <remarks>
		/// This property is lazily loaded
		/// </remarks>
        public Ingredient Ingredient
        {
            get{
				if(this.ingredient == null){
					this.ingredient = new Ingredient(this.row.FoodDescriptionId);
				}
				return this.ingredient;
			}
            set{
				
				this.ingredient = value;
			}
        }
		
		/// <summary>
		/// Gets/sets the expiration date of this FoodItem
		/// </summary>
        public DateTime ExpirationDate{
            get{
				return this.row.ExpirationDate;
			}
            set{
				this.modified = true;
				this.row.ExpirationDate = value;
			}
        }

        /// <summary>
        /// When the FoodItem was registered
        /// </summary>
        public DateTime RegisteredDate
        {
            get { return this.row.RegisteredDate; }
            set
            {
                this.modified = true;
                this.row.RegisteredDate = value;
            }
        }
		
		/// <summary>
		/// Gets if this FoodItem have been opened 
		/// </summary>
        public bool IsOpen{
            get{
				return this.row.IsOpen;
			}
        }
		
		/// <summary>
		/// Gets the consumed date
		/// </summary>
		/// <remarks>Throws an exception if not consumed</remarks>
        public DateTime ConsumedDate{
            get{
				if(this.row.ConsumedDate == null)
					throw new Exception("This FoodItem is not consumed");
				return this.row.ConsumedDate.Value;
			}
		}
		
		/// <summary>
		/// Gets the person who consumed this item, null if not consumed 
		/// </summary>
        public Person ConsumedBy
        {
            get{
				if(this.consumedBy == null){
					if(this.row.ConsumedBy == null)
						return null;
					this.consumedBy = Person.GetById(this.row.ConsumedBy.Value);
				}
				return this.consumedBy;
			}
		}
		
		/// <summary>
		/// True, if this FoodItem have been consumed 
		/// </summary>
		public bool IsConsumed{
			get{
				return this.row.ConsumedBy != null && this.row.ConsumedDate != null;
			}
		}
		
		/// <summary>
		/// Lazy loaded, reference holder for Person
		/// </summary>
		private Person consumedBy = null;
		
		/// <summary>
		/// Mark this FoodItem as consumed and opened
		/// </summary>
		/// <remarks>FoodItem.ComsumedDate will be now.</remarks>
		/// <param name="person">
		/// The <see cref="Person"/> who consumed this FoodItem
		/// </param>
		public void Consume(Person person){
			Consume(person, this.Quantity);
		}
		/// <summary>
		///partial consume an item 
		/// </summary>
		/// <param name="person">
		/// A <see cref="Person"/>
		/// </param>
		/// <param name="quantity">
		/// A <see cref="System.Double"/>
		/// </param>
		public void Consume(Person person, double quantity){
			this.modified = true;
			if (quantity < this.Quantity ){
				FoodItem partition = this.Split(quantity);
				if(person != null)
					partition.row.ConsumedBy = person.Id;
				partition.row.ConsumedDate = DateTime.Now;
				partition.OpenFoodItem(); 
				partition.Save();
			}else{
				if(person != null)
					this.row.ConsumedBy = person.Id;
				this.row.ConsumedDate = DateTime.Now;
			} 
			this.OpenFoodItem();
			this.Save();
		}
		
		/// <summary>
        /// Opens the FoodItem and changes the expiration date if it is effected by the opening 
        /// </summary>
        public void OpenFoodItem(){
			if (!this.IsOpen){
				this.modified = true;
	            this.row.IsOpen = true;
					
	            if (System.DateTime.Now + this.Ingredient.ExpirationAfterOpening < this.ExpirationDate) {
	                this.ExpirationDate = System.DateTime.Now + this.Ingredient.ExpirationAfterOpening;
	            }
			}
        }
		
        public void CloseFoodItem(){
			if (this.IsOpen){
				this.modified = true;
	            this.row.IsOpen = false;
	            this.ExpirationDate = this.row.RegisteredDate + this.Ingredient.ShelfLife;

			}
        }
		
        /// <summary>
        /// Split this FoodItem in two
        /// </summary>
        /// <remarks>Remember to save both FoodItems</remarks>
        /// <param name="quantity">
        /// Amount of the FoodItem to put in to the new FoodItem in grams
        /// </param>
        /// <returns>
        /// The new FoodItem created by splitting this in two
        /// </returns>
        public FoodItem Split(double quantity){
            if(quantity == this.Quantity) {  //return one self as the split results in only one FoodItem left
                throw new Exception("Quantity in FoodItem equals quantity for split"); //Throw exception: we cannot split something into one thingss
            } else if (quantity < this.Quantity) { //creates a new FoodItem from the split and return it
                this.Quantity -= quantity;
				this.modified = true;
				this.OpenFoodItem();
				if(this.IsConsumed)
                	throw new Exception("Cannot split consumed FoodItem");
				else
					return new FoodItem(quantity, this.Ingredient, this.ExpirationDate, this.IsOpen);
            } else {
                throw new Exception("split more than we have");	//bad stuff happens
            }
        }


        /// <summary>
        /// Save this FoodItem to database
        /// </summary>
        public void Save(){
			//Save person if loaded
			if(this.consumedBy != null){
				this.consumedBy.Save();
				//Marked modified if person changed
				if(this.row.ConsumedBy != this.consumedBy.Id){
					this.modified = true;
					this.row.ConsumedBy = this.consumedBy.Id;
				}
			}
			//Save ingredient if loaded
            if(this.ingredient != null){ 
				this.ingredient.Save();
				//Marked modified if ingredient changed
				if(this.row.FoodDescriptionId != this.ingredient.Id){
					this.modified = true;
					this.row.FoodDescriptionId = this.ingredient.Id;
				}
			}
			
			//Add/Update this if needed
			if(this.row.FoodItemRowId == 0){
				Settings.Repo.Add<FoodItemRow>(this.row); 
				Settings.Repo.Commit();  
			
			}
			else if(this.modified)
				Settings.Repo.Update<FoodItemRow>(this.row);
			//Once saved mark it unmodified
			this.modified = false;
			
        }
		
		/// <summary>
		/// Delete this FoodItem from database 
		/// </summary>
		public void Delete(){
			if(this.row.FoodItemRowId != 0)
				Settings.Repo.Delete<FoodItemRow>(this.row);
			this.modified = true;
		}
		
		/// <summary>
		/// List FoodItems on storage that haven't been consumed 
		/// </summary>
		/// <returns>
		/// All FoodItems stored that have not been consumed.
		/// </returns>
		public static IEnumerable<FoodItem> ListFoodItems(){
			var query = Settings.Repo.Where<FoodItemRow>("ConsumedBy is NULL AND ConsumedDate is NULL");
			foreach(var item in query)
				yield return new FoodItem(item);
		}
    }
}
