using System;
using System.Collections.Generic;
using System.Text;
using Foodolini.Database;

namespace Foodolini.BusinessLogic
{
    /// <summary>
    /// A nutrition definition.
    /// </summary>
    /// <remarks>
    /// There only exists a few NutritionDefinitionRows in the database, but these are often used.
    /// Therefore this class is lazily caches a static list of all NutritionDefinitions and returns these
    /// using the factory methods. This approach used to minimize required database queries.
    /// 
    /// Also note that new NutritionDefinitionRows cannot be created, and existing ones cannot be changed.
    /// </remarks>
	public class Nutrient
	{
        private NutritionDefinition nutRow;
		
		/// <summary>
		/// Construction a Nutrient
		/// </summary>
		private Nutrient(NutritionDefinition nutRow){
			this.nutRow = nutRow;
		}
		
		internal long Id
		{
			get
			{
                return this.nutRow.NutritionDefinitionId;
			}
		}

        /// <summary>
        /// Measuring unit.
        /// </summary>
		public string Unit
		{
			get
			{
                return this.nutRow.Unit;
			}
		}

        /// <summary>
        /// Abbreviated name.
        /// </summary>
		public string TagName
		{
			get
			{
                return this.nutRow.TagName;
			}
		}

        /// <summary>
        /// Full name of the nutrition definition.
        /// </summary>
		public string Description
		{
			get
			{
                return this.nutRow.Description;
			}
		}

        /// <summary>
        /// Sorder order for listing nutritional values.
        /// </summary>
        public int SortOrder { 
			get {
				return this.nutRow.SortOrder;
			} 
		}

		/// <summary>
		/// Create a Nutrient for a NutritionDefinitionRowId
		/// </summary>
		/// <remarks>
		/// This uses a static lazily loaded list to avoid querying the database.
		/// </remarks>
		internal static Nutrient CreateNutritionDefinition(long nutritionDefinitionRowId){
			LoadDefinitions();
			if(!definitions.ContainsKey(nutritionDefinitionRowId))
				throw new Exception("NutritionDefinitionId " + nutritionDefinitionRowId + " unknown!");
			return definitions[nutritionDefinitionRowId];
		}
		
		
		/// <summary>
		/// Lazily loaded dictionary of NutritionDefinitions, this is loaded by <see cref="Nutrient.LoadDefinitions"/> 
		/// </summary>
		private static Dictionary<long, Nutrient> definitions = null;
		
		/// <summary>
		/// Load NutritionDefinitions if not already loaded 
		/// </summary>
		private static void LoadDefinitions(){
			if(definitions == null){
				definitions = new Dictionary<long, Nutrient>();
				var query = Settings.Repo.Query<NutritionDefinition>("SELECT * FROM NutritionDefinitions ORDER BY SortOrder");
				foreach(var nutDef in query)
					definitions.Add(nutDef.NutritionDefinitionId, new Nutrient(nutDef));
			}
		}

        /// <summary>
        /// Gets total calories.
        /// </summary>
        /// <remarks>Id = 208</remarks>
        public static Nutrient Calories
        {
            get { return FindNutrient("ENERC_KCAL"); }
        }

        /// <summary>
        /// Gets total protein.
        /// </summary>
        /// <remarks>Id = 203</remarks>
        public static Nutrient Protein
        {
            get { return FindNutrient("PROCNT"); }
        }

        /// <summary>
        /// Gets total carbohydrates.
        /// </summary>
        /// <remarks>Id = 205</remarks>
        public static Nutrient Carbohydrates
        {
            get { return FindNutrient("CHOCDF"); }
        }

        /// <summary>
        /// Gets total fat.
        /// </summary>
        /// <remarks>Id = 204</remarks>
        public static Nutrient Fat
        {
            get { return FindNutrient("FAT"); }
        }

        /// <summary>
        /// Searches for a specific Nutrient by its TagName and returns it.
        /// </summary>
        /// <param name="tagName"></param>
        /// <returns>Nutrient</returns>
        private static Nutrient FindNutrient(string tagName)
        {
            return new List<Nutrient>(ListNutritionDefinitions()).Find(n => string.Compare(n.TagName, tagName, true) == 0);
        }
		
		/// <summary>
		/// Returns a collection of NutritionalDefinitions in sorted order
		/// </summary>
		/// <returns>
		/// A <see cref="ICollection<Nutrient>"/> sorted by SortOrder
		/// </returns>
		public static ICollection<Nutrient> ListNutritionDefinitions(){
			LoadDefinitions();
			return definitions.Values;
		}
	}
}
