using System;
using System.Collections.Generic;
using System.Text;
using Foodolini.Database;

namespace Foodolini.BusinessLogic
{
    /// <summary>
    /// A recipe.
    /// </summary>
    public partial class Recipe
    {
        /// <summary>
        /// Database-related object.
        /// </summary>
        private RecipeRow row;

        /// <summary>
        /// True, if this object have been modified with respect to the database 
        /// </summary>
        private bool modified = true;

        /// <summary>
        /// Create a recipe from an underlying row, assumed to be unchanged 
        /// </summary>
        /// <param name="row">
        /// Underlying row, assumed unmodified.
        /// </param>
        private Recipe(RecipeRow row)
        {
            this.row = row;
            this.modified = false;
        }

        public Recipe(string title)
        {
            this.row = new RecipeRow();
            this.row.Title = title;
            this.modified = true;
        }

        public Recipe()
        {
            this.row = new RecipeRow();
            this.modified = false;
        }

        /// <summary>
        /// The name of the recipe.
        /// </summary>
        public string Title
        {
            get { return this.row.Title; }
            set
            {
                this.modified = true;
                this.row.Title = value;
            }
        }

        internal long Id { get { return this.row.RecipeRowId; } }

        private IDictionary<Ingredient, double> ingredients = null;

        /// <summary>
        /// List of ingredients included in the recipe and amount in grams.
        /// </summary>
        public IDictionary<Ingredient, double> Ingredients
        {
            //Note this have type of IDictionary, so that it can easily be changed later, if we want to optimize it
            get
            {
                if (this.ingredients == null) {
                    this.ingredients = new Dictionary<Ingredient, double>();
                    if (this.row.RecipeRowId != 0) {
                        var query = Settings.Repo.Where<RecipeFoodMapping>("RecipeId = @0", this.row.RecipeRowId);
                        foreach (var row in query)
                            this.ingredients.Add(new Ingredient(row.FoodDescriptionId), row.Quantity);
                    }
                }
                return this.ingredients;
            }
            set
            {
                this.ingredients = value;
            }
        }

        public void ClearIngredients()
        {
            this.ingredients.Clear();
            Settings.Repo.DeleteWhere<RecipeFoodMapping>("RecipeId = @0", this.row.RecipeRowId);
        }


        public void UpdateCategories()
        {
            this.tags.Clear();
            this.LoadCachedTags();
            this.tags = new List<string>();
            foreach (var tag in this.cachedTags)
                this.tags.Add(tag.Title);

        }

        /// <summary>
        /// Steps if loaded 
        /// </summary>
        private List<string> directions;

        /// <summary>
        /// Lazy cached steps
        /// </summary>
        /// <remarks>Not loaded if user imidiately overwrites all directions, but setting Directions.</remarks>
        private List<RecipeStep> cachedSteps = null;

        /// <summary>
        /// Load recipe steps to cachedSteps 
        /// </summary>
        private void LoadSteps()
        {
            this.cachedSteps = new List<RecipeStep>();
            if (this.row.RecipeRowId != 0) {
                var query = Settings.Repo.Where<RecipeStep>("recipeid = @0 order by sortorder", this.row.RecipeRowId);
                foreach (var step in query)
                    this.cachedSteps.Add(step);
            }
        }

        /// <summary>
        /// Cooking directions for the recipe.
        /// </summary>
        public List<string> Directions
        {
            get
            {
                // Lazy load directions, and cache steps to track changes.
                if (this.directions == null) {
                    this.directions = new List<string>();
                    this.LoadSteps();
                    foreach (var step in this.cachedSteps)
                        this.directions.Add(step.Directions);
                }
                return directions;
            }
            set
            {
                this.directions = value;
            }
        }

        /// <summary>
        /// The average rating from all votes of the recipe.
        /// </summary>
        public double AverageRating
        {
            get { return row.AverageRating.GetValueOrDefault(); }
        }

        /// <summary>
        /// Lazy loaded ratings dictionary 
        /// </summary>
        private RatingDictionary ratings = null;

        /// <summary>
        /// Gets a dictionary containing persons, who have 
        /// rated the recipe, including their respective rating.
        /// </summary>
        public IDictionary<Person, double> Ratings
        {
            get
            {
                if (this.ratings == null)
                    this.ratings = new RatingDictionary(this.row.RecipeRowId);
                return this.ratings;
            }
        }

        /// <summary>
        /// The specified person rates the recipe.
        /// </summary>
        /// <param name="user">The rater</param>
        /// <param name="rating">The rating</param>
        public void Rate(Person user, double rating)
        {
            this.Ratings.Add(user, rating);

            double temp = 0;
            foreach (var rat in this.ratings.Values) {
                temp += rat;
                Console.WriteLine(temp);
            }

            if (this.ratings.Count == 0)
                this.row.AverageRating = temp;
            else
                this.row.AverageRating = temp / this.ratings.Count;
            this.modified = true;
        }

        /// <summary>
        /// The difficulty of the recipe.
        /// </summary>
        public Difficulty Difficulty
        {
            get { return (Difficulty)Enum.Parse(typeof(Difficulty), this.row.Difficulty.ToString()); }
            set
            {
                this.modified = true;
                this.row.Difficulty = (int)value;
            }
        }

        /// <summary>
        /// The amount of time required to cook the recipe.
        /// </summary>
        public TimeSpan PreparationTime
        {
            get { return TimeSpan.FromTicks(this.row.PreparationTime.GetValueOrDefault()); }
            set
            {
                this.modified = true;
                this.row.PreparationTime = value.Ticks;
            }
        }

        /// <summary>
        /// Lazily loaded list of cached tags 
        /// </summary>
        private List<RecipeTag> cachedTags = null;

        /// <summary>
        /// Load cached tags 
        /// </summary>
        private void LoadCachedTags()
        {
            this.cachedTags = new List<RecipeTag>();
            if (this.row.RecipeRowId != 0) {
                var query = Settings.Repo.Where<RecipeTag>("RecipeRowId = @0", this.row.RecipeRowId);
                foreach (var row in query)
                    this.cachedTags.Add(row);
            }
        }

        /// <summary>
        /// Tags assigned to this Recipe 
        /// </summary>
        /// <remarks>Lazily loaded or assigned</remarks>
        private IList<string> tags = null;

        /// <summary>
        /// The associated categories for the recipe.
        /// </summary>
        public IList<string> Categories
        {
            //Note: The type of this property is IList<string> so that in the future it may be subject to optimization
            get
            {
                if (this.tags == null) {
                    this.LoadCachedTags();
                    this.tags = new List<string>();
                    foreach (var tag in this.cachedTags)
                        this.tags.Add(tag.Title);
                }
                return this.tags;
            }
            set
            {
                this.tags = value;
            }
        }

        /// <summary>
        /// Lazy loaded underlying picture row 
        /// </summary>
        private Picture picture = null;

        /// <summary>
        /// True, if the underlying lazy loaded picture row have been changed with respect to the database 
        /// </summary>
        private bool pictureModified = false;

        /// <summary>
        /// A picture of the recipe.
        /// </summary>
        /// <remarks>You cannot reuse this memory, if you wish to alter the image you must do a set operation!</remarks>
        public byte[] Picture
        {
            get
            {
                if (this.row.PictureId == 0)
                    return null;
                if (this.picture == null) {
                    this.picture = Settings.Repo.SingleWhere<Picture>("PictureId = @0", this.row.PictureId);
                    if (this.picture == null) {
                        this.row.PictureId = 0;
                        this.modified = true;
                        return null;
                    }
                }
                return this.picture.Image;
            }
            set
            {
                this.pictureModified = true;
                if (this.picture == null)
                    this.picture = new Picture();
                this.picture.Image = value;
            }
        }

        /// <summary>
        /// The number of servings produced by the recipe.
        /// </summary>
        public double Servings
        {
            get { return this.row.Servings; }
            set
            {
                this.modified = true;
                this.row.Servings = value;
            }
        }

        /// <summary>
        /// Saves changes to the recipe.
        /// </summary>
        public void Save()
        {
            // Save picture
            if (this.pictureModified) {
                //Update or Add picture to the database
                if (this.picture.PictureId != 0)
                    Settings.Repo.Update<Picture>(this.picture);
                else
                    Settings.Repo.Add<Picture>(this.picture);
                //Mark picture not modified
                this.pictureModified = false;
                //Update picuted id to match the possibly new one
                this.row.PictureId = this.picture.PictureId;
            }

            // Insert or update the Recipe.
            if (row.RecipeRowId == 0)
                Settings.Repo.Add(this.row);
            else if (this.modified)
                Settings.Repo.Update(this.row);

            // Insert or update directions, if these may have been changed
            if (this.directions != null) {
                //Load steps if not needed
                if (this.cachedSteps == null)
                    this.LoadSteps();

                //Update/Add rows as needed
                for (int i = 0; i < this.directions.Count; i++) {
                    //If a cached step exist but it doesn't match, update it
                    if (i < this.cachedSteps.Count && this.cachedSteps[i].Directions != this.directions[i]) {
                        this.cachedSteps[i].Directions = this.directions[i];
                        Settings.Repo.Update<RecipeStep>(this.cachedSteps[i]);
                        //If a cached step doesn't exist add one
                    } else if (i >= this.cachedSteps.Count) {
                        RecipeStep step = new RecipeStep();
                        step.Directions = this.directions[i];
                        step.RecipeId = this.row.RecipeRowId;
                        step.SortOrder = i;
                        Settings.Repo.Add<RecipeStep>(step);
                        this.cachedSteps.Add(step);
                    }
                }
                //Delete rows if there were too many
                for (int i = this.directions.Count; i < this.cachedSteps.Count; i++)
                    Settings.Repo.Delete<RecipeStep>(this.cachedSteps[i]);
                //Remove the deleted rows from cache
                if (this.directions.Count < this.cachedSteps.Count)
                    this.cachedSteps.RemoveRange(this.directions.Count, this.cachedSteps.Count - this.directions.Count);
            }

            // Insert or update categories, if these may have changed
            if (this.tags != null) {
                //Load tags if not loaded
                if (this.cachedTags == null)
                    this.LoadCachedTags();

                //Delete tags that doesn't exist anymore
                List<RecipeTag> deleted = new List<RecipeTag>();
                List<string> needNotAdd = new List<string>();
                foreach (var tag in this.cachedTags) {
                    //If not a tag, it must be deleted
                    if (!this.tags.Contains(tag.Title)) {
                        Settings.Repo.Delete<RecipeTag>(tag);
                        deleted.Add(tag);
                        //If it is a tag and is cached need not be considered for Add
                    } else
                        needNotAdd.Add(tag.Title);

                }
                this.cachedTags.RemoveAll(deleted.Contains);

                //Add new tags
                List<string> newTags = new List<string>(this.tags);
                newTags.RemoveAll(needNotAdd.Contains);
                //Add new tags
                foreach (string tag in newTags) {
                    RecipeTag t = new RecipeTag();
                    t.RecipeRowId = this.row.RecipeRowId;
                    t.Title = tag;
                    Settings.Repo.Add<RecipeTag>(t);
                    this.cachedTags.Add(t);
                }
            }

            // Insert or update ingredients
            if (this.ingredients != null) {
                //Delete all relations in the database
                if (this.row.RecipeRowId != 0)
                    Settings.Repo.DeleteWhere<RecipeFoodMapping>("RecipeId = @0", this.row.RecipeRowId);
                foreach (KeyValuePair<Ingredient, double> entry in this.ingredients) {
                    RecipeFoodMapping map = new RecipeFoodMapping();
                    entry.Key.Save();
                    map.FoodDescriptionId = entry.Key.Id;
                    map.RecipeId = this.row.RecipeRowId;
                    map.Quantity = entry.Value;
                    Settings.Repo.Add<RecipeFoodMapping>(map);
                }
            }

            // Save ratings
            if (this.ratings != null)
                this.ratings.Save(this.row.RecipeRowId);

            //Mark this as unmodified, with respect to the database
            this.modified = false;
        }

        /// <summary>
        /// Loads a single recipe by id.
        /// </summary>
        /// <param name="recipeId"></param>
        /// <returns></returns>
        internal static Recipe GetById(long recipeId)
        {
            return new Recipe(Settings.Repo.SingleWhere<RecipeRow>(" RecipeRowId = @0", recipeId));
        }

        public FoodItem Cook(double servingsMultiplier)
        {
            return Cook(servingsMultiplier, 100);
        }

        /// <summary>
        /// Cooks the recipe and returns the resulting FoodItem.
        /// asume that all the FoodItems is pressent.
        /// </summary>
        /// <param name="servingsMultiplier">The number of servings to prepare</param>
        /// <returns>FoodItem</returns>
        public FoodItem Cook(double servingsMultiplier, double percentage)
        {
            double totalCalories = 0;
            double totalCarbs = 0;
            double totalFat = 0;
            double totalProtein = 0;
            double totalWeight = 0;

            List<FoodItem> inventory = new List<FoodItem>(FoodItem.ListFoodItems());

            foreach (var item in this.Ingredients) {
                // Quantity must be divided by 100, since 1 x ingredient = 100g
                double quantity = item.Value;
                Ingredient ingredient = item.Key;

                //Update fooditem
                foreach (FoodItem fooditem in inventory) {
                    if (Ingredient.Compare(fooditem.Ingredient, ingredient) & fooditem.ConsumedBy == null) {
                        fooditem.Split(quantity * servingsMultiplier);
                        fooditem.Save();
                        break;
                    }
                }

                // Get the calories for this ingredient and multiply by quantity and servings
                // NOTE: Nutritional values for ingredients are always per 100 g
                double calories = (ingredient.Nutrients[Nutrient.Calories] * quantity);
                double protein = (ingredient.Nutrients[Nutrient.Protein] * quantity); 
                double carbs = (ingredient.Nutrients[Nutrient.Carbohydrates] * quantity);
                double fat = (ingredient.Nutrients[Nutrient.Fat] * quantity);

                // increment totals
                totalCalories += calories;
                totalFat += fat;
                totalProtein += protein;
                totalCarbs += carbs;
                // Weight in grams
                totalWeight += item.Value;
            }

            // Get nutritional value for 100g
			double ingcount= this.Ingredients.Count;
            totalCalories /= ingcount;
			totalCalories /= 100.0;
            totalFat /= ingcount;
			totalFat /= 100.0;
            totalProtein /= ingcount;
			totalProtein /= 100;
            totalCarbs /= ingcount;
			totalCarbs  /= 100.0;
            totalWeight *= (percentage / 100); // How much to store

            // Create a new ingredient with the same name as the recipe
            Ingredient newIngredient = new Ingredient(this.Title);
            // Check if it already exists
            Ingredient lookUp = Ingredient.GetByLongDescription(newIngredient.LongDescription);

            if (lookUp != null) {
                newIngredient = lookUp;
                // Update the ingredient, since changes could have been made to the recipe
                newIngredient.Nutrients[Nutrient.Calories] = (double)totalCalories;
                newIngredient.Nutrients[Nutrient.Protein] = (double)totalProtein;
                newIngredient.Nutrients[Nutrient.Carbohydrates] = (double)totalCarbs;
                newIngredient.Nutrients[Nutrient.Fat] = (double)totalFat;
            } else { 
                // Add nutritional details
                // All nutrients could be added if needed...
                newIngredient.Nutrients.Add(Nutrient.Calories, (double)totalCalories);
                newIngredient.Nutrients.Add(Nutrient.Protein, (double)totalProtein);
                newIngredient.Nutrients.Add(Nutrient.Carbohydrates, (double)totalCarbs);
                newIngredient.Nutrients.Add(Nutrient.Fat, (double)totalFat);

                // Recipes are always associated with this category.
                // Not happy about hardcoding a string here for this...
                newIngredient.Category = "Meals, Entrees, and Sidedishes";
            }
			
            newIngredient.Save();

            // default expiration time is two days and fooditem is opened.
            FoodItem result = new FoodItem(totalWeight * servingsMultiplier, newIngredient, DateTime.Now.AddDays(2), true);
            return result;
        }

        /// <summary>
        /// Deletes the recipe permanently.
        /// </summary>
        public void Delete()
        {
            //Delete image
            if (this.picture != null) {
                //Now this must be a new row, because we're deleting the row below
                this.picture.PictureId = 0;
                //Picture is new with respect to the database
                this.pictureModified = true;
            }
            //If there's a picture, delete it
            if (this.row.PictureId != 0)
                Settings.Repo.DeleteWhere<Picture>("PictureId = @0", this.row.PictureId);

            //Delete directions
            if (this.cachedSteps == null)
                this.LoadSteps();
            foreach (var step in this.cachedSteps)
                Settings.Repo.Delete<RecipeStep>(step);
            this.cachedSteps.Clear();

            //Delete all ingredient relations
            Settings.Repo.DeleteWhere<RecipeFoodMapping>("RecipeId = @0", this.row.RecipeRowId);

            //Delete tags and clear tags cache
            if (this.cachedTags != null)
                this.cachedTags.Clear();
            Settings.Repo.DeleteWhere<RecipeTag>("RecipeRowId = @0", this.row.RecipeRowId);

            //Delete ratings, load them if needed
            if (this.ratings == null)
                this.ratings = new RatingDictionary(this.row.RecipeRowId);
            this.ratings.Delete(this.row.RecipeRowId);

            //Delete underlying row
            if (this.row.RecipeRowId != 0)
                Settings.Repo.Delete<RecipeRow>(this.row);

            //Mark this as modified
            this.modified = true;
        }

        private int searchPriority = 0;

        /// <summary>
        /// Gets a list of recipies filtered by the provided criteria.
        /// </summary>
        /// <param name="criteria">Search terms, see Criteria struct</param>
        /// <returns>List of recipies</returns> 
		public static List<Recipe> ListByCriteria(Criteria criteria)
		{			
			//Grab recipes of a specific meal type
			List<RecipeRow> recipeRows = new List<RecipeRow>();
			recipeRows = GetRecipeByMeal(criteria);
			
			//Convert RecipeRow into Recipe
			List<Recipe> recipes = new List<Recipe>();
			foreach(RecipeRow row in recipeRows)
				recipes.Add(new Recipe(row));
			
			//Remove irrelevant recipes
			if(criteria.searchString.Count > 0)
				recipes = RemoveIrrelevantRecipes(recipes, criteria.searchString);
			
			//Prioritise after expiration date
			if(criteria.expirationDate)
				recipes = PrioritiseRecipesAfterDate(recipes, criteria);
			
			//Prioritise after rating
			if(criteria.rating)
				recipes = PrioritiseRecipesAfterRating(recipes, criteria);
			
			//Prioritise after keywords
			if(criteria.searchString.Count > 0)
				recipes = PrioritiseRecipes(recipes, criteria.searchString);
			
			//Sort the list
			recipes = SortRecipes(recipes, 0, recipes.Count-1);
			
			recipes.Reverse();
			
			return recipes;
		}
		
		/// <summary>
		/// Prioritises recipes after their rating. 2.5 stars being middle ground, equalling 0 in priority.
		/// Less gains negative influence, more gains positive
		/// </summary>
		private static List<Recipe> PrioritiseRecipesAfterRating(List<Recipe> recipes, Criteria criteria)
		{
			List<Recipe> returnList = new List<Recipe>();
			returnList = recipes;
			
			foreach(Recipe recipe in returnList)
			{
				recipe.searchPriority += GetRatingPriority(recipe.AverageRating) * criteria.ratingPriority;
			}
		
			return returnList;
		}
		
		/// <summary>
		/// Returns the searchPriority value based on the rating value
		/// </summary>
		private static int GetRatingPriority(double rating)
		{
			//If no rating, return 0
			if(rating == 0.0 || rating == 3.0)
				return 0;
			
			if(rating < 3)
			{
				if(rating < 2)
				{
					if(rating < 1.5)
						return -4;
					else
						return -3;
				}
				else
				{
					if(rating < 2.5)
						return -2;
					else
						return -1;
				}
			}
			else
			{
				if(rating < 4)
				{
					if(rating < 3.5)
						return 1;
					else
						return 2;
				}
				else
				{
					if(rating < 4.5)
						return 3;
					else
						return 4;
				}
			}
		}
		
		/// <summary>
		/// Prioritises recipe after expiration dates. Recipes requiring ingredients 
		/// which expiration date is approaching are rated higher then others.
		/// </summary>
		private static List<Recipe> PrioritiseRecipesAfterDate(List<Recipe> recipes, Criteria criteria)
		{
			List<Recipe> returnList = new List<Recipe>();
			returnList = recipes;
			
			//Check each recipe
			foreach(Recipe recipe in returnList)
			{
				//Check each ingredient per recipe
				foreach(var ingredient in recipe.Ingredients)
				{
					FoodItem item = ingredient.Key.InStorage();
					if(item != null)
						recipe.searchPriority += GetExpirationDatePriority(item.ExpirationDate) * criteria.expirationDatePriority;
				}
			}
			return returnList;
		}
		
		/// <summary>
		/// Returns a searchPriority (int) value based on day count til item expires.
		/// </summary>
		private static int GetExpirationDatePriority(DateTime expirationDate)
		{
			TimeSpan timeTillExpires = expirationDate - DateTime.Now;
			switch((int)Math.Ceiling(timeTillExpires.TotalDays))
			{
			case 1:
				return 6;
			case 2:
				return 4;
			case 3:
				return 3;
			case 4:
				return 2;
			default:
				return 0;
			}
		}
		
		/// <summary>
		/// Sorts the recipe in order after searchPriority.
		/// Uses the Quicksort algorithm, found in Introduction to Algorithms by:
		/// Thomas H. Cormen, Charles E.Leiserson, Ronald L. Rivest and Clifford Stein
		/// </summary>
		private static List<Recipe> SortRecipes(List<Recipe> recipes, int index, int length)
		{
			if(index < length)
			{
				int q = Partition(ref recipes, index, length);
				SortRecipes(recipes, index, q - 1);
				SortRecipes(recipes, q + 1, length);
			}
			
			return recipes;
		}
				
		/// <summary>
		/// Part of the Quicksort algorithm. Rearranges the subarray passed to it
		/// </summary>
		private static int Partition(ref List<Recipe> recipes, int start, int length)
		{
			Recipe recipe = recipes[length];
			int i = start-1;
			for(int j = start; j < length; j++)
			{
				if(recipes[j].searchPriority <= recipe.searchPriority)
				{
					i += 1;
					Recipe tmp = recipes[i];
					recipes[i] = recipes[j];
					recipes[j] = tmp;
				}
			}
			recipes[length] = recipes[i+1];
			recipes[i+1] = recipe;
			
			return i+1;
		}
		
		/// <summary>
		/// Prioritise recipes after keywords
		/// </summary>
		private static List<Recipe> PrioritiseRecipes(List<Recipe> recipes, List<string> keywords)
		{
			List<Recipe> returnList = new List<Recipe>();
			returnList = recipes;
			
			//Search for each keyword in the list
			foreach(string keyword in keywords)	
			{
				foreach(Recipe recipe in returnList)
				{
					//Prioritise after title
					if(recipe.Title.Contains(keyword))
					   recipe.searchPriority += 10;
					
					//Prioritise after tags
					if(recipe.cachedTags == null)		//If tags haven't been loaded, do so
						recipe.LoadCachedTags();
					{
						bool keywordPresent = false;
						foreach(RecipeTag tag in recipe.cachedTags)
						{
							if(tag.Title.Contains(keyword))
							{
								keywordPresent = true;
								break;
							}
						}
						if(keywordPresent)
							recipe.searchPriority += 7;
					}
					
					//Prioritise after directions
					if(recipe.cachedSteps == null)		//If directions haven't been loaded, do so
						recipe.LoadSteps();
					{
						bool keywordPresent = false;
						foreach(RecipeStep step in recipe.cachedSteps)
						{
							if(step.Directions.Contains(keyword))
							{
								keywordPresent = true;
								break;
							}
						}
						if(keywordPresent)
							recipe.searchPriority += 4;
					}
				}
			}
			
			return returnList;
		}
		
		/// <summary>
		/// Removes any recipes that have no reference to the search keyword.
		/// </summary>
		private static List<Recipe> RemoveIrrelevantRecipes(List<Recipe> recipes, List<string> keywords)
		{
			List<Recipe> returnList = new List<Recipe>();
			returnList = recipes;
			
			//Search for each keyword in the list
			foreach(string keyword in keywords)
			{
				//Search title
				foreach(Recipe recipe in returnList)
				{
					//Search title
					if(recipe.Title.ToLower().Contains(keyword.ToLower()))
						recipe.searchPriority += 1;
					else
						recipe.searchPriority -= 1;
					
					//Search directions
					//If directions haven't been loaded yet, do so.
					if(recipe.cachedSteps == null)
						recipe.LoadSteps();
					{
						bool keywordPresent = false;
						//Search the list of steps
						foreach(RecipeStep step in recipe.cachedSteps)
						{
							if(step.Directions.ToLower().Contains(keyword.ToLower()))
							{
								keywordPresent = true;
								break;
							}
						}
						
						//Apply result
						if(keywordPresent)
							recipe.searchPriority += 1;
						else
							recipe.searchPriority -= 1;
					}
					
					//Search tags
					{
						//If tags aren't loaded, do so
						if(recipe.cachedTags == null)
							recipe.LoadCachedTags();
						
						bool keywordPresent = false;
						//Search the list of tags
						foreach(RecipeTag tag in recipe.cachedTags)
						{
							if(tag.Title.ToLower().Contains(keyword.ToLower()))
							{
								keywordPresent = true;
								break;
							}
						}
						
						//Apply result
						if(keywordPresent)
							recipe.searchPriority += 1;
						else
							recipe.searchPriority -= 1;
					}
				}
			}
			
			
			//Remove all recipes with a search priority of keywords.Count * (-3)
			for(int i = 0; i < returnList.Count; i++)
			{
				if(returnList[i].searchPriority == keywords.Count * (-3))
				{
					returnList.RemoveAt(i);
					i--;
				}
				else
					returnList[i].searchPriority = 0;
			}
			
			return returnList;
		}
		
		/// <summary>
		/// Grabs recipes by meal type, such as Dinner or breakfasts
		/// </summary>
		private static List<RecipeRow> GetRecipeByMeal(Criteria criteria)
		{
			List<RecipeRow> recipeRows = new List<RecipeRow>();
			
			if(criteria.Meal == Meal.None)	//Grab everything
			{
				var query = Settings.Repo.All<RecipeRow>();
				foreach(var row in query)
					recipeRows.Add(row);
			}
			else
			{
				//Grab all tags where mealType matches
				for(int i = 0; i < 4; i++) //Check if each mealtype was selected and grab respective meals
				{
					if(((int)criteria.Meal & (int)Math.Pow(2, i)) != 0)
					{
						Meal type = (Meal)(int)Math.Pow(2,i);
						var tagQuery = Settings.Repo.Where<RecipeTag>("title = @0", type.ToString());
						
						//Grab the reciperow for each tag
						foreach(var tag in tagQuery)
						{
							var rowQuery = Settings.Repo.SingleWhere<RecipeRow>("reciperowid = @0", tag.RecipeRowId);
							recipeRows.Add(rowQuery);
						}		
					}
				}
			}
			
			return recipeRows;
		}
		
		/// <summary>
        /// Loads a single Recipe by Title.
        /// </summary>
        /// <param name="description"></param>
        /// <returns></returns>
        public static Recipe GetByTitle(string title)
        {
			RecipeRow row = Settings.Repo.SingleWhere<RecipeRow>("Title = @0", title);
			if(row != null)
            	return new Recipe(row);
			return null;
        }
						
        public override string ToString()
        {
            return base.ToString() + " " + this.Title;
        }
    }

    /// <summary>
    /// The difficulty level of a recipe.
    /// </summary>
    public enum Difficulty
    {
        Easy,
        Medium,
        Hard,
		Unknown
    }
}
