using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Foodolini.Database;

namespace Foodolini.BusinessLogic
{
    /// <summary>
    /// A shopping list item that contains ingredients and recipes.
    /// </summary>
    public class ShoppingListItem
    {
        private ShoppingListItemRow row;
        private bool modified = false;

        public ShoppingListItem(Ingredient ingredient, double quantity, Person owner)
        {
            row = new ShoppingListItemRow();
            this.Ingredient = ingredient;
            this.Quantity = quantity;
            this.Owner = owner;
        }

        public ShoppingListItem(Recipe recipe, Ingredient ingredient, double quantity, Person owner)
        {
            row = new ShoppingListItemRow();
            this.Recipe = recipe;
            this.Ingredient = ingredient;
            this.Quantity = quantity;
            this.Owner = owner;
        }

        internal ShoppingListItem(ShoppingListItemRow row)
        {
            this.row = row;
            this.modified = false;
        }

        /// <summary>
        /// Unique identifer.
        /// </summary>
        internal long Id
        {
            get { return this.row.ShoppingListItemRowId; }
        }

        /// <summary>
        /// The owner of the shopping list item.
        /// </summary>
        public Person Owner
        {
            // Person is already cached.
            get { return Person.GetById(this.row.UserId); }
            set { this.row.UserId = value.Id; }
        }

        /// <summary>
        /// cached recipe instance.
        /// </summary>
        private Recipe recipe;
        /// <summary>
        /// Gets the associated Recipe
        /// </summary>
        public Recipe Recipe
        {
            get {
                // Item may not have a recipe associated.
                if (!this.row.RecipeRowId.HasValue)
                    return null;

                // Lazily load the recipe.
                if (this.recipe == null)
                    recipe = Recipe.GetById(this.row.RecipeRowId.Value);
                return this.recipe;
            }
            set
            {
                modified = true;
                this.row.RecipeRowId = value.Id;
            }
        }

        /// <summary>
        /// The amount of this item
        /// </summary>
        public double Quantity
        {
            get { return this.row.Quantity; }
            set
            {
                modified = true;
                this.row.Quantity = value;
            }
        }

        private Ingredient ingredient;
        /// <summary>
        /// The associated ingredient.
        /// </summary>
        public Ingredient Ingredient
        {
            get { 
                if(this.ingredient==null)
                    ingredient = Ingredient.GetById(this.row.FoodDescriptionId);
                return this.ingredient; 
            }
            set
            {
                modified = true;
                this.row.FoodDescriptionId = value.Id;
            }
        }

        /// <summary>
        /// Loads all shopping list items owned by the provided user.
        /// </summary>
        /// <param name="owner"></param>
        /// <returns></returns>
        static internal IEnumerable<ShoppingListItem> GetShoppingListItems(Person owner)
        {
            foreach (var row in Settings.Repo.Where<ShoppingListItemRow>(" UserId = @0 ", owner.Id)) {
                yield return new ShoppingListItem(row); 
            }
        }

        /// <summary>
        /// Saves the shopping list item to the database.
        /// </summary>
        public void Save() {
            if (this.modified) {
                if (this.Id == 0)
                    Settings.Repo.Add<ShoppingListItemRow>(this.row);
                else
                    Settings.Repo.Update<ShoppingListItemRow>(this.row);
            }
        }

        public void Delete()
        {
            // no need to call Delete() if id is zero
            if (this.Id != 0) {
                Settings.Repo.Delete<ShoppingListItemRow>(this.row);
            }
            this.modified = false;
        }
        
    }
}
