using System;
using System.Data.Common;
using System.Collections.Generic;

/// <summary>
/// Sql generation classes for Repository
/// </summary>
namespace Foodolini.Database.Sql
{
	/// <summary>
	/// An abstract class for building Sql statements for a database
	/// </summary>
	public abstract class SqlStrategy
	{
		/// <summary>
		/// Builds a query that lists all tables
		/// </summary>
		/// <returns>
		/// An sql statement for quering all tables
		/// </returns>
		public abstract string TableQuery();
		
		/// <summary>
		/// Build an Sql statement for creating a table with specified colums
		/// </summary>
		/// <remarks>
		/// This method may throw an SqlTypeException if the type is not supported
		/// </remarks>
		/// <param name="table">
		/// Table to create
		/// </param>
		/// <param name="columns">
		/// A <see cref="IDictionary<System.String, Type>"/> of column name and type
		/// </param>
		/// <returns>
		/// An Sql statement to create the table
		/// </returns>
		public abstract string CreateTableQuery(string table, IDictionary<string,Type> columns, string primaryKey);
		
		/// <summary>
		/// Builds an Sql statement to insert a row into table with parameters
		/// </summary>
		/// <remarks>
		/// The parameters will be added to the query with the keyword @[parameter], where [parameter] is the name of the parameter.
		/// This query MUST also return the id of the row.
		/// </remarks>
		/// <param name="table">
		/// A <see cref="System.String"/> table to insert into
		/// </param>
		/// <param name="parameters">
		/// A <see cref="ICollection<System.String>"/> list of parameters to insert
		/// </param>
		/// <returns>
		/// Sql statement to insert a row
		/// </returns>
		public abstract string InsertQuery(string table, ICollection<string> parameters);
		
		/// <summary>
		/// Builds an Sql statement to update a row in a table with parameters
		/// </summary>
		/// <remarks>
		/// The parameters will be added to the query with the keyword @[parameter], where [parameter] is the name of the parameter.
		/// </remarks>
		/// <param name="table">
		/// A <see cref="System.String"/> Table to update
		/// </param>
		/// <param name="parameters">
		/// A <see cref="ICollection<System.String>"/> parameters to update in the row
		/// </param>
		/// <param name="identifier">
		/// Identifier in the table
		/// </param>
		/// <returns>
		/// An Sql statement to update the row
		/// </returns>
		public abstract string UpdateQuery(string table, ICollection<string> parameters, string identifier);
		
		/// <summary>
		/// Builds a query with a condition
		/// </summary>
		/// <param name="table">
		/// A <see cref="System.String"/> table to execute the query on
		/// </param>
		/// <param name="condition">
		/// The condition to query
		/// </param>
		/// <returns>
		/// An sql statement
		/// </returns>
		public abstract string WhereQuery(string table, string condition);
		
		/// <summary>
		/// Builds a query with a condition and paging
		/// </summary>
		/// <param name="table">
		/// A <see cref="System.String"/> table to execute the query on
		/// </param>
		/// <param name="condition">
		/// The condition to query
		/// </param>
		/// <param name="page">
		/// Page to return sql statement for
		/// </param>
		/// <param name="pageSize">
		/// Number of entries per page
		/// </param>
		/// <returns>
		/// An sql statement
		/// </returns>
		public abstract string PagedWhereQuery(string table, string condition, int page, int pageSize);
		
		/// <summary>
		/// Builds an Sql statement to query all rows in a table
		/// </summary>
		/// <param name="table">
		/// Table to query
		/// </param>
		/// <returns>
		/// Sql statement
		/// </returns>
		public abstract string AllQuery(string table);
		
		/// <summary>
		/// Builds an Sql statement to query all rows in a table paged
		/// </summary>
		/// <param name="table">
		/// Table to query
		/// </param>
		/// <param name="page">
		/// Page to return sql statement for
		/// </param>
		/// <param name="pageSize">
		/// Number of entries per page
		/// </param>
		/// <returns>
		/// Sql statement to query the table paged.
		/// </returns>
		public abstract string PagedQuery(string table, int page, int pageSize);
		
		/// <summary>
		/// Builds an Sql statement to delete a row
		/// </summary>
		/// <param name="table">
		/// Table to delete from
		/// </param>
		/// <param name="identifier">
		/// Identifier key for the table
		/// </param>
		/// <returns>
		/// An sql statement to delete a row
		/// </returns>
		public abstract string DeleteQuery(string table, string identifier);
		
		/// <summary>
		/// Creates an sql statement that deletes everything satisfying a condition
		/// </summary>
		/// <param name="table">
		/// table to delete from
		/// </param>
		/// <param name="condition">
		/// Condition that must be met
		/// </param>
		/// <returns>
		/// An sql statement for deleting
		/// </returns>
		public abstract string DeleteWhereQuery(string table, string condition);
	}
}
