using System;
using Gtk;
using Gdk;
using Foodolini.Activities;
using Foodolini.BusinessLogic;
using Foodolini.Activities.ShoppingList;
using System.Reflection;
using System.IO;
using System.Collections.Generic;

namespace Foodolini.Main{

	public partial class MainWindow : Gtk.Window, IOwner
	{
	    public MainWindow() : base(Gtk.WindowType.Toplevel)
	    {
	        Build();
			
			//Load plugins
			this.LoadPlugins();
			
	        Foodolini.BusinessLogic.Settings.Instance.OpenSqliteDatabase("foodolini.db3");
	
	        this.userCombobox.Changed += this.userCombobox_Changed;
	        this.activityCombobox.Changed += this.activityCombobox_Changed;
	
	        PopulateUserCombobox();
			
			LoadShoppingList();
			
			this.HomeActivity();
	    }
		
		protected virtual void OnHomeButtonClicked (object sender, System.EventArgs e){
			this.HomeActivity();
		}
	
		#region Plugin loading
	
		private List<string> mainActivities;
		private Dictionary<string, ConstructorInfo> plugins;
		private Dictionary<string, Pixbuf> pluginIcons;
		
		private void LoadPlugins(){
			//TODO: Load plugin directories from settings
			this.plugins = new Dictionary<string, ConstructorInfo>();
			this.mainActivities = new List<string>();
			this.pluginIcons = new Dictionary<string, Pixbuf>();
			
			List<string> pluginsPaths = new List<string>();
			pluginsPaths.Add(System.IO.Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location));
			//TODO: Add a path for user installed plugins, e.g. in a home directory
			//TODO: Add a path for thirdparty packaged system-wide plugins
			
			foreach(var pluginPath in pluginsPaths){
				foreach(var file in Directory.GetFiles(pluginPath, "*.dll")){
					try{
						Assembly assembly = Assembly.LoadFile(file);
						foreach(object attr in assembly.GetCustomAttributes(false)){
							if(attr is AssemblyFoodoliniPluginAttribute){
								foreach(Type type in assembly.GetTypes()){
									foreach(var at in type.GetCustomAttributes(false)){
										if(at is FoodoliniActivityAttribute){
											var att = (FoodoliniActivityAttribute)at;
											//TODO: Check if constructor is null
											this.plugins.Add(att.Name, type.GetConstructor(new Type[]{}));
											this.pluginIcons.Add(att.Name, att.GetIcon(assembly));
											if(!att.IsSubActivity)
												this.mainActivities.Add(att.Name);
										}
									}
								}
							}
						}
					}
					catch(Exception e){
						Console.WriteLine("Exception while trying to look for plugins in: " + file + "\n\t" + e.Message);
					}
				}
			}
			
			if(mainActivities.Count == 0)
				throw new Exception("No main activities found!");
		}
		
		#endregion
		
		#region Activity handling
		
		/// <summary>
		/// The Activity stack
		/// </summary>
		private Stack<IActivity> activities = new Stack<IActivity>();
		
		/// <summary>
		/// Gets the name of an activity using reflection, return "Unknown" if attribute isn't present
		/// </summary>
		private string ActivityName(IActivity activity){
			foreach(var at in activity.GetType().GetCustomAttributes(false)){
				if(at is FoodoliniActivityAttribute){
					var att = (FoodoliniActivityAttribute)at;
					return att.Name;
				}
			}
			return "Unknown";
		}
		
		private Widget currentView = null;
		
		/// <summary>
		/// Set the top of the activity stack as active activity, e.g. display it 
		/// </summary>
		void SetActivity(){
			//Remove current activity if needed
			if(this.currentView != null){
				this.contentbox.Remove(this.currentView);
				this.currentView.Hide();  
			}
			
			//Use activity at the top of the stack
			var activity = this.activities.Peek();
			
			//Update the current activity
			this.currentView = activity.Widget;
			
			//Update the combobox accordingly
			this.UpdateActivitiesComboBox();
			
			//Add and show the activity
			this.contentbox.Add(activity.Widget);
			activity.Widget.ShowAll();
		}
		
		/// <summary>
		/// Initialize an activity
		/// </summary>
		/// <param name="parameters">
		/// Parameters for the activity constructor
		/// </param>
		void InitializeActivity(string activity){
			if(!this.HasActivity(activity))
				throw new Exception("Couldn't initialize activity: " + activity);
			//Initiate the activity
			var act = (IActivity)this.plugins[activity].Invoke(new object[]{});
			//Register activity
			act.Register(this);
			
			//Stack the activity
			this.activities.Push(act);
		}
		
		bool HasActivity(string activity){
			return this.plugins.ContainsKey(activity);
		}
		
		/// <summary>
		/// Closes the activity at the top of the activity stack
		/// </summary>
		/// <returns>True, if the activity could be unloaded.</returns>
		bool CloseActivity(){
			if(this.activities.Count == 0)
				throw new Exception("Can't close activity when there's is no activity to close!");
			
			if(this.ActivityUnloading != null){
				var peek = this.activities.Peek();
				var args = new ActivityUnloadedArgs(peek);
				this.ActivityUnloading(this, args);
				if(!args.Unload)
					return false;
			}
			var activity = this.activities.Pop();
			
			//Remove the widget if it is added
			if(this.currentView != null && this.currentView == activity.Widget){
				this.contentbox.Remove(this.currentView);
				this.currentView = null;
			}
			
			activity.Unregister();
			activity.Widget.Destroy();
			return true;
		}
		
		#endregion
		
		#region	Activity combo box
		
		private ListStore activityStore;
		
		/// <summary>
		/// Update the activities combobox to match the top of the stack 
		/// </summary>
		private void UpdateActivitiesComboBox(){
			//Use activity at the top of the stack
			var activity = this.activities.Peek();
			
			//Ignore selection changes to combobox
			ignoreComboBoxChanges = true;
			
			string name = this.ActivityName(activity);
			//Remove all entries, this is just easier that to add/remove
			if(this.activityStore == null){
				this.activityStore = new ListStore(typeof(string), typeof(Pixbuf));
				this.activityCombobox.Model = this.activityStore;
				this.activityCombobox.Clear();
				var cp = new CellRendererPixbuf();
				var ct = new CellRendererText();
				this.activityCombobox.PackStart(cp, false);
				this.activityCombobox.PackStart(ct, true);
				this.activityCombobox.AddAttribute(cp, "pixbuf", 1);
				this.activityCombobox.AddAttribute(ct, "text", 0);
			}else
				this.activityStore.Clear();
			//Add all entries again, keep iter for the current if it's a main activity
			Nullable<TreeIter> active = null;
			foreach(string entry in this.mainActivities){
				var iter = this.activityStore.AppendValues(entry, this.pluginIcons[entry]);
				if(entry == name)
					active = iter;
			}
			//If current wasn't a main activity, add it an get an iter to it
			if(active == null){
				Pixbuf icon = null;
				if(this.pluginIcons.ContainsKey(name))
					icon = this.pluginIcons[name];
				active = this.activityStore.AppendValues(name, icon);
			}
			//Set the current iter active...
			this.activityCombobox.SetActiveIter(active.Value);
			
			//Stop ignoring selection changes to combobox
			ignoreComboBoxChanges = false;
			
			// Queue a resize
			this.activityCombobox.QueueResize();
		}
		
		/// <summary>
		/// True, if the activityCombobox.Changed event should be ignored, e.g. the activityCombobox is manipulated programmatically
		/// </summary>
		private bool ignoreComboBoxChanges = false;
		
		void activityCombobox_Changed(object sender, EventArgs e){
			//Check if we should discard this event
			if(!this.ignoreComboBoxChanges){
				//Get selection
				TreeIter iter;
		        this.activityCombobox.GetActiveIter(out iter);
		        var activity = (string)this.activityCombobox.Model.GetValue(iter, 0);
				//Try to load it
				if(!this.LoadActivity(activity))
					this.UpdateActivitiesComboBox(); //If we can't unload, revert the combobox selection the user made
			}
	    }
		
		#endregion
	
		#region User combo box
		
		/// <summary>
	    /// Notifies subscribers if the current user has changed.
	    /// </summary>
	    void userCombobox_Changed(object sender, EventArgs e){
	        if(this.CurrentUserChanged != null)
	            this.CurrentUserChanged(this, new PersonEventArgs(this.CurrentUser));
	    }
		
		/// <summary>
		/// Store of all users 
		/// </summary>
		private ListStore userStore;
		
	    /// <summary>
	    /// Lists users of the system in the userCombobox.
	    /// </summary>
	    void PopulateUserCombobox(){
			//Get the current user
			Person active = this.CurrentUser;
			if(this.userStore == null){
				this.userStore = new ListStore(typeof(string), typeof(object));
				this.userCombobox.Model = this.userStore;
			}else
				this.userStore.Clear();
			
			//Add all entries
	        TreeIter defaultSelection = userStore.AppendValues("[Anonymous]", null);
	        foreach (var person in Person.ListUsers()) {
	            TreeIter item = this.userStore.AppendValues(person.FullName, person);
	            if (person == active)	//Reference comparation works because Person is fly weight
	                defaultSelection = item;
	        }
	
	        // Assigns the proper width to the combobox
	        this.userCombobox.QueueResize();
	        // Select the current user
	        this.userCombobox.SetActiveIter(defaultSelection);
	    }
	
		#endregion
		
	    private void OnDeleteEvent(object sender, DeleteEventArgs a){
			bool close = true;
			while(this.activities.Count != 0){
				if(!this.CloseActivity()){
					this.SetActivity();
					close = false;
					break;
				}
			}
	        if(close){
			    this.shoppingList.Unregister();
	    		this.shoppingList.Widget.Destroy();
				Foodolini.BusinessLogic.Settings.Instance.Dispose();
		        Application.Quit();
			}
			a.RetVal = close;
	    }
		
		#region IOwner implementation
		
		/// <summary>
		/// Add all ingredients needed for the recipe to the shoppinglist
		/// </summary>
		/// <remarks>
		/// This is all ingredients, and does not depend on what is on storage.
		/// If the nature of the shoppinglist is changed in the future, this may change.
		/// </remarks>
		public void AddRecipe(Recipe recipe, double servings){
			(this.shoppingList as ShoppingListActivity).AddRecipe(recipe, servings);
		}
		
		public void UpdateUserList(){
			this.PopulateUserCombobox();
		}
		
		
		/// <summary>
		/// Load an activity and put on top of the activity stack
		/// </summary>
		/// <param name="activity">
		/// Activity to load
		/// </param>
		/// <returns>
		/// The activity that was added, null if not found
		/// </returns>
		public IActivity PushActivity(string activity){
			if(!this.HasActivity(activity))
				return null;
			this.InitializeActivity(activity);
			this.SetActivity();
			
			return this.activities.Peek();
		}
		
		/// <summary>
		/// Load an activity and put on top of the activity stack
		/// </summary>
		/// <returns>
		/// The activity that was added, null if not found
		/// </returns>
		public T PushActivity<T>() where T : class, new(){
			string name = null;
			foreach(var att in typeof(T).GetCustomAttributes(false)){
				if(att is FoodoliniActivityAttribute){
					name = ((FoodoliniActivityAttribute)att).Name;
				}
			}
			if(name ==  null)
				return null;
			return (T)this.PushActivity(name);
		}
		
		/// <summary>
		/// Unloades the activity stack and load on activity.
		/// </summary>
		/// <param name="activity">
		/// Activity to load
		/// </param>
		/// <returns>
		/// True, if the activity stack could be unloaded
		/// </returns>
		public bool LoadActivity(string activity){
			if(!this.HasActivity(activity))
				return false;
			while(this.activities.Count != 0){
				if(!this.CloseActivity()){
					this.SetActivity();
					return false;
				}
			}
			this.InitializeActivity(activity);
			this.SetActivity();
			return true;
		}
		
		/// <summary>
		/// Pop the top of the activity stack, and set the next activity, uses "Home" if stack is empty
		/// </summary>
		/// <returns>
		/// True, if current activity could be unloaded
		/// </returns>
		public bool PopActivity(){
			if(this.CloseActivity()){
				if(this.activities.Count == 0)
					this.InitializeActivity("Home");
				this.SetActivity();
				return true;
			}
			return false;
		}
		
		/// <summary>
		/// Tries to load the home activity
		/// </summary>
		/// <returns>
		/// True, if Home was loaded
		/// </returns>
		public bool HomeActivity(){
			return this.LoadActivity("Home");
		}
	
		
		public event EventHandler<PersonEventArgs> CurrentUserChanged;
	    public event EventHandler<ActivityUnloadedArgs> ActivityUnloading;
	
	    /// <summary>
	    /// The currently selected user, null if None or Anonymous
	    /// </summary>
	    public Person CurrentUser
	    {
	        get{
	            TreeIter iter;
	            if(!this.userCombobox.GetActiveIter(out iter))
					return null;
	            return (Person)this.userCombobox.Model.GetValue(iter, 1);
	        }
	    }
	
		/// <summary>
		/// Get the main window from IOwner 
		/// </summary>
		public Gtk.Window Window{
			get{
				return this;
			}
		}
		
		#endregion
		
		#region Shopping shoppingList
		
		private IActivity shoppingList;
		private bool shoppingListCollapsed = false;
		
		private void LoadShoppingList(){
			shoppingList = (IActivity)this.plugins["Shopping shoppingList"].Invoke(new object[]{});
			shoppingList.Register(this);
			this.ShoppingListBox.Add(shoppingList.Widget);
			this.shoppingList.Widget.Show();
			ShowHideShoppingList();
		}
		
			
		public void ToggleShoppingList(bool state)
		{
			this.shoppingListCollapsed = state;
			ShowHideShoppingList();
		}
	
		protected virtual void OnBtnShowHideShoppingListClicked (object sender, System.EventArgs e)
		{
			ShowHideShoppingList();
		}
		
		private void ShowHideShoppingList(){
			this.ShoppingListHandleTable.Remove(this.ShoppingListLabel);
			if(this.shoppingListCollapsed){
				//Show shoppinglist
				this.ShoppingListLabel.Angle = 0.0;
				this.ShoppingListHandleTable.Attach(this.ShoppingListLabel, 1, 2, 0, 1);
				this.ShoppingListBox.Show();
				this.ShoppingListArrow.ArrowType = ArrowType.Down;
			}else{
				//Hide shoppinglist
				this.ShoppingListLabel.Angle = 270;
				this.ShoppingListHandleTable.Attach(this.ShoppingListLabel, 0, 1, 1, 2);
				this.ShoppingListBox.Hide();
				this.ShoppingListArrow.ArrowType = ArrowType.Left;
			}
			this.shoppingListCollapsed = !this.shoppingListCollapsed;
		}
	
	
		#endregion
	}
}