using System;
using NUnit.Framework;
using Foodolini.BusinessLogic;
using Foodolini.Database;
using System.Collections.Generic;

namespace Foodolini.BusinessLogic.Test
{
	/// <summary>
	/// Unit test to the Model layer of FoodItem
	/// </summary>
	[TestFixture]
	public class FoodItemTest : LogicTest
	{
		/// <summary>
		/// Try to create a FoodItem
		/// </summary>
		[Test]
		public void CreateFoodItemTest (){
			FoodItem food = new FoodItem(4,new Ingredient("milk") ,new DateTime(11155),false) ;
			food.Save ();
			Assert.AreNotEqual (0,food.Id,"Fooditem have been given a ID");
		}

		/// <summary>
		/// Test that a FoodItem can be saved and loaded by saving a new FoodItem and then load it again to check 
		/// if the loaded data match.
		/// </summary>
		[Test]
		public void LoadCreatedFoodItemTest (){
			
			FoodItem food = new FoodItem(4,new Ingredient("milk"),new DateTime(11155),false);
			food.Save ();
			Assert.AreNotEqual (0,food.Id,"Fooditem have been given a ID");
			
			foreach (var f in FoodItem.ListFoodItems()) {
				if (f.Ingredient.CommercialName  == "milk"){
					Assert.AreEqual (new DateTime(11155) ,f.ExpirationDate,"fooditem loaded");
					Assert.AreEqual (false  ,f.IsOpen ,"fooditem isopen loaded");
				}
			}
			List<FoodItem> count = new List<FoodItem>(FoodItem.ListFoodItems());
			Assert.AreEqual (1, count.Count ,"foodcount in database match");
		
		}
		/// <summary>
		/// test that the fooditem is consumed and moved to person consumed list
		/// and test that the fooditem is opend when consumed.
		/// </summary>
		/// <remarks>
		/// Fixed that the FoodItem should be opened when consumed.
		/// Fixed bug that ListPerson did not work.
		/// Fixed SQLite syntax error in ListFoodItem
		/// </remarks>
		[Test]
		public void ConsumeFoodItemTest (){
			Ingredient ing = new Ingredient("milk");
			ing.ExpirationAfterOpening = new TimeSpan (1,0,0,0);
			ing.ShelfLife = new TimeSpan(3,0,0,0);	
			FoodItem food = new FoodItem(4,ing,new DateTime(11155),false);
			food.Save ();
			Assert.AreEqual(false ,food.IsOpen ,"isopen new saved"); 
			List<FoodItem> count = new List<FoodItem>(FoodItem.ListFoodItems());
			Assert.AreEqual (1, count.Count ,"foodItem in database count");
			
			foreach (var f in FoodItem.ListFoodItems()) {
				if (f.Ingredient.CommercialName  == "milk"){
					Assert.AreEqual(new DateTime(11155), f.ExpirationDate, "fooditem loaded");
					Assert.AreEqual(null,f.ConsumedBy, "name match" );
					Assert.AreEqual(false, f.IsConsumed, "is consummed");
					Assert.AreEqual(false, f.IsOpen ,"isopen");
				}
			}
			
			
			foreach (var per in Person.ListUsers() ){
				if (per.UserName == "jopsen"){
					food.Consume (per);
					Assert.AreEqual(true ,food.IsOpen ,"isopen after consumed"); 
				}
			}
			food.Save ();
			Assert.AreEqual(true ,food.IsOpen,"isopen after saved" );
			
			Assert.AreNotEqual (0,food.Id,"Fooditem have been given a ID");
			
			foreach (var per in Person.ListUsers()) {
				if (per.UserName == "jopsen"){
					foreach (var f in per.GetConsumedFoodItems()){
						if (f.Ingredient.CommercialName  == "milk"){
							Assert.AreEqual(new DateTime(11155), f.ExpirationDate, "fooditem loaded");
							Assert.AreEqual("jopsen",f.ConsumedBy.UserName, "name match" );
							Assert.AreEqual(true ,f.IsConsumed, "is consummed");
							Assert.AreEqual(true,f.IsOpen ,"isopen");
							Assert.AreEqual(System.DateTime.Now.Day ,f.ConsumedDate.Day ,"consummeddate match");
						}	
					} 	
				}
			}
					
			count = new List<FoodItem>(FoodItem.ListFoodItems());
			Assert.AreEqual (0, count.Count ,"foodItem in database count"); //note consumed item do not list with listfooditems
		
		}
		
		
		/// <summary>
		/// test that a FoodItem can be partial consumed by creatin a new FoodItem and partial consume it
		/// on a Person, and then test if all the remaining FoodItems are correct and test if 
		/// the consumed FoodItem have been asisiated with the Person.
		/// </summary>
		[Test]
		public void ConsumePartialFoodItemTest (){
			Ingredient ing = new Ingredient("milk");
			ing.ExpirationAfterOpening = new TimeSpan (1,0,0,0);
			ing.ShelfLife = new TimeSpan(3,0,0,0);	
			FoodItem food = new FoodItem(4,ing,new DateTime(11155),false);
			food.Save ();
			Assert.AreEqual(false ,food.IsOpen ,"isopen new saved"); 
			List<FoodItem> count = new List<FoodItem>(FoodItem.ListFoodItems());
			Assert.AreEqual (1, count.Count ,"foodItem in database count");
			
			
			
			foreach (var per in Person.ListUsers() ){
				if (per.UserName == "jopsen"){
					food.Consume (per,1);
					Assert.AreEqual(true ,food.IsOpen ,"isopen after consumed"); 
				}
			}
			food.Save ();
			
			foreach (var per in Person.ListUsers()) {
				if (per.UserName == "jopsen"){
					foreach (var f in per.GetConsumedFoodItems()){
						if (f.Ingredient.CommercialName  == "milk"){
							Assert.AreEqual(new DateTime(11155), f.ExpirationDate, "fooditem loaded");
							Assert.AreEqual("jopsen",f.ConsumedBy.UserName, "name match" );
							Assert.AreEqual(true ,f.IsConsumed, "is consummed");
							Assert.AreEqual(true,f.IsOpen ,"isopen");
							Assert.AreEqual(1,f.Quantity,"quantity dont match" );
							Assert.AreEqual(System.DateTime.Now.Day ,f.ConsumedDate.Day ,"consummeddate match");
						}	
					} 	
				}
			}
					
			count = new List<FoodItem>(FoodItem.ListFoodItems());
			Assert.AreEqual (1, count.Count ,"remaining foodItem in database count"); //note consumed item do not list with listfooditems
				foreach (var f in FoodItem.ListFoodItems()) {
				if (f.Ingredient.CommercialName  == "milk"){
					Assert.AreEqual(new DateTime(11155), f.ExpirationDate, "remaining fooditem not loaded");
					Assert.AreEqual(null,f.ConsumedBy, " remaining fooditem name dont match" );
					Assert.AreEqual(false, f.IsConsumed, "remaining fooditem is consummed");
					Assert.AreEqual(false, f.IsOpen ,"remaining fooditem is open");
					Assert.AreEqual(3,f.Quantity,"remaining fooditem quantity dont match" );
				}
			}
		}
		
		
		
		/// <summary>
		/// Test of whether Split on FoodItem works.
		/// Test that the FoodItem is opened when split.
		/// </summary>
		/// <remarks>
		/// Fixed that FoodItem.Split did not open the two split FoodItems.
		/// </remarks>		
		[Test]
		public void SplitFoodItemTest (){
			FoodItem newfood; 
			Ingredient ing = new Ingredient("milk");
			ing.ExpirationAfterOpening = new TimeSpan (1,0,0,0);
			ing.ShelfLife = new TimeSpan(3,0,0,0);	
			FoodItem food = new FoodItem(10,ing,new DateTime(11155),false);
			food.Save ();
			
			newfood = food.Split(2);
			
			Assert.AreEqual(2, newfood.Quantity, "Quantity of newfood not 2");
			Assert.AreEqual(8, food.Quantity, "Quantity of food not 8");
			Assert.AreEqual(true, newfood.IsOpen, "newfood not open");
			Assert.AreEqual(true, food.IsOpen, "food not open");
			Assert.AreEqual(ing.LongDescription, newfood.Ingredient.LongDescription, "Ingredient for newfood is wrong");
			Assert.AreEqual(new TimeSpan(1,0,0,0).Days, food.ExpirationDate.Day, "ExpirationDate of food not 1 day");
			
			
		}
		/// <summary>
		/// Test the exceptions in SplitFoodItem by atamting to make iligal splits on the FoodItem.
		/// </summary>
		[Test]
		public void SplitFoodItemExceptionTest(){
			FoodItem newfood; 
			Ingredient ing = new Ingredient("milk");
			ing.ExpirationAfterOpening = new TimeSpan (1,0,0,0);
			ing.ShelfLife = new TimeSpan(3,0,0,0);	
			FoodItem food = new FoodItem(10,ing,new DateTime(11155),false);
			food.Save ();
			
			try {
				newfood = food.Split(10);
			} catch (Exception e){
				Assert.AreEqual("Quantity in FoodItem equals quantity for split", e.Message, "Split does not throw exception on equals");
			}

			try {
				newfood = food.Split(14);
			} catch (Exception e){
				Assert.AreEqual("split more than we have", e.Message, "Split does not throw exception on more than we have");
			}
			foreach (var per in Person.ListUsers()) {
				if (per.UserName == "jopsen"){
					food.Consume(per);				}
			}
				
			try {
				newfood = food.Split(8);
			} catch (Exception e){
				Assert.AreEqual("Cannot split consumed FoodItem", e.Message, "Split does not throw exception when splitting consumed FoodItem");
			}	
		}
		/// <summary>
		/// Test that DeleteFoodItem works by adding FoodItems to the dababase and then remove them again and check if they 
		/// have been removed correct. 
		/// </summary>
		[Test]
		public void FoodItemDeleteTest(){
			Ingredient ing = new Ingredient("milk");
			ing.ExpirationAfterOpening = new TimeSpan (1,0,0,0);
			ing.ShelfLife = new TimeSpan(3,0,0,0);	
			FoodItem food = new FoodItem(10,ing,new DateTime(11155),false);
			food.Save ();
			FoodItem food2 = new FoodItem(98,ing,new DateTime(11155),false);
			food2.Save ();
			
			List<FoodItem> count = new List<FoodItem>(FoodItem.ListFoodItems());
			Assert.AreEqual (2, count.Count ,"foodcount in database does not match");
			
			food.Delete();
			
			count = new List<FoodItem>(FoodItem.ListFoodItems());
			Assert.AreEqual (1, count.Count ,"foodcount in database does not match after deletion");
			
			foreach (var f in count) {
				Assert.AreEqual(98, f.Quantity, "Deleted wrong FoodItem");
			}
			
			food.Delete();
			count = new List<FoodItem>(FoodItem.ListFoodItems());
			Assert.AreEqual (1, count.Count ,"foodcount in database does not match after second deletion");
			
			foreach (var f in count) {
				Assert.AreEqual(98, f.Quantity, "Deleted wrong FoodItem after second deletion");
			}
		}
		
		/// <summary>
		/// Test that Open Close FoodItem works by creating a new FoodItem and atempt to open and close it
		/// and then check that the Ecpirations dates are correct.
		/// </summary>
		[Test]
		public void OpenCloseFoodItemDeleteTest(){
			Ingredient ing = new Ingredient("milk");
			ing.ExpirationAfterOpening = new TimeSpan (1,0,0,0);
			ing.ShelfLife = new TimeSpan(3,0,0,0);	
			FoodItem food = new FoodItem(10,ing,new DateTime(11155),false);
			food.Save ();
			
			food.OpenFoodItem();
			
			food.CloseFoodItem();
			DateTime temp = System.DateTime.Now + new TimeSpan(3,0,0,0);
			Assert.AreEqual(temp.Day,food.ExpirationDate.Day, "shelflife dont match"); 
		}
	}
}