﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.Common;

namespace Foodolini.Tools
{
    public abstract class Database<TConnection> where TConnection : DbConnection, new()
    {
        public delegate T RelationalMapperDelegate<T>(IDataReader rdr);
        protected delegate T QueryHandler<T>(DbCommand cmd);

        /// <summary>
        /// Generic DbConnection. Inheriting database classes specify the concrete DbConnection.
        /// </summary>
        protected TConnection connection;

        /// <summary>
        /// Executes an sql statement and returns if any rows were affected.
        /// </summary>
        /// <param name="sql"></param>
        /// <returns></returns>
        public bool ExecuteNonQuery(string sql)
        {
            return OpenDatabase<bool>(GetCommand(sql),
                delegate(DbCommand cmd)
                {
                    return cmd.ExecuteNonQuery() > 0;
                });
        }

        protected abstract DbCommand GetCommand(string sql);

        /// <summary>
        /// Executes a command against the database and closes the connection.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="cmd"></param>
        /// <param name="doQuery"></param>
        /// <returns></returns>
        T OpenDatabase<T>(DbCommand cmd, QueryHandler<T> doQuery)
        {
            return OpenDatabase<T>(cmd, doQuery, true);
        }

        public abstract TConnection GetConnection();

        T OpenDatabase<T>(DbCommand cmd, QueryHandler<T> doQuery, bool closeConnection)
        {
            T result = default(T);
            if (connection.State == ConnectionState.Closed)
                connection.Open();
            result = doQuery(cmd);
            if (closeConnection)
                connection.Close();
            return result;
        }

        /// <summary>
        /// Returns a generic type from an sql statement.
        /// </summary>
        /// <typeparam name="T">A type with an empty default constructor</typeparam>
        /// <param name="sql">SQL statement</param>
        /// <param name="map">Mapping function</param>
        /// <returns></returns>
        public T ExecuteReader<T>(string sql, RelationalMapperDelegate<T> map) where T : new()
        {
            return OpenDatabase<T>(GetCommand(sql), delegate(DbCommand cmd)
            {
                using (IDataReader rdr = cmd.ExecuteReader())
                    return map(rdr);
            });
        }

        public void CloseDatabase()
        {
            connection.Close();
        }

        /// <summary>
        /// Returns an IDataReader. Remember to close it after use.
        /// </summary>
        /// <param name="sql"></param>
        /// <returns></returns>
        public IDataReader GetReader(string sql)
        {
            return GetReader(sql, true);
        }

        /// <summary>
        /// Returns an IDataReader. Remember to close it after use.
        /// </summary>
        /// <param name="sql"></param>
        /// <returns></returns>
        public IDataReader GetReader(string sql, bool closeConnection)
        {
            return OpenDatabase<IDataReader>(GetCommand(sql), delegate(DbCommand cmd)
            {
                return cmd.ExecuteReader();
            }, closeConnection);
        }

        /// <summary>
        /// Executes a scalar sql statement.
        /// </summary>
        /// <param name="sql"></param>
        /// <returns></returns>
        public object ExecuteScalar(string sql)
        {
            return OpenDatabase<object>(GetCommand(sql),
                delegate(DbCommand cmd)
                {
                    return cmd.ExecuteScalar();
                });
        }
    }
}
