﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;
using System.Data;
using System.Data.SQLite;
using System.Configuration;
using System.Data.Common;
using System.Diagnostics;

namespace Foodolini.Tools
{
    public class SQLToSQLiteDataMigrater
    {
        private SQLiteDB sqliteDb;
        private MsSqlDB msSqlDb;

        public event EventHandler<MigraterEventArgs> RaiseMigraterEvent;

        public SQLToSQLiteDataMigrater(string sqliteConnectionString, string msSqlConnectionString)
        {
            sqliteDb = new SQLiteDB(sqliteConnectionString);
            msSqlDb = new MsSqlDB(msSqlConnectionString);
        }

        protected void OnRaiseMigraterEvent(MigraterEventArgs e)
        {
            EventHandler<MigraterEventArgs> handler = RaiseMigraterEvent;
            if (handler != null)
                handler(this, e);
        }

        public void CopyData()
        {
            // Clear existing tables before import

            OnRaiseMigraterEvent(new MigraterEventArgs("Truncating nutrition data...", MigraterEventArgs.EventColor.Red));
            sqliteDb.ExecuteNonQuery("delete from nutritionalvalues");

            OnRaiseMigraterEvent(new MigraterEventArgs("Truncating food groups...", MigraterEventArgs.EventColor.Red));
            sqliteDb.ExecuteNonQuery("delete from foodgroups");

            OnRaiseMigraterEvent(new MigraterEventArgs("Truncating food descriptions...", MigraterEventArgs.EventColor.Red));
            sqliteDb.ExecuteNonQuery("delete from fooddescriptions");

            OnRaiseMigraterEvent(new MigraterEventArgs("Truncating nutrition definitions...", MigraterEventArgs.EventColor.Red));
            sqliteDb.ExecuteNonQuery("delete from nutritiondefinitions");

            // insert statements
            string foodDescription = string.Format(@"insert into fooddescriptions (fooddescriptionid,foodgroupid,longdescription,shortdescription,commercialname,manufacturer,protein,carbohydrates,fat) values ({0});", AddParamFormat(9));
            string foodGroups = string.Format("insert into foodgroups (foodgroupid,description) values ({0});", AddParamFormat(2));
            string nutritionData = string.Format(@"insert into nutritionalvalues (fooddescriptionid,nutritiondefinitionid,[value]) values ({0});", AddParamFormat(3));
            string nutritionDefinitions = string.Format("insert into nutritiondefinitions (nutritiondefinitionid,unit,tagname,description,sortorder) values ({0});", AddParamFormat(5));

            // perform import
            OnRaiseMigraterEvent(new MigraterEventArgs("Inserting food descriptions...", MigraterEventArgs.EventColor.Yellow));
            PerformImport(msSqlDb.GetReader(@"select food_des.[NDB_No],[FdGrp_Cd],[Long_Desc],food_des.[Shrt_Desc],[ComName],[ManufacName],[Protein],[Carbohydrt],[Lipid_Tot] from 
food_des inner join abbrev on abbrev.ndb_no = food_des.ndb_no", false), foodDescription);

            OnRaiseMigraterEvent(new MigraterEventArgs("Inserting food groups...", MigraterEventArgs.EventColor.Yellow));
            PerformImport(msSqlDb.GetReader("select fdgrp_cd,fdgrp_desc from fd_group", false), foodGroups);

            OnRaiseMigraterEvent(new MigraterEventArgs("Inserting nutritional values...", MigraterEventArgs.EventColor.Yellow));
            PerformImport(msSqlDb.GetReader("select [NDB_No],[Nutr_No],[Nutr_Val] from nut_data", false), nutritionData);

            OnRaiseMigraterEvent(new MigraterEventArgs("Inserting nutrition definitions...", MigraterEventArgs.EventColor.Yellow));
            PerformImport(msSqlDb.GetReader("select nutr_no,units,tagname,nutrdesc,sr_order from nutr_def", false), nutritionDefinitions);

            msSqlDb.CloseDatabase();

            OnRaiseMigraterEvent(new MigraterEventArgs("Vacuuming SQLite database...", MigraterEventArgs.EventColor.Yellow));
            sqliteDb.ExecuteNonQuery("VACUUM;");
            OnRaiseMigraterEvent(new MigraterEventArgs("Migration complete!", MigraterEventArgs.EventColor.Green));
        }

        string AddParamFormat(int parameterCount)
        {
            StringBuilder parameters = new StringBuilder();
            for (int i = 0; i < parameterCount; i++) {
                parameters.Append("@" + i);
                if (i < parameterCount - 1)
                    parameters.Append(",");
            }
            return parameters.ToString();
        }

        void PerformImport(IDataReader rdr, string sql)
        {
            int record = 0;
            using (SQLiteConnection con = new SQLiteConnection(ConfigurationManager.ConnectionStrings["FoodoliniSQLite"].ConnectionString)) {
                con.Open();
                using (SQLiteTransaction transaction = con.BeginTransaction()) {
                    SQLiteCommand cmd = new SQLiteCommand(sql, con);
                    while (rdr.Read()) {
                        cmd.Parameters.Clear();
                        int columns = rdr.FieldCount;
                        for (int i = 0; i < columns; i++) {
                            object val = rdr.GetValue(i);
                            cmd.Parameters.AddWithValue("@" + i, val);
                        }
                        record++;
                        //if (record % 1000 == 0) OnRaiseMigraterEvent(new MigraterEventArgs(""+record));
                        cmd.ExecuteNonQuery();
                    }
                    transaction.Commit();
                }
                rdr.Close();
            }
            OnRaiseMigraterEvent(new MigraterEventArgs("    Done! ("+record+" records inserted)", MigraterEventArgs.EventColor.Grey));
        }
    }

    public class MigraterEventArgs : EventArgs
    {
        public string Message { get; set; }
        public EventColor Color { get; set; }
        public MigraterEventArgs(string s)
        {
            this.Message = s;
        }
        public MigraterEventArgs(string s, EventColor color)
        {
            this.Message = s;
            this.Color = color;
        }

        public enum EventColor
        {
            Grey,
            Red,
            Yellow,
            Green
        }
    }
}